// teamAI.cpp

#include "CvGameCoreDLL.h"
#include "CvTeamAI.h"
#include "CvPlayerAI.h"
#include "CvRandom.h"
#include "CvGlobals.h"
#include "CvGameCoreUtils.h"
#include "CvMap.h"
#include "CvPlot.h"
#include "CvDLLInterfaceIFaceBase.h"
#include "CvGameAI.h"
#include "CvInfos.h"
#include "FProfiler.h"
#include "CyArgsList.h"
#include "CvDLLPythonIFaceBase.h"

/************************************************************************************************/
/* BETTER_BTS_AI_MOD                      10/02/09                                jdog5000      */
/*                                                                                              */
/* AI logging                                                                                   */
/************************************************************************************************/
#include "BetterBTSAI.h"
/************************************************************************************************/
/* BETTER_BTS_AI_MOD                       END                                                  */
/************************************************************************************************/

// statics

CvTeamAI* CvTeamAI::m_aTeams = NULL;

void CvTeamAI::initStatics()
{
	m_aTeams = new CvTeamAI[MAX_TEAMS];
	for (int iI = 0; iI < MAX_PLAYERS; iI++)
	{
		m_aTeams[iI].m_eID = ((TeamTypes)iI);
	}
}

void CvTeamAI::freeStatics()
{
	SAFE_DELETE_ARRAY(m_aTeams);
}

// inlined for performance reasons
DllExport CvTeamAI& CvTeamAI::getTeamNonInl(TeamTypes eTeam)
{
	return getTeam(eTeam);
}


// Public Functions...

CvTeamAI::CvTeamAI()
{
	m_aiWarPlanStateCounter = new int[MAX_TEAMS];
	m_aiAtWarCounter = new int[MAX_TEAMS];
	m_aiAtPeaceCounter = new int[MAX_TEAMS];
	m_aiHasMetCounter = new int[MAX_TEAMS];
	m_aiOpenBordersCounter = new int[MAX_TEAMS];
	m_aiDefensivePactCounter = new int[MAX_TEAMS];
	m_aiShareWarCounter = new int[MAX_TEAMS];
	m_aiWarSuccess = new int[MAX_TEAMS];
	m_aiEnemyPeacetimeTradeValue = new int[MAX_TEAMS];
	m_aiEnemyPeacetimeGrantValue = new int[MAX_TEAMS];
	m_aeWarPlan = new WarPlanTypes[MAX_TEAMS];


	AI_reset(true);
}


CvTeamAI::~CvTeamAI()
{
	AI_uninit();

	SAFE_DELETE_ARRAY(m_aiWarPlanStateCounter);
	SAFE_DELETE_ARRAY(m_aiAtWarCounter);
	SAFE_DELETE_ARRAY(m_aiAtPeaceCounter);
	SAFE_DELETE_ARRAY(m_aiHasMetCounter);
	SAFE_DELETE_ARRAY(m_aiOpenBordersCounter);
	SAFE_DELETE_ARRAY(m_aiDefensivePactCounter);
	SAFE_DELETE_ARRAY(m_aiShareWarCounter);
	SAFE_DELETE_ARRAY(m_aiWarSuccess);
	SAFE_DELETE_ARRAY(m_aiEnemyPeacetimeTradeValue);
	SAFE_DELETE_ARRAY(m_aiEnemyPeacetimeGrantValue);
	SAFE_DELETE_ARRAY(m_aeWarPlan);
}


void CvTeamAI::AI_init()
{
	AI_reset(false);

	//--------------------------------
	// Init other game data
}


void CvTeamAI::AI_uninit()
{
}


void CvTeamAI::AI_reset(bool bConstructor)
{
	AI_uninit();

	m_eWorstEnemy = NO_TEAM;

	for (int iI = 0; iI < MAX_TEAMS; iI++)
	{
		m_aiWarPlanStateCounter[iI] = 0;
		m_aiAtWarCounter[iI] = 0;
		m_aiAtPeaceCounter[iI] = 0;
		m_aiHasMetCounter[iI] = 0;
		m_aiOpenBordersCounter[iI] = 0;
		m_aiDefensivePactCounter[iI] = 0;
		m_aiShareWarCounter[iI] = 0;
		m_aiWarSuccess[iI] = 0;
		m_aiEnemyPeacetimeTradeValue[iI] = 0;
		m_aiEnemyPeacetimeGrantValue[iI] = 0;
		m_aeWarPlan[iI] = NO_WARPLAN;

		if (!bConstructor && getID() != NO_TEAM)
		{
			TeamTypes eLoopTeam = (TeamTypes) iI;
			CvTeamAI& kLoopTeam = GET_TEAM(eLoopTeam);
			kLoopTeam.m_aiWarPlanStateCounter[getID()] = 0;
			kLoopTeam.m_aiAtWarCounter[getID()] = 0;
			kLoopTeam.m_aiAtPeaceCounter[getID()] = 0;
			kLoopTeam.m_aiHasMetCounter[getID()] = 0;
			kLoopTeam.m_aiOpenBordersCounter[getID()] = 0;
			kLoopTeam.m_aiDefensivePactCounter[getID()] = 0;
			kLoopTeam.m_aiShareWarCounter[getID()] = 0;
			kLoopTeam.m_aiWarSuccess[getID()] = 0;
			kLoopTeam.m_aiEnemyPeacetimeTradeValue[getID()] = 0;
			kLoopTeam.m_aiEnemyPeacetimeGrantValue[getID()] = 0;
			kLoopTeam.m_aeWarPlan[getID()] = NO_WARPLAN;
		}
	}
}


void CvTeamAI::AI_doTurnPre()
{
	AI_doCounter();

	if (isHuman())
	{
		return;
	}

	if (isBarbarian())
	{
		return;
	}

	if (isMinorCiv())
	{
		return;
	}
}


void CvTeamAI::AI_doTurnPost()
{
	AI_updateWorstEnemy();

	AI_updateAreaStragies(false);

	if (isHuman())
	{
		return;
	}

	if (isBarbarian())
	{
		return;
	}

	if (isMinorCiv())
	{
		return;
	}

	AI_doWar();
}


void CvTeamAI::AI_makeAssignWorkDirty()
{
	int iI;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				GET_PLAYER((PlayerTypes)iI).AI_makeAssignWorkDirty();
			}
		}
	}
}

/********************************************************************************/
/* 	BETTER_BTS_AI_MOD						10/6/08				jdog5000	*/
/* 																			*/
/* 	General AI																*/
/********************************************************************************/
// Find plot strength of teammates and potentially vassals
int CvTeamAI::AI_getOurPlotStrength(CvPlot* pPlot, int iRange, bool bDefensiveBonuses, bool bTestMoves, bool bIncludeVassals)
{
	int iI;
	int iPlotStrength = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID() || (bIncludeVassals && GET_TEAM(GET_PLAYER((PlayerTypes)iI).getTeam()).isVassal(getID())) )
			{
				iPlotStrength += GET_PLAYER((PlayerTypes)iI).AI_getOurPlotStrength(pPlot,iRange,bDefensiveBonuses,bTestMoves);
			}
		}
	}

	return iPlotStrength;
}
/********************************************************************************/
/* 	BETTER_BTS_AI_MOD						END								*/
/********************************************************************************/


void CvTeamAI::AI_updateAreaStragies(bool bTargets)
{
	CvArea* pLoopArea;
	int iLoop;

	if (!(GC.getGameINLINE().isFinalInitialized()))
	{
		return;
	}

	for(pLoopArea = GC.getMapINLINE().firstArea(&iLoop); pLoopArea != NULL; pLoopArea = GC.getMapINLINE().nextArea(&iLoop))
	{
		pLoopArea->setAreaAIType(getID(), AI_calculateAreaAIType(pLoopArea));
	}

	if (bTargets)
	{
		AI_updateAreaTargets();
	}
}


void CvTeamAI::AI_updateAreaTargets()
{
	int iI;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				GET_PLAYER((PlayerTypes)iI).AI_updateAreaTargets();
			}
		}
	}
}


int CvTeamAI::AI_countFinancialTrouble() const
{
	int iCount;
	int iI;

	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				if (GET_PLAYER((PlayerTypes)iI).AI_isFinancialTrouble())
				{
					iCount++;
				}
			}
		}
	}

	return iCount;
}


int CvTeamAI::AI_countMilitaryWeight(CvArea* pArea) const
{
	int iCount;
	int iI;

	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iCount += GET_PLAYER((PlayerTypes)iI).AI_militaryWeight(pArea);
			}
		}
	}

	return iCount;
}


bool CvTeamAI::AI_isAnyCapitalAreaAlone() const
{
	int iI;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				if (GET_PLAYER((PlayerTypes)iI).AI_isCapitalAreaAlone())
				{
					return true;
				}
			}
		}
	}

	return false;
}


bool CvTeamAI::AI_isPrimaryArea(CvArea* pArea) const
{
	int iI;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				if (GET_PLAYER((PlayerTypes)iI).AI_isPrimaryArea(pArea))
				{
					return true;
				}
			}
		}
	}

	return false;
}


bool CvTeamAI::AI_hasCitiesInPrimaryArea(TeamTypes eTeam) const
{
	CvArea* pLoopArea;
	int iLoop;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	for(pLoopArea = GC.getMapINLINE().firstArea(&iLoop); pLoopArea != NULL; pLoopArea = GC.getMapINLINE().nextArea(&iLoop))
	{
		if (AI_isPrimaryArea(pLoopArea))
		{
			if (GET_TEAM(eTeam).countNumCitiesByArea(pLoopArea))
			{
				return true;
			}
		}
	}

	return false;
}

AreaAITypes CvTeamAI::AI_calculateAreaAIType(CvArea* pArea, bool bPreparingTotal) const
{
	PROFILE_FUNC();

	bool bRecentAttack;
	bool bTargets;
	bool bChosenTargets;
	bool bDeclaredTargets;
	int iOffensiveThreshold;
	int iAreaCities;
	int iI;

	if (!(pArea->isWater()))
	{
		if (isBarbarian())
		{
			if ((pArea->getNumCities() - pArea->getCitiesPerPlayer(BARBARIAN_PLAYER)) == 0)
			{
				return AREAAI_ASSAULT;
			}

			if ((countNumAIUnitsByArea(pArea, UNITAI_ATTACK) + countNumAIUnitsByArea(pArea, UNITAI_ATTACK_CITY) + countNumAIUnitsByArea(pArea, UNITAI_PILLAGE) + countNumAIUnitsByArea(pArea, UNITAI_ATTACK_AIR)) > (((AI_countMilitaryWeight(pArea) * 20) / 100) + 1))
			{
				return AREAAI_OFFENSIVE; // XXX does this ever happen?
			}

			return AREAAI_MASSING;
		}

		bRecentAttack = false;
		bTargets = false;
		bChosenTargets = false;
		bDeclaredTargets = false;

		bool bAssault = false;
		bool bPreparingAssault = false;

		if (bPreparingTotal)
		{
			iOffensiveThreshold = 25;
		}
		else
		{
			iOffensiveThreshold = 20;
		}

		iAreaCities = countNumCitiesByArea(pArea);

		for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
		{
			if (GET_TEAM((TeamTypes)iI).isAlive())
			{
				/********************************************************************************/
				/**		REVOLUTION_MOD							6/23/08				jdog5000	*/
				/**																				*/
				/**		Revolution AI, minor civs												*/
				/********************************************************************************/
				// Minor civs will declare war on all, but won't get has met set so that they don't show in score board
				//if (AI_getWarPlan((TeamTypes)iI) != NO_WARPLAN)
				if (AI_getWarPlan((TeamTypes)iI) != NO_WARPLAN && isHasMet((TeamTypes)iI))
				{
					/********************************************************************************/
					/**		REVOLUTION_MOD							END								*/
					/********************************************************************************/
					FAssert(((TeamTypes)iI) != getID());
					FAssert(isHasMet((TeamTypes)iI) || GC.getGameINLINE().isOption(GAMEOPTION_ALWAYS_WAR));

					if (AI_getWarPlan((TeamTypes)iI) == WARPLAN_ATTACKED_RECENT)
					{
						FAssert(isAtWar((TeamTypes)iI));
						bRecentAttack = true;
					}

					if ((GET_TEAM((TeamTypes)iI).countNumCitiesByArea(pArea) > 0) || (GET_TEAM((TeamTypes)iI).countNumUnitsByArea(pArea) > 4))
					{
						bTargets = true;

						if (AI_isChosenWar((TeamTypes)iI))
						{
							bChosenTargets = true;

							if ((isAtWar((TeamTypes)iI)) ? (AI_getAtWarCounter((TeamTypes)iI) < 10) : AI_isSneakAttackReady((TeamTypes)iI))
							{
								bDeclaredTargets = true;
							}
						}
					}
					else
					{
						bAssault = true;
						if (AI_isSneakAttackPreparing((TeamTypes)iI))
						{
							bPreparingAssault = true;
						}
					}
				}
			}
		}

		/************************************************************************************************/
		/* BETTER_BTS_AI_MOD                         01/12/10                             jdog5000      */
		/*                                                                                              */
		/* War strategy AI                                                                              */
		/************************************************************************************************/
		if( bTargets )
		{
			if(iAreaCities > 0 && getAtWarCount(true) > 0) 
			{
				int iPower = countPowerByArea(pArea);
				int iEnemyPower = countEnemyPowerByArea(pArea);

				if( AI_isPrimaryArea(pArea) && AI_getWarSuccessCapitulationRatio() < -50 )
				{
					// Use defensive stance if enemy has strong presence, we've been losing badly
					if( iPower < iEnemyPower )
					{
						return AREAAI_DEFENSIVE;
					}
				}
				else
				{
					// Use defensive area AI right after invasions, to force vulnerable captured cities to be better defended
					if( 2*iPower < iEnemyPower )
					{
						return AREAAI_DEFENSIVE;
					}
				}
			}
		}

		if (bDeclaredTargets)
		{
			return AREAAI_OFFENSIVE;
		}

		if (bTargets)
		{
			// AI_countMilitaryWeight is based on this team's pop and cities ... if this team is the biggest, it will over estimate needed units
			int iMilitaryWeight = AI_countMilitaryWeight(pArea);
			int iCount = 1;

			for( int iJ = 0; iJ < MAX_CIV_TEAMS; iJ++ )
			{
				if( iJ != getID() && GET_TEAM((TeamTypes)iJ).isAlive() )
				{
					if( !(GET_TEAM((TeamTypes)iJ).isBarbarian() || GET_TEAM((TeamTypes)iJ).isMinorCiv()) )
					{
						if( AI_getWarPlan((TeamTypes)iJ) != NO_WARPLAN )
						{
							iMilitaryWeight += GET_TEAM((TeamTypes)iJ).AI_countMilitaryWeight(pArea);
							iCount++;

							if( GET_TEAM((TeamTypes)iJ).isAVassal() )
							{
								for( int iK = 0; iK < MAX_CIV_TEAMS; iK++ )
								{
									if( iK != getID() && GET_TEAM((TeamTypes)iK).isAlive() )
									{
										if( GET_TEAM((TeamTypes)iJ).isVassal((TeamTypes)iK) )
										{
											iMilitaryWeight += GET_TEAM((TeamTypes)iK).AI_countMilitaryWeight(pArea);
										}
									}
								}
							}
						}
					}
				}
			}

			iMilitaryWeight /= iCount;

			if ((countNumAIUnitsByArea(pArea, UNITAI_ATTACK) + countNumAIUnitsByArea(pArea, UNITAI_ATTACK_CITY) + countNumAIUnitsByArea(pArea, UNITAI_PILLAGE) + countNumAIUnitsByArea(pArea, UNITAI_ATTACK_AIR)) > (((iMilitaryWeight * iOffensiveThreshold) / 100) + 1))
			{
				return AREAAI_OFFENSIVE;
			}
		}
		/************************************************************************************************/
		/* BETTER_BTS_AI_MOD                       END                                                  */
		/************************************************************************************************/

		if (bTargets)
		{
			for (int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++)
			{
				/************************************************************************************************/
				/* UNOFFICIAL_PATCH                       09/17/09                      cephalo & jdog5000      */
				/*                                                                                              */
				/* Bugfix                                                                                       */
				/************************************************************************************************/
				/* original BTS code
				CvPlayerAI& kPlayer = GET_PLAYER((PlayerTypes)iI);
				*/
				CvPlayerAI& kPlayer = GET_PLAYER((PlayerTypes)iPlayer);
				/************************************************************************************************/
				/* UNOFFICIAL_PATCH                        END                                                  */
				/************************************************************************************************/

				if (kPlayer.isAlive())
				{
					if (kPlayer.getTeam() == getID())
					{
						if (kPlayer.AI_isDoStrategy(AI_STRATEGY_DAGGER) || kPlayer.AI_isDoStrategy(AI_STRATEGY_FINAL_WAR))
						{
							if (pArea->getCitiesPerPlayer((PlayerTypes)iPlayer) > 0)
							{
								return AREAAI_MASSING;
							}
						}
					}
				}
			}
			if (bRecentAttack)
			{
				int iPower = countPowerByArea(pArea);
				int iEnemyPower = countEnemyPowerByArea(pArea);
				if (iPower > iEnemyPower)
				{
					return AREAAI_MASSING;
				}
				return AREAAI_DEFENSIVE;
			}
		}

		if (iAreaCities > 0)
		{
			if (countEnemyDangerByArea(pArea) > iAreaCities)
			{
				return AREAAI_DEFENSIVE;
			}
		}

		if (bChosenTargets)
		{
			return AREAAI_MASSING;
		}

		if (bTargets)
		{
			if (iAreaCities > (getNumMembers() * 3))
			{
				if (GC.getGameINLINE().isOption(GAMEOPTION_AGGRESSIVE_AI) || GC.getGameINLINE().isOption(GAMEOPTION_ALWAYS_WAR) || (countPowerByArea(pArea) > ((countEnemyPowerByArea(pArea) * 3) / 2)))
				{
					return AREAAI_MASSING;
				}
			}
			return AREAAI_DEFENSIVE;
		}
		else
		{
			if (bAssault)
			{
				if (AI_isPrimaryArea(pArea))
				{
					if (bPreparingAssault)
					{
						return AREAAI_ASSAULT_MASSING;
					}
				}
				else if (countNumCitiesByArea(pArea) > 0)
				{
					return AREAAI_ASSAULT_ASSIST;
				}

				return AREAAI_ASSAULT;
			}
		}
	}

	return AREAAI_NEUTRAL;
}

int CvTeamAI::AI_calculateAdjacentLandPlots(TeamTypes eTeam) const
{
	PROFILE_FUNC();

	CvPlot* pLoopPlot;
	int iCount;
	int iI;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	iCount = 0;

	for (iI = 0; iI < GC.getMapINLINE().numPlotsINLINE(); iI++)
	{
		pLoopPlot = GC.getMapINLINE().plotByIndexINLINE(iI);

		if (!(pLoopPlot->isWater()))
		{
			if ((pLoopPlot->getTeam() == eTeam) && pLoopPlot->isAdjacentTeam(getID(), true))
			{
				iCount++;
			}
		}
	}

	return iCount;
}


int CvTeamAI::AI_calculatePlotWarValue(TeamTypes eTeam) const
{
	FAssert(eTeam != getID());

	int iValue = 0;
	bool bAssault = !AI_hasCitiesInPrimaryArea(eTeam);
	bool bFastMovers = false;
	for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
	{
		if(  GET_PLAYER((PlayerTypes)iPlayer).isAlive() && GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
		{
			if (GET_PLAYER((PlayerTypes)iPlayer).AI_isDoStrategy(AI_STRATEGY_FASTMOVERS))
			{
				bFastMovers = true;
				break;
			}
		}
	}

	for (int iI = 0; iI < GC.getMapINLINE().numPlotsINLINE(); iI++)
	{
		CvPlot* pLoopPlot = GC.getMapINLINE().plotByIndexINLINE(iI);

		if (pLoopPlot->getTeam() == eTeam)
		{
			//CD Tweaks
			/* old code
			if (!pLoopPlot->isWater() && pLoopPlot->isAdjacentTeam(getID(), true))
			{
			iValue += 4;
			}*/
			if (!pLoopPlot->isWater() &&!pLoopPlot->isImpassable())
			{
				if(pLoopPlot->isAdjacentTeam(getID(), true))
				{
					iValue += 4;
					if (pLoopPlot->isCity())	//extra value for being able to walk right into city
					{
						iValue += 8;
					}
					else if (pLoopPlot->isCultureRangeCity(pLoopPlot->getOwnerINLINE(), 1))	//extra value for being one tile away from a city
					{
						iValue += 4 + std::min(3,(pLoopPlot->defenseModifier(getID(), true) / 25));
					}
					else if (pLoopPlot->isCultureRangeCity(pLoopPlot->getOwnerINLINE(), 2))	//two tiles away is good two, if we can reach the city more than one way
					{
						int iTemp = 0;
						for (int iJ = 0; iJ < NUM_DIRECTION_TYPES; ++iJ)
						{
							CvPlot* pAdjacentPlot = plotDirection(pLoopPlot->getX_INLINE(), pLoopPlot->getY_INLINE(), ((DirectionTypes)iJ));
							if (!pAdjacentPlot->isWater() && !pAdjacentPlot->isImpassable() && pAdjacentPlot->isCultureRangeCity(pLoopPlot->getOwnerINLINE(), 1))
							{
								iTemp++;
							}
							if (iTemp >= 4)
							{
								break;
							}
						}
						iValue += std::max(0, iTemp-1);
					}
				}
				//find good coastal landing sites - for now this is just for close cities
				else if (bAssault && pLoopPlot->isCoastalLand() && !AI_isPrimaryArea(pLoopPlot->area()))
				{
					bool bClose = false;
					CvCity* pCity = pLoopPlot->getPlotCity();
					if (pCity == NULL)
					{
						pCity = pLoopPlot->getWorkingCity();
					}
					if( pCity != NULL )
					{
						for( int iJ = 0; iJ < MAX_CIV_PLAYERS; iJ++ )
						{
							if( getID() == GET_PLAYER((PlayerTypes)iJ).getTeam() && GET_PLAYER((PlayerTypes)iJ).isAlive() )
							{
								if( pCity->AI_playerCloseness((PlayerTypes)iJ ) > 0 )
								{
									bClose = true;
									break;
								}
							}
						}
					}
					if (bClose)
					{
						if (pLoopPlot->isCity())
						{
							iValue += 4;
						}
						else if (pLoopPlot->isCultureRangeCity(pLoopPlot->getOwnerINLINE(), 1))
						{
							iValue += pLoopPlot->isHills() ? 2 : 1;
						}
					}
				}
				//get back stolen plots by force
				if (pLoopPlot->isWithinTeamCityRadius(getID()))
				{
					iValue += 4;
				}
			}
			//CD Tweaks End
			/************************************************************************************************/
			/* BETTER_BTS_AI_MOD                      06/17/08                                jdog5000      */
			/*                                                                                              */
			/* Notes                                                                                        */
			/************************************************************************************************/
			// This section of code does nothing without XML modding as AIObjective is 0 for all bonuses
			// Left alone for mods to use
			// Resource driven war in BBAI is done with CvTeamAI::AI_calculateBonusWarValue
			// without using the XML variable AIObjective
			/************************************************************************************************/
			/* BETTER_BTS_AI_MOD                       END                                                  */
			/************************************************************************************************/
			BonusTypes eBonus = pLoopPlot->getBonusType(getID());
			if (NO_BONUS != eBonus)
			{
				iValue += 40 * GC.getBonusInfo(eBonus).getAIObjective();
			}
		}
	}

	return iValue;
}

/************************************************************************************************/
/* BETTER_BTS_AI_MOD                      07/21/08                                jdog5000      */
/*                                                                                              */
/* War Strategy AI                                                                              */
/************************************************************************************************/
int CvTeamAI::AI_calculateBonusWarValue(TeamTypes eTeam) const
{
	FAssert(eTeam != getID());

	int iValue = 0;

	for (int iI = 0; iI < GC.getMapINLINE().numPlotsINLINE(); iI++)
	{
		CvPlot* pLoopPlot = GC.getMapINLINE().plotByIndexINLINE(iI);

		if (pLoopPlot->getTeam() == eTeam)
		{
			BonusTypes eNonObsoleteBonus = pLoopPlot->getNonObsoleteBonusType(getID());
			if (NO_BONUS != eNonObsoleteBonus)
			{
				int iThisValue = 0;
				for( int iJ = 0; iJ < MAX_CIV_PLAYERS; iJ++ )
				{
					if( getID() == GET_PLAYER((PlayerTypes)iJ).getTeam() && GET_PLAYER((PlayerTypes)iJ).isAlive() )
					{
						// 10 seems like a typical value for a health/happiness resource the AI doesn't have
						// Values for strategic resources can be 60 or higher
						iThisValue += GET_PLAYER((PlayerTypes)iJ).AI_bonusVal(eNonObsoleteBonus);
					}
				}
				iThisValue /= getAliveCount();

				//CD Tweaks - water plots in close cities can still get a boost
				if (!pLoopPlot->isWater() && pLoopPlot->isAdjacentTeam(getID(), true))
				{
					iThisValue *= 3;
				}
				else
				{
					CvCity* pWorkingCity = pLoopPlot->getWorkingCity();
					if( pWorkingCity != NULL )
					{
						for( int iJ = 0; iJ < MAX_CIV_PLAYERS; iJ++ )
						{
							if( getID() == GET_PLAYER((PlayerTypes)iJ).getTeam() && GET_PLAYER((PlayerTypes)iJ).isAlive() )
							{
								if( pWorkingCity->AI_playerCloseness((PlayerTypes)iJ ) > 0 )
								{
									iThisValue += std::max(0, (iThisValue - (pLoopPlot->isWater() ? 4 : 0)));
									break;
								}
							}
						}
					}
				}
				//CD Tweaks End

				iThisValue = std::max(0, iThisValue - 4);
				iThisValue /= 5;

				iValue += iThisValue;
			}
		}
	}

	//CD Tweaks - boost value for small civs, if we could vassalise them (up to double for very small civs)
	if (!GET_TEAM(eTeam).isAVassal() && isVassalStateTrading() && getPower(false) > GET_TEAM(eTeam).getPower(true))
	{
		int iTheirLandPercent = 100 * GET_TEAM(eTeam).getTotalLand(false) / getTotalLand(false);
		int iTheirPopPercent = 100 * GET_TEAM(eTeam).getTotalPopulation(false) / getTotalPopulation(false);
		if (std::min(iTheirLandPercent, iTheirPopPercent) < 100)
		{
			iValue *= 100 + (100-std::min(iTheirLandPercent, iTheirPopPercent));
			iValue /= 100;
		}
	}
	//CD Tweaks End

	return iValue;
}
/************************************************************************************************/
/* BETTER_BTS_AI_MOD                       END                                                  */
/************************************************************************************************/

//CD Tweaks - since we're counting values for bonuses, we should also count value for cities - wonders etc.
//I'm deliberatley not looking at the city plots, these are counted elsewhere
int CvTeamAI::AI_calculateCityWarValue(TeamTypes eTeam) const
{
	int iValue = 0;
	int iCultureDivisor = std::max(5,(20 * GET_TEAM(eTeam).getPower(false))/getPower(true));
	int iBigCityThreshold = 10 + GC.getGameINLINE().getCurrentEra();
	int iSmallCityThreshold = AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_DOMINATION3) ? 1 : 10;
	for (PlayerTypes eLoopPlayer = (PlayerTypes)0; eLoopPlayer < MAX_PLAYERS; eLoopPlayer=(PlayerTypes)(eLoopPlayer+1))
	{
		const CvPlayerAI& kLoopPlayer = GET_PLAYER(eLoopPlayer);
		if (kLoopPlayer.getTeam() == eTeam && kLoopPlayer.isAlive())
		{
			int iLoop;
			for (CvCity* pLoopCity = kLoopPlayer.firstCity(&iLoop); pLoopCity; pLoopCity = kLoopPlayer.nextCity(&iLoop))
			{
				/*
				if (pLoopCity->hasActiveWorldWonder() || pLoopCity->getPopulation() > iSmallCityThreshold || pLoopCity->isHolyCity() ||
				pLoopCity->isGovernmentCenter() || pLoopCity->getCultureLevel() > (GC.getNumCultureLevelInfos() - 2))
				{*/
				bool bClose = false;
				for( int iJ = 0; iJ < MAX_CIV_PLAYERS; iJ++ )
				{
					if( getID() == GET_PLAYER((PlayerTypes)iJ).getTeam() && GET_PLAYER((PlayerTypes)iJ).isAlive() )
					{
						if( pLoopCity->AI_playerCloseness((PlayerTypes)iJ ) > 0 )
						{
							bClose = true;
							break;
						}
					}
				}
				int iCityValue;
				if (pLoopCity->getPopulation() > iBigCityThreshold)
				{
					iCityValue = (pLoopCity->getPopulation() - iBigCityThreshold) * (bClose ? 2 : 1);
					iCityValue += (iBigCityThreshold - iSmallCityThreshold) / (bClose ? 1 : 2); 
				}
				else
				{
					iCityValue = std::max(0, pLoopCity->getPopulation() - iSmallCityThreshold) / (bClose ? 1 : 2);
				}
				iCityValue += (bClose ? 10 : 5) * pLoopCity->getNumActiveWorldWonders();
				for (int iI = 0; iI < GC.getNumReligionInfos(); iI++)
				{
					if (pLoopCity->isHolyCity((ReligionTypes)iI))
					{
						iCityValue += GC.getGameINLINE().calculateReligionPercent((ReligionTypes)iI) / (bClose ? 5 : 10);
					}
				}
				//Victory stopping stuff, only if we're close, stopping other players winning in general is already taken into account
				if (bClose)
				{
					if (pLoopCity->isGovernmentCenter())
					{
						iCityValue++;
						if (pLoopCity->isCapital() && GET_TEAM(eTeam).getVictoryCountdown(GC.getGameINLINE().getSpaceVictory()) >= 0)
						{
							iCityValue += 30;
						}
					}
					if (pLoopCity->getCultureLevel() > GC.getNumCultureLevelInfos() - 2)
					{
						iCityValue += pLoopCity->getCultureLevel();
						if (2*pLoopCity->getCulture(eLoopPlayer) > pLoopCity->getCultureThreshold(GC.getGameINLINE().culturalVictoryCultureLevel()))
						{
							iCityValue += 10;
						}
					}
				}
				if (iCityValue > 0)
				{
					iCityValue *= 100 + std::min(pLoopCity->calculateTeamCulturePercent(getID()), 50);
					iCityValue /= 100;
					iValue += iCityValue;
				}
			}
		}
		//CD Tweaks - if our own cities have their culture, try to destroy them if they're weaker
		else if (kLoopPlayer.getTeam() == getID() && kLoopPlayer.isAlive() && iCultureDivisor < 60)
		{
			int iLoop;
			for (CvCity* pLoopCity = kLoopPlayer.firstCity(&iLoop); pLoopCity; pLoopCity = kLoopPlayer.nextCity(&iLoop))
			{
				if (!GC.getGameINLINE().isOption(GAMEOPTION_NO_REVOLUTION) || 
					(pLoopCity->plot()->findHighestCultureTeam() == eTeam && pLoopCity->plot()->isWithinCultureRange(pLoopCity->plot()->findHighestCulturePlayer())))
				{
					iValue += std::max(0, (pLoopCity->plot()->calculateTeamCulturePercent(eTeam)-40) / iCultureDivisor); 
				}
			}
		}
	}
	return iValue;
}
//CD Tweaks End

int CvTeamAI::AI_calculateTechWarValue(TeamTypes eTeam) const
{
	//if we can't trade
	if (!isTechTrading() || GET_TEAM(eTeam).isAVassal())
	{
		return 0;
	}

	int iScienceMultiplier = 0;
	int iCount = 0;
	for( int iJ = 0; iJ < MAX_CIV_PLAYERS; iJ++ )
	{
		if( getID() == GET_PLAYER((PlayerTypes)iJ).getTeam() && GET_PLAYER((PlayerTypes)iJ).isAlive() )
		{
			iScienceMultiplier += GET_PLAYER((PlayerTypes)iJ).AI_averageCommerceMultiplier(COMMERCE_RESEARCH);
			iCount++;
		}
	}
	iScienceMultiplier /= std::max(1, iCount);

	int iValue = 0;
	for (int iTech = 0; iTech < GC.getNumTechInfos(); iTech++)
	{
		TechTypes eTech = (TechTypes)iTech;
		if (!isHasTech(eTech) && GET_TEAM(eTeam).isHasTech(eTech))
		{
			int iTechValue = std::max(0, (getResearchCost(eTech) - getResearchProgress(eTech) * (GC.getGameINLINE().isOption(GAMEOPTION_NO_TECH_BROKERING) ? 2 : 1)));

			iTechValue *= 100;
			iTechValue /= iScienceMultiplier;

			//use of train percent instead of research percent is deliberate, if tech takes too long waring is preferable
			//no scaling down by map size is also deliberate, bonus and city values are also not scaled down by map size
			iTechValue *= 100;	
			iTechValue /= GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getTrainPercent();

			iValue += std::max(getTechScore(eTech),iTechValue / 100);	//1000 beakers = 10 value
		}
	}
	//scale up or down the value based on how small or weak as they should be easier too bully
	int iBestPercent = ((100 * getPower(true)) / std::max(1,GET_TEAM(eTeam).getDefensivePower()));
	iBestPercent = std::max(iBestPercent, (100 * getTotalLand(false)) / std::max(1,GET_TEAM(eTeam).getTotalLand(false)));
	iBestPercent = std::max(iBestPercent, (100 * getTotalPopulation(false)) / std::max(1,GET_TEAM(eTeam).getTotalPopulation(false)));
	iValue *= range(iBestPercent, 50, 400);
	iValue /= 100;
	//limit based on civ size
	iValue = std::min(iValue, 10*GET_TEAM(eTeam).getNumCities());
	return iValue;
}
//CD Tweaks End

//CD Tweaks - raw capital proximity plays a huge part in deciding who to attack, but there are many cases when it shouldn't
//this modifier will return a percentage to modify the value down and allow other values to have more relative importance
int CvTeamAI::AI_calculateWarProximityModifier() const
{
	int iBlitzCount = 0;
	int iFriendlyCount = 0;
	int iLandTargetCount = 0;
	int iSeaTargetCount = 0;
	int iTotalMoveCost = 0;
	int iOwnedTiles = 0;
	int iUnownedTiles = 0;
	int iPrimaryAreaCount = 0;
	CvCity* pLoopCity;
	int iLoop;
	int iFastestShipSpeed = 0;
	for (int iI = 0; iI < GC.getNumUnitClassInfos(); iI++)
	{
		UnitTypes eLoopUnit = (UnitTypes)GC.getUnitClassInfo((UnitClassTypes)iI).getDefaultUnitIndex();
		if (eLoopUnit == NO_UNIT)
			continue;
		CvUnitInfo& kUnitInfo = GC.getUnitInfo(eLoopUnit);
		if (kUnitInfo.getDomainType() == DOMAIN_LAND && kUnitInfo.getUnitAIType(UNITAI_ASSAULT_SEA))
		{
			for( int iJ = 0; iJ < MAX_CIV_PLAYERS; iJ++ )
			{
				if (GET_PLAYER((PlayerTypes)iJ).isAlive() && getID() == GET_PLAYER((PlayerTypes)iJ).getTeam())
				{
					if (GET_PLAYER((PlayerTypes)iJ).getUnitClassCountPlusMaking((UnitClassTypes)iI) > 0)
					{
						iFastestShipSpeed = std::max(iFastestShipSpeed, kUnitInfo.getMoves());
					}
				}
			}
		}
	}
	iFastestShipSpeed += getExtraMoves(DOMAIN_SEA);
	CvArea* pLoopArea = NULL;
	for(pLoopArea = GC.getMapINLINE().firstArea(&iLoop); pLoopArea != NULL; pLoopArea = GC.getMapINLINE().nextArea(&iLoop))
	{
		if (AI_isPrimaryArea(pLoopArea))
		{
			iLandTargetCount += pLoopArea->getNumCities() - countNumCitiesByArea(pLoopArea) - GET_TEAM(BARBARIAN_TEAM).countNumCitiesByArea(pLoopArea);
			iOwnedTiles += pLoopArea->getNumOwnedTiles();
			iUnownedTiles += std::max(pLoopArea->getNumUnownedTiles(), pLoopArea->getNumUnrevealedTiles(getID())/2);
			iPrimaryAreaCount++;
		}
		else if (pLoopArea->getNumRevealedTiles(getID()) > 0)
		{
			iSeaTargetCount += pLoopArea->getNumCities() - countNumCitiesByArea(pLoopArea) - GET_TEAM(BARBARIAN_TEAM).countNumCitiesByArea(pLoopArea);
		}
	}
	for( int iJ = 0; iJ < MAX_CIV_PLAYERS; iJ++ )
	{
		if (GET_PLAYER((PlayerTypes)iJ).isAlive() )
		{
			if (isFriendlyTerritory(GET_PLAYER((PlayerTypes)iJ).getTeam()) || isOpenBorders(GET_PLAYER((PlayerTypes)iJ).getTeam()))
			{
				if( getID() == GET_PLAYER((PlayerTypes)iJ).getTeam())
				{
					if (GET_PLAYER((PlayerTypes)iJ).AI_isDoStrategy(AI_STRATEGY_AIR_BLITZ) || GET_PLAYER((PlayerTypes)iJ).AI_isDoStrategy(AI_STRATEGY_LAND_BLITZ))
					{
						iBlitzCount++;
					}
				}
				for (pLoopCity = GET_PLAYER((PlayerTypes)iJ).firstCity(&iLoop); pLoopCity != NULL; pLoopCity = GET_PLAYER((PlayerTypes)iJ).nextCity(&iLoop))
				{
					if (GET_PLAYER((PlayerTypes)iJ).AI_isPrimaryArea(pLoopCity->area()))
					{
						if (pLoopCity->area()->getNumCities() - countNumCitiesByArea(pLoopCity->area()) - GET_TEAM(BARBARIAN_TEAM).countNumCitiesByArea(pLoopCity->area()) > 2)
						{
							RouteTypes eRoute = pLoopCity->plot()->getRevealedRouteType(getID(), false);
							if (eRoute != NO_ROUTE)
							{
								int iMoveCost = GC.getRouteInfo(pLoopCity->plot()->getRouteType()).getFlatMovementCost();
								iTotalMoveCost += iMoveCost;
								iFriendlyCount++;
							}
						}
					}
				}
			}			
		}
	}
	int iLandTargetPercent = iSeaTargetCount == 0 ? 100 : (100 * iLandTargetCount) / (iLandTargetCount + iSeaTargetCount);
	int iSeaTargetPercent = 100 - iLandTargetPercent;
	//modify down based on routes that our used in our areas
	if (iFriendlyCount > 0)
	{
		int iAverageLandMoveCost = iTotalMoveCost / iFriendlyCount; //this is the average movement cost of cities in our primary areas that we can move through but have other teams cities on them
		if (iAverageLandMoveCost < 20)
		{
			iLandTargetPercent *= iAverageLandMoveCost;
			iLandTargetPercent /= 20;
		}
	}
	//if there are many unowned/unexplored tiles, venture further
	if (iUnownedTiles > 0)
	{
		iLandTargetPercent *= iOwnedTiles;
		iLandTargetPercent /= iOwnedTiles + iUnownedTiles;
	}
	//modify down based on ship speed
	if (iFastestShipSpeed > 4)
	{
		iSeaTargetPercent *= 4;
		iSeaTargetPercent /= iFastestShipSpeed;
	}
	int iResult = iLandTargetPercent + iSeaTargetPercent;
	//less importance on proximity if we're blitzing
	iResult *= (50 * getAliveCount() + 50 * (getAliveCount() - iBlitzCount)) / getAliveCount();
	iResult /= 100;
	//less importance on proximity if we have lots of vassals
	if (getVassalCount() > 0)
	{
		iResult *= getTotalLand(false);
		iResult /= getTotalLand(true);
	}
	//if we have multipler primary areas care less about proximity
	iResult *= 2;
	iResult /= iPrimaryAreaCount + 1;
	return iResult;
}


//CD Tweaks - calculates the relative strength of our best unit for a domain compared to theirs.
//for us it's the best unit we can train, if it requires a bonus we need to have built or be building at least one to prove we can
//for them we use a combination of units we've seen, what we know they have the tech to build, and some cheating, consider their average unit
//strength as well than total
//doesn't yet work for air units
int CvTeamAI::AI_calculateRelativeDomainStrength(TeamTypes eTeam, DomainTypes eDomain) const
{
	int iOurBestUnitCombat = 0;
	int iTheirBestUnitCombat = 0;
	int iTheirTotalCombat = 0;
	int iCount = 0;
	int iHighestSeenUnitCombat = 0;
	for (int iI = 0; iI < GC.getNumUnitClassInfos(); iI++)
	{
		UnitTypes eLoopUnit = (UnitTypes)GC.getUnitClassInfo((UnitClassTypes)iI).getDefaultUnitIndex();
		if (eLoopUnit == NO_UNIT)
			continue;
		CvUnitInfo& kUnitInfo = GC.getUnitInfo(eLoopUnit);
		if (kUnitInfo.getDomainType() == eDomain)
		{
			bool bRequiresBonus = false;
			if (kUnitInfo.getPrereqAndBonus() != NO_BONUS)
			{
				bRequiresBonus = true;
			}
			else
			{
				for (int iJ = 0; iJ < GC.getNUM_UNIT_PREREQ_OR_BONUSES(); ++iJ)
				{
					if (kUnitInfo.getPrereqOrBonuses(iJ) != NO_BONUS)
					{
						bRequiresBonus = true;
						break;
					}
				}
			}
			for( int iJ = 0; iJ < MAX_CIV_PLAYERS; iJ++ )
			{
				if (GET_PLAYER((PlayerTypes)iJ).isAlive())
				{
					if (getID() == GET_PLAYER((PlayerTypes)iJ).getTeam())
					{
						if (bRequiresBonus)
						{
							if (GET_PLAYER((PlayerTypes)iJ).getUnitClassCountPlusMaking((UnitClassTypes)iI) > 0)
							{
								iOurBestUnitCombat = std::max(iOurBestUnitCombat, kUnitInfo.getCombat());
							}	
						}
						else if (GET_PLAYER((PlayerTypes)iJ).canTrain(eLoopUnit))
						{
							iOurBestUnitCombat = std::max(iOurBestUnitCombat, kUnitInfo.getCombat());
						}
						if (GET_PLAYER((PlayerTypes)iJ).AI_getUnitClassWeight((UnitClassTypes)iI) > 0)
						{
							iHighestSeenUnitCombat = std::max(iHighestSeenUnitCombat, kUnitInfo.getCombat());
						}
					}
					else if (eTeam == GET_PLAYER((PlayerTypes)iJ).getTeam())
					{
						int iTotalPlusMaking = GET_PLAYER((PlayerTypes)iJ).getUnitClassCountPlusMaking((UnitClassTypes)iI);
						if (iTotalPlusMaking > 0)
						{
							iTheirBestUnitCombat = std::max(iTheirBestUnitCombat, kUnitInfo.getCombat());
							iTheirTotalCombat += iTotalPlusMaking * kUnitInfo.getCombat();
							//modify total combat based on unit abilities
							iTheirTotalCombat += (std::min(iTotalPlusMaking,GET_PLAYER((PlayerTypes)iJ).getNumCities()) * kUnitInfo.getCityDefenseModifier()) / 100;
							if (eDomain = DOMAIN_LAND)
							{
								//want to do proper analysis vs our units but too complicated for now, at least this will give some boost to machine gunners etc.
								iTheirTotalCombat += (std::min(iTotalPlusMaking,GET_PLAYER((PlayerTypes)iJ).getNumCities()) * kUnitInfo.getUnitCombatModifier(GC.getGameINLINE().getBestLandUnitCombat())) / 100;
							}
							iCount += iTotalPlusMaking;
						}
						if (!bRequiresBonus && GET_PLAYER((PlayerTypes)iJ).canTrain(eLoopUnit))
						{
							iTheirBestUnitCombat = std::max(iOurBestUnitCombat, kUnitInfo.getCombat());
							iHighestSeenUnitCombat = std::max(iHighestSeenUnitCombat, kUnitInfo.getCombat());	//counts as "seen" as we know they can build it
						}
					}
				}
			}
		}
	}
	iTheirBestUnitCombat = std::min(iTheirBestUnitCombat,iHighestSeenUnitCombat);
	//for them, average their best possible unit with their average unit strength
	//for AI total is more heavily weighted since they can upgrade more easily
	if (iCount > 0)
	{
		int iTheirAverageUnitCombat = iTheirTotalCombat / iCount;
		if (iTheirAverageUnitCombat > iTheirBestUnitCombat)
		{
			iTheirBestUnitCombat = iTheirAverageUnitCombat;	//this can happen as average is modified and can include unseen units
		}
		else
		{
			int iRatio = 50;
			if (!GET_TEAM(eTeam).isHuman())
			{
				iRatio *= GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAIUnitUpgradePercent();
				iRatio /= 100;
			}
			iRatio = std::max(1,iRatio);
			iTheirBestUnitCombat = (iTheirAverageUnitCombat*iRatio + iTheirBestUnitCombat*(100-iRatio)) / 100;
		}
	}
	return (100*iOurBestUnitCombat)/std::max(1,iTheirBestUnitCombat);	
}
//CD Tweaks End

int CvTeamAI::AI_calculateCapitalProximity(TeamTypes eTeam) const
{
	CvCity* pOurCapitalCity;
	CvCity* pTheirCapitalCity;
	int iTotalDistance;
	int iCount;
	int iI, iJ;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	iTotalDistance = 0;
	iCount = 0;

	int iMinDistance = MAX_INT;
	int iMaxDistance = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				pOurCapitalCity = GET_PLAYER((PlayerTypes)iI).getCapitalCity();

				if (pOurCapitalCity != NULL)
				{
					for (iJ = 0; iJ < MAX_PLAYERS; iJ++)
					{
						if (GET_PLAYER((PlayerTypes)iJ).isAlive())
						{
							if (GET_PLAYER((PlayerTypes)iJ).getTeam() != getID())
							{
								pTheirCapitalCity = GET_PLAYER((PlayerTypes)iJ).getCapitalCity();

								if (pTheirCapitalCity != NULL)
								{
									int iDistance = (plotDistance(pOurCapitalCity->getX_INLINE(), pOurCapitalCity->getY_INLINE(), pTheirCapitalCity->getX_INLINE(), pTheirCapitalCity->getY_INLINE()) * (pOurCapitalCity->area() != pTheirCapitalCity->area() ? 3 : 2));
									if (GET_PLAYER((PlayerTypes)iJ).getTeam() == eTeam)
									{
										iTotalDistance += iDistance;
										iCount++;
									}
									iMinDistance = std::min(iDistance, iMinDistance);
									iMaxDistance = std::max(iDistance, iMaxDistance);
								}
							}
						}
					}
				}
			}
		}
	}

	if (iCount > 0)
	{
		FAssert(iMaxDistance > 0);
		return ((GC.getMapINLINE().maxPlotDistance() * (iMaxDistance - ((iTotalDistance / iCount) - iMinDistance))) / iMaxDistance);
	}

	return 0;
}

//finds the distance between what we estimate to be our nearest two cities:
//finds the nearest city to our capital, and then our nearest one of our cities to that city
//returns the step distance if we didn't find one of their cities in one of our capital areas
//and the path distance otherwise. returns -1 if it fails to find anything
//Not guaranteed to be the nearest two cities but faster than checking every one
int CvTeamAI::AI_estimateClosestCityDistance(TeamTypes eTeam) const
{
	CvCity* pOurCapitalCity;
	CvCity* pTheirNearestCity;
	CvCity* pOurNearestCity;
	int iBestDistance = MAX_INT;
	int iI;
	bool bSameArea = true;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				pOurCapitalCity = GET_PLAYER((PlayerTypes)iI).getCapitalCity();

				if (pOurCapitalCity != NULL)
				{
					CvCity* pTestCity = GC.getMapINLINE().findCity(pOurCapitalCity->plot()->getX_INLINE(), pOurCapitalCity->plot()->getY_INLINE(), NO_PLAYER, eTeam, true);

					if (pTestCity == NULL)
					{
						CvCity* pTestCity = GC.getMapINLINE().findCity(pOurCapitalCity->plot()->getX_INLINE(), pOurCapitalCity->plot()->getY_INLINE(), NO_PLAYER, eTeam, false);
					}

					if (pTestCity != NULL)
					{
						int iDistance = stepDistance(pOurCapitalCity->getX_INLINE(), pOurCapitalCity->getY_INLINE(), pTestCity->getX_INLINE(), pTestCity->getY_INLINE());
						if (iDistance < iBestDistance)
						{
							iBestDistance = iDistance;	
							pTheirNearestCity = pTestCity;
							bSameArea = AI_isPrimaryArea(pTestCity->area());
						}
					}
				}
			}
		}
	}

	if (pTheirNearestCity != NULL)
	{
		pOurNearestCity = GC.getMapINLINE().findCity(pTheirNearestCity->plot()->getX_INLINE(), pTheirNearestCity->plot()->getY_INLINE(), NO_PLAYER, getID(), bSameArea);
		if (pOurNearestCity != NULL)
		{
			if (bSameArea)
			{
				return GC.getMapINLINE().calculatePathDistance(pOurNearestCity->plot(), pTheirNearestCity->plot());				
			}
			else
			{
				return stepDistance(pOurNearestCity->getX_INLINE(), pOurNearestCity->getY_INLINE(), pTheirNearestCity->getX_INLINE(), pTheirNearestCity->getY_INLINE());
			}
		}
	}

	return -1;
}

//CD Tweaks - calculates the raw step distance between (one of) our capitals and one of their capitals
//optionally including their vassals (but never ours). May be of some help determining if another team
//is a good target for war (or peace) but is obviously quite limited
int CvTeamAI::AI_getClosetCapitalDistance(TeamTypes eTeam, bool bIncludeVassals) const
{
	CvCity* pOurCapitalCity;
	CvCity* pTheirCapitalCity;
	int iI, iJ;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	int iMinDistance = GC.getMapINLINE().maxStepDistance();

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				pOurCapitalCity = GET_PLAYER((PlayerTypes)iI).getCapitalCity();

				if (pOurCapitalCity != NULL)
				{
					for (iJ = 0; iJ < MAX_PLAYERS; iJ++)
					{
						if (GET_PLAYER((PlayerTypes)iJ).isAlive())
						{
							if (GET_PLAYER((PlayerTypes)iJ).getTeam() != getID())
							{
								pTheirCapitalCity = GET_PLAYER((PlayerTypes)iJ).getCapitalCity();

								if (pTheirCapitalCity != NULL)
								{
									if (GET_PLAYER((PlayerTypes)iJ).getTeam() == eTeam || (bIncludeVassals && GET_TEAM(GET_PLAYER((PlayerTypes)iJ).getTeam()).isVassal(eTeam)))
									{
										int iDistance = (stepDistance(pOurCapitalCity->getX_INLINE(), pOurCapitalCity->getY_INLINE(), pTheirCapitalCity->getX_INLINE(), pTheirCapitalCity->getY_INLINE()));									
										iMinDistance = std::min(iDistance, iMinDistance);
									}

								}
							}
						}
					}
				}
			}
		}
	}

	return iMinDistance;
}
//CD Tweaks End

bool CvTeamAI::AI_isWarPossible() const
{
	if (getAtWarCount(false) > 0)
	{
		return true;
	}

	if (GC.getGameINLINE().isOption(GAMEOPTION_ALWAYS_WAR))
	{
		return true;
	}

	if (!(GC.getGameINLINE().isOption(GAMEOPTION_ALWAYS_PEACE)) && !(GC.getGameINLINE().isOption(GAMEOPTION_NO_CHANGING_WAR_PEACE)))
	{
		return true;
	}

	return false;
}

/************************************************************************************************/
/* BETTER_BTS_AI_MOD                      06/12/10                         Fuyu & jdog5000      */
/*                                                                                              */
/* War Strategy AI                                                                              */
/************************************************************************************************/
bool CvTeamAI::AI_isLandTarget(TeamTypes eTeam, bool bNeighborsOnly) const
{
	if (!AI_hasCitiesInPrimaryArea(eTeam))
	{
		return false;
	}

	// If shared capital area is largely unclaimed, then we can reach over land
	int iModifier = 100;

	if( !bNeighborsOnly )
	{
		for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
		{
			if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() && GET_PLAYER((PlayerTypes)iPlayer).getNumCities() > 0 )
			{
				CvCity* pCapital = GET_PLAYER((PlayerTypes)iPlayer).getCapitalCity();
				if( pCapital != NULL )
				{
					if( GET_TEAM(eTeam).AI_isPrimaryArea(pCapital->area()) )
					{
						iModifier *= pCapital->area()->getNumOwnedTiles();
						iModifier /= pCapital->area()->getNumTiles();
					}
				}
			}
		}
	}

	//CD Tweaks if we have a capitulated vassal, and it could be a land target of us, and the passed in team could be a land target of it
	//then the passed in team is a land target of us
	//note - modifier will mean we can get away with no adjacent plots if the land in our capital area is less than 45% owned
	if (AI_calculateAdjacentLandPlots(eTeam) >= range((8 * (iModifier - 40))/40, 0, 8))
	{
		return true;
	}
	else if ( !bNeighborsOnly )
	{
		for (int iTeam = 0; iTeam < MAX_CIV_TEAMS; iTeam++)
		{
			CvTeam& kLoopTeam = GET_TEAM((TeamTypes)iTeam);
			if (kLoopTeam.isVassal(getID()) && kLoopTeam.isCapitulated() && !kLoopTeam.canVassalRevolt(getID()) && 
				AI_isLandTarget((TeamTypes)iTeam, true) && GET_TEAM((TeamTypes)iTeam).AI_isLandTarget(eTeam,true))
			{
				return true;
			}
		}
	}
	return false;
	//CD Tweaks End
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                       END                                                  */
	/************************************************************************************************/

	return true;
}

// this determines if eTeam or any of its allies are land targets of us
bool CvTeamAI::AI_isAllyLandTarget(TeamTypes eTeam) const
{
	for (int iTeam = 0; iTeam < MAX_CIV_TEAMS; iTeam++)
	{
		CvTeam& kLoopTeam = GET_TEAM((TeamTypes)iTeam);
		if (iTeam != getID())
		{
			if (iTeam == eTeam || kLoopTeam.isVassal(eTeam) || GET_TEAM(eTeam).isVassal((TeamTypes)iTeam) || kLoopTeam.isDefensivePact(eTeam))
			{
				if (AI_isLandTarget((TeamTypes)iTeam))
				{
					return true;
				}
			}
		}
	}

	return false;
}


bool CvTeamAI::AI_shareWar(TeamTypes eTeam) const
{
	int iI;

	/************************************************************************************************/
	/* REVOLUTION_MOD                         10/25/08                                jdog5000      */
	/*                                                                                              */
	/* For minor civs, StartAsMinors                                                                */
	/************************************************************************************************/
	/* original BTS code
	for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
	{
	if (GET_TEAM((TeamTypes)iI).isAlive() && !GET_TEAM((TeamTypes)iI).isMinorCiv())
	{
	if ((iI != getID()) && (iI != eTeam))
	{
	if (isAtWar((TeamTypes)iI) && GET_TEAM(eTeam).isAtWar((TeamTypes)iI))
	{
	return true;
	}
	}
	}
	}
	*/
	// No dealing with minor civs
	if( isMinorCiv() || GET_TEAM(eTeam).isMinorCiv() )
	{
		return false;
	}

	// Only accumulate if someone actually declared war, not a left over from StartAsMinors
	for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
	{
		if (GET_TEAM((TeamTypes)iI).isAlive() && !GET_TEAM((TeamTypes)iI).isMinorCiv())
		{
			if ((iI != getID()) && (iI != eTeam))
			{
				if (isAtWar((TeamTypes)iI) && GET_TEAM(eTeam).isAtWar((TeamTypes)iI))
				{
					if( AI_getWarPlan((TeamTypes)iI) != WARPLAN_LIMITED || GET_TEAM(eTeam).AI_getWarPlan((TeamTypes)iI) != WARPLAN_LIMITED || GET_TEAM((TeamTypes)iI).AI_getWarPlan(getID()) != WARPLAN_LIMITED || GET_TEAM((TeamTypes)iI).AI_getWarPlan(eTeam) != WARPLAN_LIMITED )
					{
						return true;
					}
				}
			}
		}
	}
	/************************************************************************************************/
	/* REVOLUTION_MOD                          END                                                  */
	/************************************************************************************************/

	return false;
}


AttitudeTypes CvTeamAI::AI_getAttitude(TeamTypes eTeam, bool bForced) const
{
	int iAttitude;
	int iCount;
	int iI, iJ;

	//FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");
	// K-Mod
	if (eTeam == getID())
		return ATTITUDE_FRIENDLY;
	// K-Mod end
	// CD - I'm guessing from the above code that this is in fact called alot on ourselves

	iAttitude = 0;
	iCount = 0;

	//CD Tweaks - vassals count less.
	//1 vassal - count as half as much as master
	//2 or more vassals - all vassals combined count as much as master
	int iVassalCount = GET_TEAM(eTeam).getVassalCount();
	int iMasterWeight = iVassalCount;
	if (iVassalCount <= 1)
	{
		iMasterWeight++;
	}
	//CD Tweaks End

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				for (iJ = 0; iJ < MAX_PLAYERS; iJ++)
				{
					if (GET_PLAYER((PlayerTypes)iJ).isAlive() && iI != iJ)
					{
						// BBAI TODO: Attitude averaging ... what to do?
						TeamTypes eTeamLoop = GET_PLAYER((PlayerTypes)iJ).getTeam();
						//CD Tweaks - vassals count less - master attitutude is weighted
						if (eTeamLoop == eTeam)
						{
							/************************************************************************************************/
							/* BETTER_BTS_AI_MOD                      01/25/09                                jdog5000      */
							/*                                                                                              */
							/* Diplomacy AI                                                                                 */
							/************************************************************************************************/
							/*
							iAttitude += GET_PLAYER((PlayerTypes)iI).AI_getAttitude((PlayerTypes)iJ, bForced);
							*/
							iAttitude += GET_PLAYER((PlayerTypes)iI).AI_getAttitudeVal((PlayerTypes)iJ, bForced) * iMasterWeight;
							/************************************************************************************************/
							/* BETTER_BTS_AI_MOD                       END                                                  */
							/************************************************************************************************/
							iCount += iMasterWeight;
						} 
						else if (GET_TEAM(eTeamLoop).isVassal(eTeam) || GET_TEAM(eTeam).isVassal(eTeamLoop))
						{
							iAttitude += GET_PLAYER((PlayerTypes)iI).AI_getAttitudeVal((PlayerTypes)iJ, bForced);
							iCount ++;
						}
						//CD Tweaks end
					}
				}
			}
		}
	}

	if (iCount > 0)
	{
		/************************************************************************************************/
		/* BETTER_BTS_AI_MOD                      01/25/09                                jdog5000      */
		/*                                                                                              */
		/* Diplomacy AI                                                                                 */
		/************************************************************************************************/
		/*
		return ((AttitudeTypes)(iAttitude / iCount));
		*/
		// This function is the same for all players, regardless of leader or whatever
		// so it's fine to use it for the team's attitude
		return GET_PLAYER(getLeaderID()).AI_getAttitudeFromValue(iAttitude/iCount);
		/************************************************************************************************/
		/* BETTER_BTS_AI_MOD                       END                                                  */
		/************************************************************************************************/
	}

	return ATTITUDE_CAUTIOUS;
}


int CvTeamAI::AI_getAttitudeVal(TeamTypes eTeam, bool bForced, bool bIgnorePeaceWeight) const
{
	int iAttitudeVal;
	int iCount;
	int iI, iJ;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	iAttitudeVal = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				for (iJ = 0; iJ < MAX_PLAYERS; iJ++)
				{
					if (GET_PLAYER((PlayerTypes)iJ).isAlive())
					{
						if (GET_PLAYER((PlayerTypes)iJ).getTeam() == eTeam)
						{
							iAttitudeVal += GET_PLAYER((PlayerTypes)iI).AI_getAttitudeVal((PlayerTypes)iJ, bForced, bIgnorePeaceWeight);
							iCount++;
						}
					}
				}
			}
		}
	}

	if (iCount > 0)
	{
		return (iAttitudeVal / iCount);
	}

	return 0;
}

//new function - get distrust value based on how many times team has attacked us or our friends
int CvTeamAI::AI_getWarDistrustVal(TeamTypes eTeam) const
{
	return (AI_getMemoryCount(eTeam, MEMORY_DECLARED_WAR) * 3) + 
		AI_getMemoryCount(eTeam, MEMORY_DECLARED_WAR_ON_FRIEND);
}

int CvTeamAI::AI_getMemoryCount(TeamTypes eTeam, MemoryTypes eMemory) const
{
	int iMemoryCount;
	int iCount;
	int iI, iJ;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	iMemoryCount = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				for (iJ = 0; iJ < MAX_PLAYERS; iJ++)
				{
					if (GET_PLAYER((PlayerTypes)iJ).isAlive())
					{
						if (GET_PLAYER((PlayerTypes)iJ).getTeam() == eTeam)
						{
							iMemoryCount += GET_PLAYER((PlayerTypes)iI).AI_getMemoryCount(((PlayerTypes)iJ), eMemory);
							iCount++;
						}
					}
				}
			}
		}
	}

	if (iCount > 0)
	{
		return (iMemoryCount / iCount);
	}

	return 0;
}


int CvTeamAI::AI_chooseElection(const VoteSelectionData& kVoteSelectionData) const
{
	VoteSourceTypes eVoteSource = kVoteSelectionData.eVoteSource;

	FAssert(!isHuman());
	FAssert(GC.getGameINLINE().getSecretaryGeneral(eVoteSource) == getID());

	int iBestVote = -1;
	int iBestValue = 0;

	for (int iI = 0; iI < (int)kVoteSelectionData.aVoteOptions.size(); ++iI)
	{
		VoteTypes eVote = kVoteSelectionData.aVoteOptions[iI].eVote;
		CvVoteInfo& kVoteInfo = GC.getVoteInfo(eVote);

		FAssert(kVoteInfo.isVoteSourceType(eVoteSource));

		FAssert(GC.getGameINLINE().isChooseElection(eVote));
		bool bValid = true;

		if (!GC.getGameINLINE().isTeamVote(eVote))
		{
			for (int iJ = 0; iJ < MAX_PLAYERS; iJ++)
			{
				if (GET_PLAYER((PlayerTypes)iJ).isAlive())
				{
					if (GET_PLAYER((PlayerTypes)iJ).getTeam() == getID())
					{
						PlayerVoteTypes eVote = GET_PLAYER((PlayerTypes)iJ).AI_diploVote(kVoteSelectionData.aVoteOptions[iI], eVoteSource, true);

						if (eVote != PLAYER_VOTE_YES || eVote == GC.getGameINLINE().getVoteOutcome((VoteTypes)iI))
						{
							bValid = false;
							break;
						}
					}
				}
			}
		}

		if (bValid)
		{
			int iValue = (1 + GC.getGameINLINE().getSorenRandNum(10000, "AI Choose Vote"));

			if (iValue > iBestValue)
			{
				iBestValue = iValue;
				iBestVote = iI;
			}
		}
	}

	return iBestVote;
}

/// \brief Relative value of starting a war against eTeam.
///
/// This function computes the value of starting a war against eTeam.
/// The returned value should be compared against other possible targets
/// to pick the best target.
int CvTeamAI::AI_startWarVal(TeamTypes eTeam) const
{
	PROFILE_FUNC();

	int iValue;
	bool bAssault = !AI_hasCitiesInPrimaryArea(eTeam);	//CD Tweaks

	//CD Tweaks - capital proximity value is modified down based on situation
	int iPlotValue = AI_calculatePlotWarValue(eTeam);
	iValue = (3 * AI_calculateCapitalProximity(eTeam)) / ((iPlotValue > 0) ? 2 : 3);
	int iProximityModifier = AI_calculateWarProximityModifier();
	iValue *= iProximityModifier;
	iValue /= 100;
	//also include the plot value of our nearby vassals
	for (int iI = 0; iI < MAX_CIV_TEAMS; iI++)
	{
		TeamTypes eLoopTeam = (TeamTypes) iI;
		if (iI != getID() && GET_TEAM(eLoopTeam).isAlive())
		{
			if (GET_TEAM(eLoopTeam).isVassal(getID()) && AI_teamCloseness(eLoopTeam) > 0)
			{
				iPlotValue += GET_TEAM(eLoopTeam).AI_calculatePlotWarValue(eTeam);
			}
		}
	}
	iValue += iPlotValue;
	//CD Tweaks End

	int iClosenessValue = AI_teamCloseness(eTeam);
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                      05/16/10                                jdog5000      */
	/*                                                                                              */
	/* War Strategy AI, Victory Strategy AI                                                         */
	/************************************************************************************************/
	/* original code
	if (iClosenessValue == 0)
	{
	iValue /= 4;
	}
	iValue += iClosenessValue / 4;
	*/
	// Dividing iValue by 4 is a drastic move, will result in more backstabbing between friendly neighbors
	// which is appropriate for Aggressive
	// Closeness values are much smaller after the fix to CvPlayerAI::AI_playerCloseness, no need to divide by 4
	if (iClosenessValue == 0)
	{
		//CD Tweaks - more aggressive if we're near the winning human player
		int iDivisor = 2;
		if (GC.getGameINLINE().isOption(GAMEOPTION_AGGRESSIVE_AI))
		{
			iDivisor = 4;
		}
		else if (GET_TEAM(GC.getGameINLINE().getRankTeam(0)).isHuman() && AI_teamCloseness(GC.getGameINLINE().getRankTeam(0)) > 0)
		{
			iDivisor = 3;
		}
		iValue /= iDivisor;
		//CD Tweaks end
	}
	iValue += iClosenessValue;

	iValue += AI_calculateBonusWarValue(eTeam);

	iValue += AI_calculateCityWarValue(eTeam);	//CD Tweaks

	iValue += AI_calculateTechWarValue(eTeam);	//CD Tweaks

	//boost assault value slightly if we already have transports
	if (bAssault)
	{
		for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
		{
			if(  GET_PLAYER((PlayerTypes)iPlayer).isAlive() && GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
			{
				iValue += GET_PLAYER((PlayerTypes)iPlayer).AI_totalUnitAIs(UNITAI_ASSAULT_SEA);
			}
		}
	}

	//CD tweaks - extend our reach - look for targets close to civs we're already winning a war against
	//or near to our nearby vassals or nearby small civs we're planning a war against
	for (int iI = 0; iI < MAX_CIV_TEAMS; iI++)
	{
		TeamTypes eLoopTeam = (TeamTypes) iI;
		if (iI != getID() && GET_TEAM(eLoopTeam).isAlive())
		{
			if (isAtWar(eLoopTeam) && AI_getWarSuccess(eLoopTeam) >= GET_TEAM(eLoopTeam).AI_getWarSuccess(getID()))
			{
				iValue += GET_TEAM(eLoopTeam).AI_teamCloseness(eTeam);
			}
			else if ((GET_TEAM(eLoopTeam).isVassal(getID()) || (AI_isChosenWar(eLoopTeam) && !GET_TEAM(eLoopTeam).canVassalRevolt(getID()) && AI_hasCitiesInPrimaryArea(eLoopTeam))) && AI_teamCloseness(eLoopTeam) > 0)
			{
				iValue += GET_TEAM(eLoopTeam).AI_teamCloseness(eTeam);
			}				
		}
	}
	//CD Tweaks end

	//CD Tweaks - logic for multiple wars (include minor civs for this)
	if (getAnyWarPlanCount(false) > 0)
	{
		//we don't want to plan assaults from areas that are already fighting ground wars
		//we don't want to fight new ground wars in areas that are plannin assaults, or are defensive
		int iLoop;
		int iGoodAreas = 0;
		int iBadAreas = 0;
		CvArea* pLoopArea = NULL;
		for(pLoopArea = GC.getMapINLINE().firstArea(&iLoop); pLoopArea != NULL; pLoopArea = GC.getMapINLINE().nextArea(&iLoop))
		{
			if (AI_isPrimaryArea(pLoopArea))
			{
				int iTheirAreaTargets = GET_TEAM(eTeam).countNumCitiesByArea(pLoopArea);
				int iExistingEnemyTargets = countNumEnemyCitiesByArea(pLoopArea, true);
				if (iTheirAreaTargets == 0 && iExistingEnemyTargets == 0)
				{
					iGoodAreas++;	//no new cities to attack in an area where we should already be planning an assault
				}
				else if (iTheirAreaTargets > 0 && iExistingEnemyTargets > 0)
				{
					if (pLoopArea->getAreaAIType(getID()) == AREAAI_DEFENSIVE)
					{
						iBadAreas++;	//planning a new ground war in an area we are already losing is a bad idea
					}
					else
					{
						iGoodAreas++;	//but otherwise adding to an existing land war is a good idea
					}
				}
				else
				{
					iBadAreas++;	
				}
			}
		}
		iValue *= iGoodAreas + 1;
		iValue /= iBadAreas + 1;
	}

	//disincentivise going for very far away land targets
	if (!bAssault)
	{
		if (iClosenessValue == 0 && iPlotValue == 0)
		{
			//travel less far if they're already at war with a powerful enemy by as by the time we arrive they could have lost or the situation changed
			int iMaxDistance = (GET_TEAM(eTeam).getPower(true) < (120 * GET_TEAM(eTeam).getEnemyPower()) / 100) ? 15 : 25;
			for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
			{
				if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
				{
					if (GET_PLAYER((PlayerTypes)iPlayer).AI_isDoStrategy(AI_STRATEGY_FASTMOVERS) || GET_PLAYER((PlayerTypes)iPlayer).AI_isDoStrategy(AI_STRATEGY_AIR_BLITZ))
					{
						iMaxDistance += 15;
						break;
					}
				}
			}
			int iNearestCityDistance = AI_estimateClosestCityDistance(eTeam);
			if (iNearestCityDistance > iMaxDistance)
			{
				iValue *= iMaxDistance;
				iValue /= iNearestCityDistance;			
				iValue *= 100;
				iValue /= 100 + iProximityModifier;
			}
		}
		else if (AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_DOMINATION3))
		{
			iValue *= 3;
			iValue /= 2;
		}
	}
	//CD Tweaks End

	// Target other teams close to victory
	if( GET_TEAM(eTeam).AI_isAnyMemberDoVictoryStrategyLevel3() )
	{
		iValue += 10;

		bool bAggressive = GC.getGameINLINE().isOption(GAMEOPTION_AGGRESSIVE_AI);
		bool bConq4 = AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CONQUEST4);
		bool bTheyreBigger = GET_TEAM(eTeam).getAliveCount() > getAliveCount();	//cd tweaks - team up on bigger teams

		// Prioritize targets closer to victory
		if( bConq4 || bAggressive || bTheyreBigger)
		{
			iValue *= 3;
			iValue /= 2;
		}

		if( GET_TEAM(eTeam).AI_isAnyMemberDoVictoryStrategyLevel4() )
		{
			if( GET_TEAM(eTeam).AI_getLowestVictoryCountdown() >= 0 )
			{
				iValue += 50;
			}

			iValue *= 2;

			if( bConq4 || bAggressive )
			{
				iValue *= 4;
			}
			else if (bTheyreBigger)
			{
				iValue *= 3;
			}
			else if( AI_isAnyMemberDoVictoryStrategyLevel3() )
			{
				iValue *= 2;
			}
		}	
	}

	/* CD Tweaks - this did nothing, this function is only useful to find the relative value of going to war
	// This adapted legacy code just makes us more willing to enter a war in a trade deal 
	// as boost applies to all rivals
	if( AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_DOMINATION3) )
	{
	iValue *= (GC.getGameINLINE().isOption(GAMEOPTION_AGGRESSIVE_AI) ? 3 : 2);
	}*/

	// If occupied or conquest inclined and early/not strong, value weak opponents
	if( getAnyWarPlanCount(true) > 0 || 
		(AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CONQUEST2) && !(AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CONQUEST3))) )
	{
		int iMultiplier = (75 * getPower(false))/std::max(1, GET_TEAM(eTeam).getDefensivePower());

		iValue *= range(iMultiplier, 50, 400);
		iValue /= 100;
	}
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                       END                                                  */
	/************************************************************************************************/
	//CD Tweaks - target backwards civs, as conquering them is easy
	if (!AI_isAnyMemberDoVictoryStrategyLevel4() && !GET_TEAM(eTeam).isAVassal() &&
		(AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CONQUEST2) || AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_DOMINATION3)))
	{
		int iOurTechScore = GET_PLAYER(getLeaderID()).getTechScore();
		int iTheirTechScore = GET_PLAYER(GET_TEAM(eTeam).getLeaderID()).getTechScore();
		if (iOurTechScore * 9 >= iTheirTechScore * 10)
		{
			iValue += iOurTechScore - iTheirTechScore;
			int iMultiplier = (100 * iOurTechScore)/std::max(1, iTheirTechScore);
			iValue *= range(iMultiplier, 100, 200);
			iValue /= 100;
		}
	}
	//the above is mostly superseeded by relative unit strengths below (which has a stronger effect), 
	//but I'm keeping both as they work differently
	//now consider the costs of invading
	//consider the strength of their units
	int iRelativeUnitStrength = AI_calculateRelativeDomainStrength(eTeam, DOMAIN_LAND);
	//this modifier gets exponentially large if unit strength difference is large, is limited before being applied though
	int iStrengthModifier = (iRelativeUnitStrength * iRelativeUnitStrength) / 100;
	if (bAssault)
	{
		iRelativeUnitStrength = AI_calculateRelativeDomainStrength(eTeam, DOMAIN_SEA);
		//if our ships are too weak to reach shore, ship unit strength matters
		if (iRelativeUnitStrength < 100)
		{
			iStrengthModifier = (iStrengthModifier + ((iRelativeUnitStrength * iRelativeUnitStrength) / 100)) / 2;			
		}
		//but otherwise it's less important than land unit strength
		else
		{
			iStrengthModifier = (iStrengthModifier + std::min(200,iRelativeUnitStrength)) / 2;			
			//but if we have strong ships, boost value, even if the civ even is far away
			if (iRelativeUnitStrength > 110)
			{
				iValue *= std::max(GC.getNumEraInfos(), GC.getNumEraInfos() / 2 + GET_PLAYER(getLeaderID()).getCurrentEra() + getExtraMoves(DOMAIN_SEA) + ((iClosenessValue == 0) ? 1 : 0));
				iValue /= GC.getNumEraInfos();
			}
		}
	}
	if (iStrengthModifier < 100)
	{
		iStrengthModifier = std::min(100,iStrengthModifier + (bAssault ? 10 : 20));
	}
	else
	{
		//reduce if they're stronger overall, we could still get bogged down by lots of obsolete units
		if (getPower(false) < GET_TEAM(eTeam).getDefensivePower())
		{
			iStrengthModifier *= getPower(false);
			iStrengthModifier /= GET_TEAM(eTeam).getDefensivePower();
		}
		iStrengthModifier = std::max(100,iStrengthModifier-20);		
	}
	iValue *= range(iStrengthModifier, 50, 400);
	iValue /= 100;

	// cost of upsetting their friends/benefit of pleasing their enemies
	// if iBase were based purely on their population, offending 3 mutual friends of equal size, or one mutual friend of triple the 
	// size, would halve the value of starting a war but joining the war on the side of a civ of equal size could double it
	// iBase is increased a bit to add the perception of some loyalty/friendship over and above pure calculations
	int iDiploCost = 0;
	int iBase = GET_TEAM(eTeam).getTotalPopulation(false) * 3 
		+ GC.getGameINLINE().getTotalPopulation() / GC.getGameINLINE().countCivPlayersAlive()
		+ (getTotalPopulation(true) * range(AI_getAttitudeVal(eTeam), 0, 10)) / 10;
	for (TeamTypes i = (TeamTypes)0; i < MAX_CIV_TEAMS; i=(TeamTypes)(i+1))
	{
		const CvTeamAI& kLoopTeam = GET_TEAM(i);

		if (kLoopTeam.isAlive() && i != getID() && i != eTeam)
		{
			if (isHasMet(i) && GET_TEAM(eTeam).isHasMet(i) && !kLoopTeam.isCapitulated() && !kLoopTeam.isHuman() && AI_getWarPlan(i) == NO_WARPLAN)
			{
				int iPop = kLoopTeam.getTotalPopulation(false);
				if (kLoopTeam.AI_getAttitude(eTeam) >= ATTITUDE_PLEASED && AI_getAttitude(i) >= ATTITUDE_PLEASED)
				{
					iDiploCost += iPop;
				}
				else if (kLoopTeam.isAtWar(eTeam) && AI_getAttitude(i) > ATTITUDE_FURIOUS && !AI_shareWar(i))
				{
					iDiploCost -= (iPop * range(AI_getAttitudeVal(i) - (AI_getShareWarCounter(i) / 40), 1, 6)) / 2;
				}
			}
		}
	}
	iDiploCost = range(iDiploCost, iBase*-2, iBase*2);
	if (iDiploCost > 0)
	{
		if ( !GET_TEAM(eTeam).AI_isAnyMemberDoVictoryStrategyLevel4() && AI_getWorstEnemy() != eTeam )
		{
			iValue *= iBase;
			iValue /= iBase + iDiploCost;
		}
	}
	else
	{
		iValue *= iBase - iDiploCost;
		iValue /= iBase;
	}
	//CD Tweaks End

	switch (AI_getAttitude(eTeam))
	{
	case ATTITUDE_FURIOUS:
		iValue *= 9;	//CD Tweaks - was 16
		break;

	case ATTITUDE_ANNOYED:
		iValue *= 6;	//CD Tweaks - was 8
		break;


	case ATTITUDE_CAUTIOUS:
		iValue *= 4;
		break;

	case ATTITUDE_PLEASED:
		iValue *= 2;
		break;

	case ATTITUDE_FRIENDLY:
		iValue *= 1;
		break;

	default:
		FAssert(false);
		break;
	}

	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                      03/21/10                                jdog5000      */
	/*                                                                                              */
	/* Victory Strategy AI                                                                          */
	/************************************************************************************************/
	// Make it harder to bribe player to start a war
	/*CD Tweaks - again, this did nothing
	if ( AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CULTURE4))
	{
		iValue /= 8;
	}
	else if ( AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_SPACE4))
	{
		iValue /= 4;
	}
	else if ( AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CULTURE3))
	{
		iValue /= 3;
	}*/
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                       END                                                  */
	/************************************************************************************************/

	return iValue;
}


// XXX this should consider area power...
int CvTeamAI::AI_endWarVal(TeamTypes eTeam) const
{
	int iValue;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");
	FAssertMsg(isAtWar(eTeam), "Current AI Team instance is expected to be at war with eTeam");

	//CD Tweaks - moved from below
	int iTheirAttackers = 0;
	int iTheirLandAttackers = 0;
	CvArea* pLoopArea = NULL;
	int iLoop;
	int iHighestEnemyPowerRatio = 0;
	int iClosenessValue = AI_teamCloseness(eTeam);	
	for(pLoopArea = GC.getMapINLINE().firstArea(&iLoop); pLoopArea != NULL; pLoopArea = GC.getMapINLINE().nextArea(&iLoop))
	{
		int iAreaDanger = countEnemyDangerByArea(pLoopArea, eTeam);
		iTheirAttackers += iAreaDanger;
		if (!(pLoopArea->isWater()))
		{
			iTheirLandAttackers += iAreaDanger;
		}
		//CD Tweaks - work out how much they overpower us by in this area
		if ((!(pLoopArea->isWater()) && countNumCitiesByArea(pLoopArea) > 0) || (pLoopArea->isWater() && GET_TEAM(getID()).AI_isWaterAreaRelevant(pLoopArea)))
		{
			int iPower = countPowerByArea(pLoopArea);
			int iEnemyPower = countEnemyPowerByArea(pLoopArea);
			int iDivisor = 1;
			if (pLoopArea->isWater())
			{
				iDivisor = 20;
			} 
			else if (iClosenessValue == 0)
			{
				iDivisor = 2;
			}
			int iRatio = (100 * iEnemyPower) / (std::max(iPower,1) * iDivisor);
			iHighestEnemyPowerRatio = std::max(iHighestEnemyPowerRatio, iRatio);
		}
	}
	//sea units were being considered too much, now only half as important
	iTheirAttackers = (iTheirAttackers + iTheirLandAttackers) / 2;
	//all this is to work out how much we fear their attacks, if they are no threat in any of our areas we will value peace lowly
	//even if they have been winning overall
	int iAttackFear = isHuman() ? 20 : range(iHighestEnemyPowerRatio / 4 + iTheirLandAttackers + AI_getMemoryCount(eTeam,MEMORY_RAZED_CITY), 1, 20);
	//CD Tweaks end

	iValue = 100;

	iValue += (getNumCities() * 3);
	iValue += (GET_TEAM(eTeam).getNumCities() * 3);

	iValue += getTotalPopulation();
	iValue += GET_TEAM(eTeam).getTotalPopulation();

	//CD Tweaks
	int iTheirWarSuccess = GET_TEAM(eTeam).AI_getWarSuccess(getID());
	//lets try to stop war success get to out of hand if we've just killed lots of units
	int iSoftLimit = std::max(GET_TEAM(eTeam).getNumCities(), GC.getWorldInfo(GC.getMapINLINE().getWorldSize()).getTargetNumCities()) * GC.getDefineINT("WAR_SUCCESS_CITY_CAPTURING");
	if (iTheirWarSuccess > iSoftLimit)
	{
		iTheirWarSuccess = (iTheirWarSuccess + iSoftLimit) / 2;
	}
	iValue += (iTheirWarSuccess * iAttackFear);  //CD Tweaks - was always * 20
	//CD Tweaks End

	int iOurPower = std::max(1, getPower(true));
	int iTheirPower = std::max(1, GET_TEAM(eTeam).getDefensivePower());

	//CD Tweaks - also factor in the power of other teams who have recently attacked them
	//so we don't miss out on a dogpile. We only know about teams we've met
	for (int iTeam = 0; iTeam < MAX_CIV_TEAMS; iTeam++)
	{
		TeamTypes eThirdTeam = (TeamTypes) iTeam;
		if (eThirdTeam != getID() && !GET_TEAM(eThirdTeam).isAVassal() && isHasMet(eThirdTeam) && 
			GET_TEAM(eThirdTeam).AI_isChosenWar(eTeam) && GET_TEAM(eThirdTeam).isAtWar(eTeam))
		{
			iOurPower += GET_TEAM(eThirdTeam).getPower(true) / (1 + GET_TEAM(eThirdTeam).AI_getAtWarCounter(eTeam));
		}
	}
	//CD Tweaks End

	iValue *= iTheirPower + 10;
	iValue /= std::max(1, iOurPower + iTheirPower + 10);

	WarPlanTypes eWarPlan = AI_getWarPlan(eTeam);

	// if we are not human, do we want to continue war for strategic reasons?
	// only check if our power is at least 120% of theirs
	if (!isHuman() && iOurPower > ((120 * iTheirPower) / 100))
	{
		bool bDagger = false;

		bool bAnyFinancialTrouble = false;
		for (int iI = 0; iI < MAX_PLAYERS; iI++)
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
				{
					if (GET_PLAYER((PlayerTypes)iI).AI_isDoStrategy(AI_STRATEGY_DAGGER))
					{
						bDagger = true;
					}

					if (GET_PLAYER((PlayerTypes)iI).AI_isFinancialTrouble())
					{
						bAnyFinancialTrouble = true;
					}
				}
			}
		}

		// if dagger, value peace at 90% * power ratio
		if (bDagger)
		{
			iValue *= 9 * iTheirPower;
			iValue /= 10 * iOurPower;
		}

		// for now, we will always do the land mass check for domination
		// if we have more than half the land, then value peace at 90% * land ratio 
		int iLandRatio = ((getTotalLand(true) * 100) / std::max(1, GET_TEAM(eTeam).getTotalLand(true)));
		if (iLandRatio > 120)
		{
			iValue *= 9 * 100;
			iValue /= 10 * iLandRatio;
		}

		// if in financial trouble, warmongers will continue the fight to make more money
		if (bAnyFinancialTrouble)
		{
			switch (eWarPlan)
			{
			case WARPLAN_TOTAL:
				// if we total warmonger, value peace at 70% * power ratio factor
				if (bDagger || AI_maxWarRand() < 100)
				{
					iValue *= 7 * (5 * iTheirPower);
					iValue /= 10 * (iOurPower + (4 * iTheirPower));
				}
				break;

			case WARPLAN_LIMITED:
				// if we limited warmonger, value peace at 70% * power ratio factor
				if (AI_limitedWarRand() < 100)
				{
					iValue *= 7 * (5 * iTheirPower);
					iValue /= 10 * (iOurPower + (4 * iTheirPower));
				}
				break;

			case WARPLAN_DOGPILE:
				// if we dogpile warmonger, value peace at 70% * power ratio factor
				if (AI_dogpileWarRand() < 100)
				{
					iValue *= 7 * (5 * iTheirPower);
					iValue /= 10 * (iOurPower + (4 * iTheirPower));
				}
				break;

			}
		}
	}

	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                      05/19/10                                jdog5000      */
	/*                                                                                              */
	/* War strategy AI, Victory Strategy AI                                                         */
	/************************************************************************************************/	
	if( AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CULTURE4) )
	{
		//CD Tweaks - value peace less if we don't trust them not to restart the war
		//iValue *= 4;
		int iDistrust = isHuman() ? 0 : AI_getWarDistrustVal(eTeam);
		iValue *= 4 + iDistrust;
		iValue /= 1 + iDistrust;
		//CD Tweaks End
	}
	else if( AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CULTURE3) || AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_SPACE4) )
	{
		//CD Tweaks - value peace less if we don't trust them not to restart the war
		//iValue *= 2;
		int iDistrust = isHuman() ? 0 : AI_getWarDistrustVal(eTeam);		
		iValue *= 2 + iDistrust;
		iValue /= 1 + iDistrust;
		//CD Tweaks End
	}


	if ((!(isHuman()) && (eWarPlan == WARPLAN_TOTAL)) ||
		(!(GET_TEAM(eTeam).isHuman()) && (GET_TEAM(eTeam).AI_getWarPlan(getID()) == WARPLAN_TOTAL)))
	{
		iValue *= 2;
	}
	else if ((!(isHuman()) && (eWarPlan == WARPLAN_DOGPILE) && (GET_TEAM(eTeam).getAtWarCount(true) > 1)) ||
		(!(GET_TEAM(eTeam).isHuman()) && (GET_TEAM(eTeam).AI_getWarPlan(getID()) == WARPLAN_DOGPILE) && (getAtWarCount(true) > 1)))
	{
		iValue *= 3;
		iValue /= 2;
	}

	// Do we have a big stack en route?
	int iOurAttackers = 0;
	for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
	{
		if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
		{
			iOurAttackers += GET_PLAYER((PlayerTypes)iPlayer).AI_enemyTargetMissions(eTeam);
		}
	}

	int iAttackerRatio = (100 * iOurAttackers) / std::max(1 + GC.getGameINLINE().getCurrentEra(), iTheirAttackers);

	if( GC.getGameINLINE().isOption(GAMEOPTION_AGGRESSIVE_AI) )
	{
		iValue *= 150;
		iValue /= range(iAttackerRatio, 150, 900);
	}
	else
	{
		iValue *= 200;
		iValue /= range(iAttackerRatio, 200, 600);
	}
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                       END                                                  */
	/************************************************************************************************/

	iValue -= (iValue % GC.getDefineINT("DIPLOMACY_VALUE_REMAINDER"));

	if (isHuman())
	{
		return std::max(iValue, GC.getDefineINT("DIPLOMACY_VALUE_REMAINDER"));
	}
	else
	{
		return iValue;
	}
}

/********************************************************************************/
/**		REVOLUTION_MOD							6/9/08				jdog5000	*/
/**																				*/
/**		Revolution AI															*/
/********************************************************************************/
int CvTeamAI::AI_minorKeepWarVal(TeamTypes eTeam) const
{
	int iValue = 0;

	if( (isMinorCiv() && !isRebel()) || getAnyWarPlanCount(true) == 0 )
	{
		if( AI_hasCitiesInPrimaryArea(eTeam) )
		{
			bool bIsGetBetterUnits = false;
			bool bFinancialTrouble = false;
			bool bAggressive = GC.getGame().isOption(GAMEOPTION_AGGRESSIVE_AI);
			int iDaggerCount = 0;
			for( int iJ = 0; iJ < MAX_PLAYERS; iJ++)
			{
				if (GET_PLAYER((PlayerTypes)iJ).getTeam() == getID())
				{
					if (GET_PLAYER((PlayerTypes)iJ).AI_isDoStrategy(AI_STRATEGY_GET_BETTER_UNITS))
					{
						bIsGetBetterUnits = true;
					}
					if (GET_PLAYER((PlayerTypes)iJ).AI_isDoStrategy(AI_STRATEGY_DAGGER))
					{
						iDaggerCount++;
						bAggressive = true;
					}
					if (GET_PLAYER((PlayerTypes)iJ).AI_isFinancialTrouble())
					{
						bFinancialTrouble = true;
						break;
					}
				}
			}

			if( !bFinancialTrouble )
			{
				int iPower = getPower(true);
				if( bAggressive && !bIsGetBetterUnits )
				{
					iPower *= 4;
					iPower /= 3;
				}

				if( GET_TEAM(eTeam).AI_getWarSuccess(getID()) > GC.getDefineINT("WAR_SUCCESS_CITY_CAPTURING") || GC.getGameINLINE().getSorenRandNum(AI_maxWarRand()/100, "Keep war on minor") == 0 )
				{
					if (GET_TEAM(eTeam).getDefensivePower() < ((iPower * AI_maxWarNearbyPowerRatio()) / 100))
					{
						int iNoWarRoll = GC.getGameINLINE().getSorenRandNum(100, "AI No War") - 20;
						iNoWarRoll += (bAggressive ? 10 : 0);
						iNoWarRoll += ((AI_getWarSuccess(eTeam) > GC.getDefineINT("WAR_SUCCESS_CITY_CAPTURING")) ? 10 : 0);
						iNoWarRoll -= (bIsGetBetterUnits ? 15 : 0);
						iNoWarRoll = range(iNoWarRoll, 0, 99);

						if( iNoWarRoll >= AI_noWarAttitudeProb(AI_getAttitude(eTeam)) )
						{
							if( AI_teamCloseness(eTeam) > 0 )
							{
								iValue = AI_startWarVal(eTeam);
							}
						}
					}
				}
			}
		}
	}

	return iValue;
}

int CvTeamAI::AI_getBarbarianCivWarVal(TeamTypes eTeam, int iMaxDistance) const
{
	int iValue = 0;

	if( GET_TEAM(eTeam).isAlive() && !GET_TEAM(eTeam).isMinorCiv() && !GET_TEAM(eTeam).isBarbarian() )
	{
		if( AI_hasCitiesInPrimaryArea(eTeam) )
		{
			int iClosenessValue = AI_teamCloseness(eTeam, iMaxDistance);
			if (iClosenessValue <= 0)
			{
				return 0;
			}
			iValue += iClosenessValue;

			iValue += AI_calculatePlotWarValue(eTeam)/2;

			iValue += (3 * AI_calculateCapitalProximity(eTeam)) / 2;

			iValue += 3*AI_getWarSuccess(eTeam);

			if( GET_TEAM(eTeam).getDefensivePower() > ((3*getPower(true))/2*AI_maxWarNearbyPowerRatio())/100 )
			{
				iValue /= 2;
			}
			else if( GET_TEAM(eTeam).getDefensivePower() < (getPower(true)*AI_maxWarNearbyPowerRatio())/100 )
			{
				iValue *= 2;
			}

			switch (AI_getAttitude(eTeam))
			{
			case ATTITUDE_FURIOUS:
				iValue *= 16;
				break;

			case ATTITUDE_ANNOYED:
				iValue *= 8;
				break;

			case ATTITUDE_CAUTIOUS:
				iValue *= 4;
				break;

			case ATTITUDE_PLEASED:
				iValue *= 1;
				break;

			case ATTITUDE_FRIENDLY:
				iValue *= 0;
				break;

			default:
				FAssert(false);
				break;
			}
		}
	}

	return iValue;
}
/********************************************************************************/
/**		REVOLUTION_MOD							END								*/
/********************************************************************************/


int CvTeamAI::AI_techTradeVal(TechTypes eTech, TeamTypes eTeam) const
{
	FAssert(eTeam != getID());
	int iKnownCount;
	int iPossibleKnownCount;
	int iCost;
	int iValue;
	int iI;

	iCost = std::max(0, (getResearchCost(eTech) - getResearchProgress(eTech)));

	//CD Tweaks - scale down with era - ideally this would be done based on average science modifier etc. In the late game we probably have large modifiers, and techs are too expensive to trade for gold anyway
	//iValue = ((iCost * 3) / 2);
	iValue = ((iCost * 3 * GC.getNumEraInfos()) / (2 * GC.getNumEraInfos() + GET_PLAYER(getLeaderID()).getCurrentEra()));

	iKnownCount = 0;
	iPossibleKnownCount = 0;

	for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
	{
		if (GET_TEAM((TeamTypes)iI).isAlive())
		{
			if (iI != getID())
			{
				if (isHasMet((TeamTypes)iI))
				{
					if (GET_TEAM((TeamTypes)iI).isHasTech(eTech))
					{
						iKnownCount++;
					}

					iPossibleKnownCount++;
				}
			}
		}
	}

	iValue += (((iCost / 2) * (iPossibleKnownCount - iKnownCount)) / iPossibleKnownCount);

	iValue *= std::max(0, (GC.getTechInfo(eTech).getAITradeModifier() + 100));
	iValue /= 100;

	iValue -= (iValue % GC.getDefineINT("DIPLOMACY_VALUE_REMAINDER"));

	if (isHuman())
	{
		return std::max(iValue, GC.getDefineINT("DIPLOMACY_VALUE_REMAINDER"));
	}
	else
	{
		return iValue;
	}
}


DenialTypes CvTeamAI::AI_techTrade(TechTypes eTech, TeamTypes eTeam) const
{
	PROFILE_FUNC();

	AttitudeTypes eAttitude;
	int iNoTechTradeThreshold;
	int iTechTradeKnownPercent;
	int iKnownCount;
	int iPossibleKnownCount;
	int iI, iJ;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");


	/************************************************************************************************/
	/* Afforess                      Start         01/14/10                                               */
	/*                                                                                              */
	/*                                                                                              */
	/************************************************************************************************/
	TechTypes eCurrentResearch = NO_TECH;  //CD Tweaks - bug fix
	TeamTypes eStrongestTeam = (TeamTypes)0;
	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
		{
			if (GET_TEAM((GET_PLAYER((PlayerTypes)iI)).getTeam()).getPower(true) > GET_TEAM(eStrongestTeam).getPower(true))
			{
				eCurrentResearch = GET_PLAYER((PlayerTypes)iI).getCurrentResearch();
				eStrongestTeam = GET_PLAYER((PlayerTypes)iI).getTeam();
			}
		}
	}


	CvTeam& kTeam = GET_TEAM(eTeam);  //CD tweaks - moved from inside if
	if (GC.getGameINLINE().isOption(GAMEOPTION_NO_TECH_BROKERING))
	{            
		if (!kTeam.isHasTech(eTech))
		{
			if (!kTeam.isHuman())
			{
				if ((eTech == eCurrentResearch) || (GC.getGameINLINE().isOption(GAMEOPTION_NO_TECH_DIFFUSION)))
				{
					if (2 * kTeam.getResearchProgress(eTech) > kTeam.getResearchCost(eTech))
					{
						return DENIAL_NO_GAIN;
					}
				}
			}
		}
	}

	//CD Tweaks - let's not trade for techs we are near to completing ourselves even without tech brokering off
	//need to be near completion both as a percentage (adjusted down if we are recieving tech diffusion) and an absolute number of beakers
	//if (!kTeam.isHasTech(eTech) && !kTeam.isHuman())
	//{
	//	int iNearAmount = GC.getTechInfo(eTech).getEra() + 1;
	//	iNearAmount *= GC.getWorldInfo(GC.getMapINLINE().getWorldSize()).getResearchPercent();
	//	iNearAmount *= GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getResearchPercent();
	//	iNearAmount /= 100;
	//	iNearAmount *= GC.getEraInfo(GC.getGameINLINE().getStartEra()).getResearchPercent();
	//	iNearAmount /= 100;
	//	int iNearPercent = 80;
	//	if ( GC.getDefineINT("TECH_DIFFUSION_ENABLE") )
	//	{
	//		int iTechScorePercent = kTeam.getBestKnownTechScorePercent();
	//		int iWelfareThreshold = GC.getTECH_DIFFUSION_WELFARE_THRESHOLD();
	//		if (iTechScorePercent < iWelfareThreshold)
	//		{
	//			//Decrease near percent and increase near amount
	//			iNearPercent = std::max(50, iNearPercent + iTechScorePercent - iWelfareThreshold);
	//			iNearAmount *= 100 + iWelfareThreshold - iTechScorePercent;
	//			iNearAmount /= 100;
	//			iNearAmount += (isHuman()) ? 100 : 0;
	//		}
	//	}
	//	int iTotalCost = kTeam.getResearchCost(eTech);
	//	int iProgress = kTeam.getResearchProgress(eTech);
	//	if ((100 * iProgress > iTotalCost * iNearPercent) && (iTotalCost - iProgress < iNearAmount))
	//	{
	//		return DENIAL_NO_GAIN;
	//	}
	//}
	//CD Tweaks End

	/************************************************************************************************/
	/* Afforess                         END                                                            */
	/************************************************************************************************/ 

	if (isHuman())
	{
		return NO_DENIAL;
	}

	if (isVassal(eTeam))
	{
		return NO_DENIAL;
	}

	if (isAtWar(eTeam))
	{
		return NO_DENIAL;
	}

	if (AI_getWorstEnemy() == eTeam)
	{
		return DENIAL_WORST_ENEMY;
	}

	eAttitude = AI_getAttitude(eTeam);

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				if (eAttitude <= GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getTechRefuseAttitudeThreshold())
				{
					/************************************************************************************************/
					/* Afforess	                  Start		 02/19/10                                               */
					/*                                                                                              */
					/* Ruthless AI: Attitude is irrelevant                                                          */
					/************************************************************************************************/
					if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
					{
						if (eAttitude > ATTITUDE_FURIOUS)
							continue;
					}
					/************************************************************************************************/
					/* Afforess	                     END                                                            */
					/************************************************************************************************/
					return DENIAL_ATTITUDE;
				}
			}
		}
	}

	/************************************************************************************************/
	/* Afforess	                  Start		 03/19/10                                               */
	/*                                                                                              */
	/* Ruthless AI: Don't Sell Our Military Secrets                                                 */
	/************************************************************************************************/
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	{
		if (GC.getTechInfo(eTech).getFlavorValue(GC.getInfoTypeForString("FLAVOR_MILITARY")) > 3)
		{
			//We don't want to spread military techs when we are gearing for war
			//If there is tech brokering, selling the tech to anyone could get it in the hands of our enemy. If there is no brokering, just worry about the current team.
			//CD Tweaks - also consider close by teams we aren't friendly with as potential targets, we are being ruthless after all. 
			if (getAnyWarPlanCount(true) > 0 && (!GC.getGameINLINE().isOption(GAMEOPTION_NO_TECH_BROKERING) || AI_getWarPlan(eTeam) != NO_WARPLAN || 
				(AI_teamCloseness(eTeam) > 0 && eAttitude < ATTITUDE_FRIENDLY)))
			{
				return DENIAL_TECH_MONOPOLY;	//CD Tweaks - less likely to give the game away
			}
		}
	}
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/

	if (eAttitude < ATTITUDE_FRIENDLY)
	{
		if ((GC.getGameINLINE().getTeamRank(getID()) < (GC.getGameINLINE().countCivTeamsEverAlive() / 2)) ||
			(GC.getGameINLINE().getTeamRank(eTeam) < (GC.getGameINLINE().countCivTeamsEverAlive() / 2)))
		{
			iNoTechTradeThreshold = AI_noTechTradeThreshold();

			iNoTechTradeThreshold *= std::max(0, (GC.getHandicapInfo(GET_TEAM(eTeam).getHandicapType()).getNoTechTradeModifier() + 100));
			iNoTechTradeThreshold /= 100;

			if (AI_getMemoryCount(eTeam, MEMORY_RECEIVED_TECH_FROM_ANY) > iNoTechTradeThreshold)
			{
				return DENIAL_TECH_WHORE;
			}
		}

		iKnownCount = 0;
		iPossibleKnownCount = 0;

		for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
		{
			if (GET_TEAM((TeamTypes)iI).isAlive())
			{
				if ((iI != getID()) && (iI != eTeam))
				{
					if (isHasMet((TeamTypes)iI))
					{
						if (GET_TEAM((TeamTypes)iI).isHasTech(eTech))
						{
							iKnownCount++;
						}

						iPossibleKnownCount++;
					}
				}
			}
		}

		iTechTradeKnownPercent = AI_techTradeKnownPercent();

		iTechTradeKnownPercent *= std::max(0, (GC.getHandicapInfo(GET_TEAM(eTeam).getHandicapType()).getTechTradeKnownModifier() + 100));
		iTechTradeKnownPercent /= 100;

		iTechTradeKnownPercent *= AI_getTechMonopolyValue(eTech, eTeam);
		iTechTradeKnownPercent /= 100;

		//CD Tweaks - if we are backwards then lets trade what we do have to offer in order to catch up, not too much if we are getting tech diffusion through
		int iMinReduction = 0;
		if (GC.getDefineINT("TECH_DIFFUSION_ENABLE")) 
		{
			iMinReduction = GC.getTECH_DIFFUSION_WELFARE_THRESHOLD() - (200/GC.getTECH_DIFFUSION_WELFARE_MODIFIER());
		}
		iTechTradeKnownPercent *= std::max(iMinReduction, getBestKnownTechScorePercent());
		iTechTradeKnownPercent /= 100;

		//if there is no tech brokering be slightly more liberal with tech trading
		if (GC.getGameINLINE().isOption(GAMEOPTION_NO_TECH_BROKERING))
		{
			iTechTradeKnownPercent *= 90;
			iTechTradeKnownPercent /= 100;
		}
		//CD Tweaks End

		if ((iPossibleKnownCount > 0) ? (((iKnownCount * 100) / iPossibleKnownCount) < iTechTradeKnownPercent) : (iTechTradeKnownPercent > 0))
		{
			return DENIAL_TECH_MONOPOLY;
		}
	}

	for (iI = 0; iI < GC.getNumUnitInfos(); iI++)
	{
		if (isTechRequiredForUnit(eTech, ((UnitTypes)iI)))
		{
			if (isWorldUnitClass((UnitClassTypes)(GC.getUnitInfo((UnitTypes)iI).getUnitClassType())))
			{
				if (getUnitClassMaking((UnitClassTypes)(GC.getUnitInfo((UnitTypes)iI).getUnitClassType())) > 0)
				{
					return DENIAL_MYSTERY;
				}
			}
		}
	}

	for (iI = 0; iI < GC.getNumBuildingInfos(); iI++)
	{
		if (isTechRequiredForBuilding(eTech, ((BuildingTypes)iI)))
		{
			if (isWorldWonderClass((BuildingClassTypes)(GC.getBuildingInfo((BuildingTypes)iI).getBuildingClassType())))
			{
				if (getBuildingClassMaking((BuildingClassTypes)(GC.getBuildingInfo((BuildingTypes)iI).getBuildingClassType())) > 0)
				{
					return DENIAL_MYSTERY;
				}
			}
		}
	}

	for (iI = 0; iI < GC.getNumProjectInfos(); iI++)
	{
		if (GC.getProjectInfo((ProjectTypes)iI).getTechPrereq() == eTech)
		{
			if (isWorldProject((ProjectTypes)iI))
			{
				if (getProjectMaking((ProjectTypes)iI) > 0)
				{
					return DENIAL_MYSTERY;
				}
			}

			for (iJ = 0; iJ < GC.getNumVictoryInfos(); iJ++)
			{
				if (GC.getGameINLINE().isVictoryValid((VictoryTypes)iJ))
				{
					if (GC.getProjectInfo((ProjectTypes)iI).getVictoryThreshold((VictoryTypes)iJ))
					{
						return DENIAL_VICTORY;
					}
				}
			}
		}
	}

	return NO_DENIAL;
}


int CvTeamAI::AI_mapTradeVal(TeamTypes eTeam) const
{
	CvPlot* pLoopPlot;
	int iValue;
	int iI;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	iValue = 0;

	for (iI = 0; iI < GC.getMapINLINE().numPlotsINLINE(); iI++)
	{
		pLoopPlot = GC.getMapINLINE().plotByIndexINLINE(iI);

		if (!(pLoopPlot->isRevealed(getID(), false)) && pLoopPlot->isRevealed(eTeam, false))
		{
			if (pLoopPlot->isWater())
			{
				iValue++;
			}
			else
			{
				iValue += 5;
			}
		}
	}

	iValue /= 10;
	/************************************************************************************************/
	/* Afforess	                  Start		 03/19/10                                               */
	/*                                                                                              */
	/* Ruthless AI                                                                                  */
	/************************************************************************************************/
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	{
		//Planning war against the team, we need their map!
		if (AI_getWarPlan(eTeam) != NO_WARPLAN)
		{
			iValue *= 6;	//CD Tweaks - was 15
		}
		//we should check to see if their map covers the team(s) we are gearing for war with
		else if (getAnyWarPlanCount(true) > 0)
		{
			for (iI = 0; iI < MAX_TEAMS; iI++)
			{
				if (GET_TEAM((TeamTypes)iI).isAlive())
				{
					if (GET_TEAM((TeamTypes)iI).getID() != getID())
					{
						//victim
						if (AI_getWarPlan((TeamTypes)iI) != NO_WARPLAN)
						{
							//if they are friends of the victim, they probably shared maps
							if (GET_TEAM(eTeam).AI_getAttitudeVal((TeamTypes)iI) > 5)
							{
								iValue *= 2;	//CD Tweaks - was 5
								break;
							}
						}
					}
				}
			}
		}
	}
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/
	if (GET_TEAM(eTeam).isVassal(getID()))
	{
		iValue /= 2;
	}

	iValue -= (iValue % GC.getDefineINT("DIPLOMACY_VALUE_REMAINDER"));

	if (isHuman())
	{
		return std::max(iValue, GC.getDefineINT("DIPLOMACY_VALUE_REMAINDER"));
	}
	else
	{
		return iValue;
	}
}


DenialTypes CvTeamAI::AI_mapTrade(TeamTypes eTeam) const
{
	PROFILE_FUNC();

	AttitudeTypes eAttitude;
	int iI;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	if (isHuman())
	{
		return NO_DENIAL;
	}

	if (isVassal(eTeam))
	{
		return NO_DENIAL;
	}

	if (isAtWar(eTeam))
	{
		return NO_DENIAL;
	}

	if (AI_getWorstEnemy() == eTeam)
	{
		return DENIAL_WORST_ENEMY;
	}
	/************************************************************************************************/
	/* Afforess	                  Start		 03/30/10                                               */
	/*                                                                                              */
	/* Ruthless AI: Selling Maps right before we go to war is stupid                                */
	/************************************************************************************************/
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	{
		if (AI_getWarPlan(eTeam) != NO_WARPLAN)
		{
			return DENIAL_MYSTERY;
		}
		//we should check to see if their map covers the team(s) we are gearing for war with
		if (getAnyWarPlanCount(true) > 0)
		{
			for (iI = 0; iI < MAX_TEAMS; iI++)
			{
				if (GET_TEAM((TeamTypes)iI).isAlive())
				{
					if (GET_TEAM((TeamTypes)iI).getID() != getID())
					{
						//victim
						if (AI_getWarPlan((TeamTypes)iI) != NO_WARPLAN)
						{
							//Friends of out victim will sell them their maps, so don't trade our secrets to them
							if (GET_TEAM(eTeam).AI_getAttitudeVal((TeamTypes)iI) > 5)
							{
								return DENIAL_NO_GAIN;
							}
						}
					}
				}
			}
		}
	}
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/
	eAttitude = AI_getAttitude(eTeam);

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				if (eAttitude <= GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getMapRefuseAttitudeThreshold())
				{
					return DENIAL_ATTITUDE;
				}
			}
		}
	}

	return NO_DENIAL;
}


int CvTeamAI::AI_vassalTradeVal(TeamTypes eTeam) const
{
	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	return AI_surrenderTradeVal(eTeam);
}


DenialTypes CvTeamAI::AI_vassalTrade(TeamTypes eTeam) const
{
	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	CvTeamAI& kMasterTeam = GET_TEAM(eTeam);

	for (int iLoopTeam = 0; iLoopTeam < MAX_TEAMS; iLoopTeam++)
	{
		CvTeam& kLoopTeam = GET_TEAM((TeamTypes)iLoopTeam);
		if (kLoopTeam.isAlive() && iLoopTeam != getID() && iLoopTeam != kMasterTeam.getID())
		{
			if (!kLoopTeam.isAtWar(kMasterTeam.getID()) && kLoopTeam.isAtWar(getID()))
			{
				if (kMasterTeam.isForcePeace((TeamTypes)iLoopTeam) || !kMasterTeam.canChangeWarPeace((TeamTypes)iLoopTeam))
				{
					if (!kLoopTeam.isAVassal())
					{
						return DENIAL_WAR_NOT_POSSIBLE_YOU;
					}
				}

				if (!kMasterTeam.isHuman())
				{
					DenialTypes eWarDenial = kMasterTeam.AI_declareWarTrade((TeamTypes)iLoopTeam, getID(), true);
					if (NO_DENIAL != eWarDenial)
					{
						return DENIAL_WAR_NOT_POSSIBLE_YOU;
					}
				}
			}
			else if (kLoopTeam.isAtWar(kMasterTeam.getID()) && !kLoopTeam.isAtWar(getID()))
			{
				if (!kMasterTeam.canChangeWarPeace((TeamTypes)iLoopTeam))
				{
					if (!kLoopTeam.isAVassal())
					{
						return DENIAL_PEACE_NOT_POSSIBLE_YOU;
					}
				}

				if (!kMasterTeam.isHuman())
				{
					DenialTypes ePeaceDenial = kMasterTeam.AI_makePeaceTrade((TeamTypes)iLoopTeam, getID());
					if (NO_DENIAL != ePeaceDenial)
					{
						return DENIAL_PEACE_NOT_POSSIBLE_YOU;
					}
				}
			}
		}
	}

	return AI_surrenderTrade(eTeam);
}


int CvTeamAI::AI_surrenderTradeVal(TeamTypes eTeam) const
{
	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	return 0;
}


DenialTypes CvTeamAI::AI_surrenderTrade(TeamTypes eTeam, int iPowerMultiplier) const
{
	PROFILE_FUNC();

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	CvTeam& kMasterTeam = GET_TEAM(eTeam);
	bool bAlone = true;	//CD Tweaks - are we alone - set to false if we share primary area with another team (only set/used if we aren't at war with eTeam)
	bool bMasterBordersEnemy = false; //CD Tweaks - does the proposed master border someone we're at war with (only set/used if we aren't at war with eTeam)

	for (int iLoopTeam = 0; iLoopTeam < MAX_TEAMS; iLoopTeam++)
	{
		CvTeam& kLoopTeam = GET_TEAM((TeamTypes)iLoopTeam);
		if (kLoopTeam.isAlive() && iLoopTeam != getID() && iLoopTeam != kMasterTeam.getID())
		{
			if (kLoopTeam.isAtWar(kMasterTeam.getID()) && !kLoopTeam.isAtWar(getID()))
			{
				if (isForcePeace((TeamTypes)iLoopTeam) || !canChangeWarPeace((TeamTypes)iLoopTeam))
				{
					if (!kLoopTeam.isAVassal())
					{
						return DENIAL_WAR_NOT_POSSIBLE_US;
					}
				}
			}
			//CD Tweaks - only check peace not possible if capitulating. Otherwise store if proposed master borders one of our enemies
			else if (kLoopTeam.isAtWar(getID()))
			{
				if (isAtWar(eTeam))
				{
					if (!kLoopTeam.isAtWar(eTeam))
					{
						if (!canChangeWarPeace((TeamTypes)iLoopTeam))
						{
							if (!kLoopTeam.isAVassal())
							{
								return DENIAL_PEACE_NOT_POSSIBLE_US;
							}
						}
					}
				}
				else if (!bMasterBordersEnemy)
				{
					bMasterBordersEnemy = GET_TEAM(eTeam).AI_isLandTarget((TeamTypes)iLoopTeam);
				}
			}
			//CD Tweaks End
		}
	}

	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                      12/07/09                                jdog5000      */
	/*                                                                                              */
	/* Diplomacy                                                                                    */
	/************************************************************************************************/
	if (isHuman() && kMasterTeam.isHuman())
	{
		return NO_DENIAL;
	}
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                       END                                                  */
	/************************************************************************************************/

	int iAttitudeModifier = 0;

	if (!GET_TEAM(eTeam).isParent(getID()))
	{
		int iPersonalityModifier = 0;
		int iMembers = 0;
		for (int iI = 0; iI < MAX_PLAYERS; iI++)
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
				{
					iPersonalityModifier += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getVassalPowerModifier();
					++iMembers;
				}
			}
		}

		/************************************************************************************************/
		/* REVOLUTION_MOD                         06/03/09                                jdog5000      */
		/*                                                                                              */
		/* War Strategy AI                                                                              */
		/************************************************************************************************/
		/* original BTS code
		int iTotalPower = GC.getGameINLINE().countTotalCivPower();
		int iAveragePower = iTotalPower / std::max(1, GC.getGameINLINE().countCivTeamsAlive());
		*/

		int iTotalPower = 0;
		int iNumNonVassals = 0;
		for (int iI = 0; iI < MAX_CIV_TEAMS; iI++)
		{
			CvTeam& kTeam = GET_TEAM((TeamTypes) iI);
			if ( kTeam.isAlive() )
			{
				if( kTeam.isAVassal() && kTeam.isCapitulated() )
				{
					// Count capitulated vassals as a fractional add to their master's power
					iTotalPower += (2*kTeam.getPower(false))/5;
				}
				else if( !(kTeam.isSingleCityTeam()) && !(kTeam.isMinorCiv()) && !(kTeam.isRebel()) )
				{
					iTotalPower += kTeam.getPower(false);
					iNumNonVassals++;
				}
			}
		}
		int iAveragePower = iTotalPower / std::max(1, iNumNonVassals);
		/************************************************************************************************/
		/* REVOLUTION_MOD                       END                                                     */
		/************************************************************************************************/
		int iMasterPower = GET_TEAM(eTeam).getPower(false);
		int iVassalPower = (getPower(true) * (iPowerMultiplier + iPersonalityModifier / std::max(1, iMembers))) / 100;

		if (isAtWar(eTeam))
		{
			int iTheirSuccess = std::max(10, GET_TEAM(eTeam).AI_getWarSuccess(getID()));
			int iOurSuccess = std::max(10, AI_getWarSuccess(eTeam));
			int iOthersBestSuccess = 0;
			for (int iTeam = 0; iTeam < MAX_CIV_TEAMS; ++iTeam)
			{
				if (iTeam != eTeam && iTeam != getID())
				{
					CvTeam& kLoopTeam = GET_TEAM((TeamTypes)iTeam);

					if (kLoopTeam.isAlive() && kLoopTeam.isAtWar(getID()))
					{
						int iSuccess = kLoopTeam.AI_getWarSuccess(getID());
						if (iSuccess > iOthersBestSuccess)
						{
							iOthersBestSuccess = iSuccess;
						}
					}
				}
			}

			// Discourage capitulation to a team that has not done the most damage
			if (iTheirSuccess < iOthersBestSuccess)
			{
				iOurSuccess += iOthersBestSuccess - iTheirSuccess;
			}

			iMasterPower = (2 * iMasterPower * iTheirSuccess) / (iTheirSuccess + iOurSuccess);

			if (AI_getWorstEnemy() == eTeam)
			{
				iMasterPower *= 3;
				iMasterPower /= 4;
			}
		}
		else
		{
			/************************************************************************************************/
			/* BETTER_BTS_AI_MOD                      06/12/10                                jdog5000      */
			/*                                                                                              */
			/* Diplomacy AI                                                                                 */
			/************************************************************************************************/
			//CD Tweaks - Civs that aren't near to our enemies and aren't near us are counted as weak
			if (!bMasterBordersEnemy && !GET_TEAM(eTeam).AI_isLandTarget(getID()))
			{
				iMasterPower /= 2;
			}
			//CD Tweaks End
			/************************************************************************************************/
			/* BETTER_BTS_AI_MOD                       END                                                  */
			/************************************************************************************************/
		}

		//CD Tweaks - move from lower down for speed, simplicity
		if (3 * iVassalPower > 2 * iMasterPower)
		{
			return DENIAL_POWER_US;
		}
		//CD Tweaks End

		for (int iLoopTeam = 0; iLoopTeam < MAX_CIV_TEAMS; iLoopTeam++)
		{
			if (iLoopTeam != getID() && iLoopTeam != eTeam)	//CD Tweaks, moved iLoopTeam != eTeam here for speed, simplicity
			{
				CvTeamAI& kLoopTeam = GET_TEAM((TeamTypes)iLoopTeam);
				if (kLoopTeam.isAlive())
				{
					//CD Tweaks - are we alone
					if (bAlone && (AI_hasCitiesInPrimaryArea((TeamTypes)iLoopTeam) || kLoopTeam.AI_hasCitiesInPrimaryArea(getID())))
					{
						bAlone = false;
					}
					//CD Tweaks End

					if (kLoopTeam.AI_isLandTarget(getID()))
					{
						if (kLoopTeam.getPower(true) > getPower(true))
						{
							if (kLoopTeam.isAtWar(eTeam) && !kLoopTeam.isAtWar(getID()))
							{
								return DENIAL_POWER_YOUR_ENEMIES;
							}
							//CD Tweaks
							else if (kLoopTeam.isAtWar(getID()))
							{
								//if we're losing badly enough that we would capitulate to them, look to vassalise to save ourselves
								if( (AI_getWarSuccess((TeamTypes)iLoopTeam) + std::min(getNumCities(), 4) * GC.getWAR_SUCCESS_CITY_CAPTURING()) < kLoopTeam.AI_getWarSuccess(getID()))
								{
									iAttitudeModifier++;
									if (AI_getWorstEnemy() == (TeamTypes)iLoopTeam)
									{
										iAttitudeModifier++;
									}
								}
							}
							//CD Tweaks end

							iAveragePower = (2 * iAveragePower * kLoopTeam.getPower(true)) / std::max(1, kLoopTeam.getPower(true) + getPower(true));

							iAttitudeModifier += (3 * kLoopTeam.getPower(true)) / std::max(1, getPower(true)) - 2;
						}

						if (!kLoopTeam.isAtWar(eTeam) && kLoopTeam.isAtWar(getID()))
						{
							iAveragePower = (iAveragePower * (getPower(true) + GET_TEAM(eTeam).getPower(false))) / std::max(1, getPower(true));
						}
					}

					if (!atWar(getID(), eTeam))
					{
						if (kLoopTeam.isAtWar(eTeam) && !kLoopTeam.isAtWar(getID()))
						{
							DenialTypes eDenial = AI_declareWarTrade((TeamTypes)iLoopTeam, eTeam, false);
							if (eDenial != NO_DENIAL)
							{
								return eDenial;
							}
						}
					}
				}
			}
		}

		if (!isVassal(eTeam) && canVassalRevolt(eTeam))
		{
			return DENIAL_POWER_US;
		}

		/************************************************************************************************/
		/* BETTER_BTS_AI_MOD                      06/02/09                                jdog5000      */
		/*                                                                                              */
		/* War Strategy AI                                                                              */
		/************************************************************************************************/
		/* original bts code
		if (iVassalPower > iAveragePower || 3 * iVassalPower > 2 * iMasterPower)
		*/
		if (5*iVassalPower > 4*iAveragePower)
			/************************************************************************************************/
				/* BETTER_BTS_AI_MOD                       END                                                  */
					/************************************************************************************************/
		{
			return DENIAL_POWER_US;
		}

		for (int i = 0; i < GC.getNumVictoryInfos(); i++)
		{
			bool bPopulationThreat = true;
			if (GC.getGameINLINE().getAdjustedPopulationPercent((VictoryTypes)i) > 0)
			{
				bPopulationThreat = false;

				int iThreshold = GC.getGameINLINE().getTotalPopulation() * GC.getGameINLINE().getAdjustedPopulationPercent((VictoryTypes)i);
				if (400 * getTotalPopulation(!isAVassal()) > 3 * iThreshold)
				{
					return DENIAL_VICTORY;
				}

				if (!atWar(getID(), eTeam))
				{
					if (400 * (getTotalPopulation(isAVassal()) + GET_TEAM(eTeam).getTotalPopulation()) > 3 * iThreshold)
					{
						bPopulationThreat = true;
					}
				}
			}

			bool bLandThreat = true;
			if (GC.getGameINLINE().getAdjustedLandPercent((VictoryTypes)i) > 0)
			{
				bLandThreat = false;

				int iThreshold = GC.getMapINLINE().getLandPlots() * GC.getGameINLINE().getAdjustedLandPercent((VictoryTypes)i);
				if (400 * getTotalLand(!isAVassal()) > 3 * iThreshold)
				{
					return DENIAL_VICTORY;
				}

				if (!atWar(getID(), eTeam))
				{
					if (400 * (getTotalLand(isAVassal()) + GET_TEAM(eTeam).getTotalLand()) > 3 * iThreshold)
					{
						bLandThreat = true;
					}
				}
			}

			if (GC.getGameINLINE().getAdjustedPopulationPercent((VictoryTypes)i) > 0 || GC.getGameINLINE().getAdjustedLandPercent((VictoryTypes)i) > 0)
			{
				if (bLandThreat && bPopulationThreat)
				{
					return DENIAL_POWER_YOU;
				}
			}
		}
	}

	if (!isAtWar(eTeam))
	{
		if (!GET_TEAM(eTeam).isParent(getID()))
		{
			if (AI_getWorstEnemy() == eTeam)
			{
				return DENIAL_WORST_ENEMY;
			}

			//CD Tweaks - only if we aren't alone or the master doesn't border an enemy
			if (!bAlone && !bMasterBordersEnemy && !AI_hasCitiesInPrimaryArea(eTeam) && AI_calculateAdjacentLandPlots(eTeam) == 0)
				//CD Tweaks End
			{
				return DENIAL_TOO_FAR;
			}
		}

		AttitudeTypes eAttitude = AI_getAttitude(eTeam, false);

		AttitudeTypes eModifiedAttitude = CvPlayerAI::AI_getAttitudeFromValue(AI_getAttitudeVal(eTeam, false) + iAttitudeModifier);

		for (int iI = 0; iI < MAX_PLAYERS; iI++)
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
				{
					if (eAttitude <= ATTITUDE_FURIOUS)
					{
						return DENIAL_ATTITUDE;
					}

					if (eModifiedAttitude <= GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getVassalRefuseAttitudeThreshold())
					{
						return DENIAL_ATTITUDE;
					}
				}
			}
		}
	}
	else
	{
		/************************************************************************************************/
		/* BETTER_BTS_AI_MOD                      12/07/09                                jdog5000      */
		/*                                                                                              */
		/* Diplomacy AI                                                                                 */
		/************************************************************************************************/
		/* original BTS code
		if (AI_getWarSuccess(eTeam) + 4 * GC.getDefineINT("WAR_SUCCESS_CITY_CAPTURING") > GET_TEAM(eTeam).AI_getWarSuccess(getID()))
		{
		return DENIAL_JOKING;
		}
		*/
		// Scale better for small empires, particularly necessary if WAR_SUCCESS_CITY_CAPTURING > 10
		//CD Tweaks - small civs now fight for longer (possibly to the death), but may cave when their last city is surrounded
		int iNeededSuccess = std::min(getNumCities()+1, 4) * GC.getWAR_SUCCESS_CITY_CAPTURING();  //BTS Code was always * 4, BBAI code was * std::min(getNumCities(), 4) wothout the below code
		for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
		{
			if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() && GET_PLAYER((PlayerTypes)iPlayer).getNumCities() == 1 )
			{
				CvCity* pCapital = GET_PLAYER((PlayerTypes)iPlayer).getCapitalCity();
				if (pCapital != NULL && pCapital->AI_isDanger() && !pCapital->AI_isDefended())
				{
					iNeededSuccess -= GC.getWAR_SUCCESS_CITY_CAPTURING();
					break;
				}
			}
		}
		if (AI_getWarSuccess(eTeam) + iNeededSuccess > GET_TEAM(eTeam).AI_getWarSuccess(getID()))
			//CD Tweaks End
		{
			return DENIAL_JOKING;
		}

		if( !kMasterTeam.isHuman() )
		{
			if( !(GET_TEAM(kMasterTeam.getID()).AI_acceptSurrender(getID())) )
			{
				return DENIAL_JOKING;
			}
		}
		/************************************************************************************************/
		/* BETTER_BTS_AI_MOD                       END                                                  */
		/************************************************************************************************/
	}

	return NO_DENIAL;
}

/************************************************************************************************/
/* BETTER_BTS_AI_MOD                      03/20/10                                jdog5000      */
/*                                                                                              */
/* Victory Strategy AI                                                                          */
/************************************************************************************************/
bool CvTeamAI::AI_isAnyMemberDoVictoryStrategy( int iVictoryStrategy ) const
{
	for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
	{
		if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
		{
			if( GET_PLAYER((PlayerTypes)iPlayer).isAlive() )
			{
				if( GET_PLAYER((PlayerTypes)iPlayer).AI_isDoVictoryStrategy(iVictoryStrategy) )
				{
					return true;
				}
			}
		}
	}

	return false;
}

bool CvTeamAI::AI_isAnyMemberDoVictoryStrategyLevel4() const
{
	for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
	{
		if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
		{
			if( GET_PLAYER((PlayerTypes)iPlayer).isAlive() )
			{
				if( GET_PLAYER((PlayerTypes)iPlayer).AI_isDoVictoryStrategyLevel4() )
				{
					return true;
				}
			}
		}
	}

	return false;
}

bool CvTeamAI::AI_isAnyMemberDoVictoryStrategyLevel3() const
{
	for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
	{
		if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
		{
			if( GET_PLAYER((PlayerTypes)iPlayer).isAlive() )
			{
				if( GET_PLAYER((PlayerTypes)iPlayer).AI_isDoVictoryStrategyLevel3() )
				{
					return true;
				}
			}
		}
	}

	return false;
}
/************************************************************************************************/
/* BETTER_BTS_AI_MOD                       END                                                  */
/************************************************************************************************/
/************************************************************************************************/
/* BETTER_BTS_AI_MOD                      03/20/10                                jdog5000      */
/*                                                                                              */
/* War Strategy AI                                                                              */
/************************************************************************************************/
/// \brief Compute how close our wars are to capitulation.
///
/// At -99, this means we're losing on net enough to capitulate.  At +99, we're winning enough
/// to perhaps have another player capitulate to us.
int CvTeamAI::AI_getWarSuccessCapitulationRatio() const
{
	int iSumWarSuccess = 0;

	for( int iI = 0; iI < MAX_CIV_TEAMS; iI++ )
	{
		if( iI != getID() )
		{
			if( GET_TEAM((TeamTypes)iI).isAlive() && isAtWar((TeamTypes)iI) )
			{
				iSumWarSuccess += AI_getWarSuccess((TeamTypes)iI);
				iSumWarSuccess -= GET_TEAM((TeamTypes)iI).AI_getWarSuccess(getID());
			}
		}
	}

	int iDivisor = std::max(25, std::min(getNumCities(), 4) * GC.getWAR_SUCCESS_CITY_CAPTURING());

	iSumWarSuccess = range((100*iSumWarSuccess)/iDivisor, -99, 99);

	return iSumWarSuccess;
}

/// \brief Compute power of enemies as percentage of our power.
/// CD Tweaks - add considersucess - enemies that we are winning a war against will be considered less powerful
/// and vice-versa
int CvTeamAI::AI_getEnemyPowerPercent( bool bConsiderOthers, bool bConsiderSuccess ) const
{
	int iEnemyPower = 0;

	for( int iI = 0; iI < MAX_CIV_TEAMS; iI++ )
	{
		if( iI != getID() )
		{
			if( GET_TEAM((TeamTypes)iI).isAlive() && isHasMet((TeamTypes)iI) )
			{
				if( isAtWar((TeamTypes)iI) )
				{
					int iTempPower = 220 * GET_TEAM((TeamTypes)iI).getPower(false);
					iTempPower /= (AI_hasCitiesInPrimaryArea((TeamTypes)iI) ? 2 : 3);
					iTempPower /= (GET_TEAM((TeamTypes)iI).isMinorCiv() ? 3 : 1);
					iTempPower /= std::max(1, (bConsiderOthers ? GET_TEAM((TeamTypes)iI).getAtWarCount(true,true) : 1));
					//CD Tweaks
					if (bConsiderSuccess)
					{
						int iWeight = range(GET_TEAM((TeamTypes)iI).getNumCities(), 1, 4) * GC.getWAR_SUCCESS_CITY_CAPTURING();
						int iTheirSuccess = GET_TEAM((TeamTypes)iI).AI_getWarSuccess(getID());
						int iOurSuccess =AI_getWarSuccess((TeamTypes)iI);
						iTempPower = (2 * iTempPower * (iTheirSuccess + iWeight)) / (iTheirSuccess + iOurSuccess + iWeight);
					}
					//CD Tweaks end
					iEnemyPower += iTempPower;
				}
				else if( AI_isChosenWar((TeamTypes)iI) )
				{
					// Haven't declared war yet
					int iTempPower = 240 * GET_TEAM((TeamTypes)iI).getDefensivePower();
					iTempPower /= (AI_hasCitiesInPrimaryArea((TeamTypes)iI) ? 2 : 3);
					iTempPower /= 1 + (bConsiderOthers ? GET_TEAM((TeamTypes)iI).getAtWarCount(true,true) : 0);
					iEnemyPower += iTempPower;
				}
			}
		}
	}

	return (iEnemyPower/std::max(1, (isAVassal() ? getCurrentMasterPower(true) : getPower(true))));
}

/// \brief Sum up air power of enemies plus average of other civs we've met.
int CvTeamAI::AI_getRivalAirPower( ) const
{
	// Count enemy air units, not just those visible to us
	int iRivalAirPower = 0;
	int iEnemyAirPower = 0;
	//int iOurAirPower = 0;	//CD Tweaks - our air power
	int iOurPower = getPower(true);  //CD Tweaks, used in loop

	for (int iI = 0; iI < GC.getNumUnitInfos(); iI++)
	{
		CvUnitInfo& kUnitInfo = GC.getUnitInfo((UnitTypes)iI);

		if( kUnitInfo.getDomainType() == DOMAIN_AIR ) 
		{
			if( kUnitInfo.getAirCombat() > 0 )
			{
				for( int iTeam = 0; iTeam < MAX_CIV_TEAMS; iTeam++ )
				{
					if( (GET_TEAM((TeamTypes)iTeam).isAlive() && isHasMet((TeamTypes)iTeam) ) || (iTeam == getID()))
					{
						int iUnitPower = GET_TEAM((TeamTypes)iTeam).getUnitClassCount((UnitClassTypes)kUnitInfo.getUnitClassType());

						if( iUnitPower > 0 )
						{
							iUnitPower *= kUnitInfo.getPowerValue();

							//CD Tweaks - paraniod AI, treat more powerful nearby civs as your enemy
							if( (AI_getWarPlan((TeamTypes)iTeam) == NO_WARPLAN) && (GET_TEAM((TeamTypes)iTeam).getPower(true) < iOurPower || !AI_isAllyLandTarget((TeamTypes)iTeam) ))
							{
								iRivalAirPower += iUnitPower;
							}
							else
							{
								iEnemyAirPower += iUnitPower;
							}
						}
					}
				}
			}
		}
	}

	//CD Tweaks - scale enemy power slightly with world size, smaller = larger value
	//using this value (yeah slight hack) increase power by 5% for map sizes below huge
	iEnemyAirPower *= (80 +  GC.getWorldInfo(GC.getMapINLINE().getWorldSize()).getNumCitiesMaintenancePercent());
	iEnemyAirPower /= 100;
	//CD Tweaks End

	return (iEnemyAirPower + (iRivalAirPower / std::max(1,getHasMetCivCount(true))));
}

//CD Tweaks - does any member have the technology and resources to build nukes. Does not check whether nukes are banned or if the
//manhattan project has been built - rather it is used to evaluate those things
bool CvTeamAI::AI_anyMemberNukeReady() const
{
	for (UnitTypes i = (UnitTypes)0; i < GC.getNumUnitInfos(); i = (UnitTypes)(i+1))
	{
		const CvUnitInfo& kLoopUnit = GC.getUnitInfo(i);
		if (kLoopUnit.getNukeRange() >= 0 && kLoopUnit.getProductionCost() >= 0)
		{
			bool hasTechs = isHasTech((TechTypes)(kLoopUnit.getPrereqAndTech()));
			if (hasTechs)
			{
				for (int k = 0; k < GC.getNUM_UNIT_AND_TECH_PREREQS(); k++)
				{
					TechTypes ePrereqTech = (TechTypes)kLoopUnit.getPrereqAndTechs(k);
					if (ePrereqTech != NO_TECH && !isHasTech(ePrereqTech))
					{
						hasTechs = false;
					}
				}							
			}
			if (hasTechs)
			{
				for (PlayerTypes j = (PlayerTypes)0; j < MAX_CIV_PLAYERS; j = (PlayerTypes)(j+1))
				{
					const CvPlayer& kLoopPlayer = GET_PLAYER(j);
					if (kLoopPlayer.isAlive() && GC.getCivilizationInfo(kLoopPlayer.getCivilizationType()).getCivilizationUnits(kLoopUnit.getUnitClassType()) == i &&
						(kLoopPlayer.getTeam() == getID()))
					{
						// resource prereq.
						BonusTypes ePrereqBonus = (BonusTypes)kLoopUnit.getPrereqAndBonus();
						if (ePrereqBonus == NO_BONUS || kLoopPlayer.hasBonus(ePrereqBonus) || kLoopPlayer.countOwnedBonuses(ePrereqBonus) > 0)
						{
							return true;
						}
					}
				}
			}
		}
	}
	return false;
}


bool CvTeamAI::AI_acceptSurrender( TeamTypes eSurrenderTeam )
{
	PROFILE_FUNC();

	if( isHuman() )
	{
		return true;
	}

	if( !isAtWar(eSurrenderTeam) )
	{
		return true;
	}

	if( GET_TEAM(eSurrenderTeam).AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_SPACE3) )
	{
		// Capturing capital or Apollo city will stop space
		return false;
	}

	if( GET_TEAM(eSurrenderTeam).AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CULTURE3) )
	{
		// Capturing top culture cities will stop culture
		return false;
	}

	//CD Tweaks - this check is now in it's own function
	if (AI_isLikelyToCapitulateToOther(eSurrenderTeam))
	{
		return true;		
	}
	//CD Tweaks End

	int iValuableCities = 0;
	int iCitiesThreatenedByUs = 0;
	int iValuableCitiesThreatenedByUs = 0;
	int iCitiesThreatenedByOthers = 0;

	CvCity* pLoopCity;
	int iLoop;

	for (int iI = 0; iI < MAX_CIV_PLAYERS; iI++)
	{
		if( GET_PLAYER((PlayerTypes)iI).getTeam() == eSurrenderTeam && GET_PLAYER((PlayerTypes)iI).isAlive() )
		{
			for (pLoopCity = GET_PLAYER((PlayerTypes)iI).firstCity(&iLoop); pLoopCity != NULL; pLoopCity = GET_PLAYER((PlayerTypes)iI).nextCity(&iLoop))
			{
				bool bValuable = false;

				if( pLoopCity->isHolyCity() )
				{
					bValuable = true;
				}
				else if( pLoopCity->isHeadquarters() )
				{
					bValuable = true;
				}
				else if( pLoopCity->hasActiveWorldWonder() )
				{
					bValuable = true;
				}
				else if( AI_isPrimaryArea(pLoopCity->area()) && (GET_TEAM(eSurrenderTeam).countNumCitiesByArea(pLoopCity->area()) < 3) )
				{
					bValuable = true;
				}
				else if( pLoopCity->isCapital() && (GET_TEAM(eSurrenderTeam).getNumCities() > GET_TEAM(eSurrenderTeam).getNumMembers() || countNumCitiesByArea(pLoopCity->area()) > 0) )
				{
					bValuable = true;
				}
				else
				{
					// Valuable terrain bonuses
					CvPlot* pLoopPlot = NULL;
					for (int iJ = 0; iJ < NUM_CITY_PLOTS; iJ++)
					{
						pLoopPlot = plotCity(pLoopCity->getX_INLINE(), pLoopCity->getY_INLINE(), iJ);

						if (pLoopPlot != NULL)
						{
							BonusTypes eBonus = pLoopPlot->getNonObsoleteBonusType(getID());
							if ( eBonus != NO_BONUS)
							{
								if(GET_PLAYER(getLeaderID()).AI_bonusVal(eBonus) > 15)
								{
									bValuable = true;
									break;
								}
							}
						}
					}
				}

				int iOwnerPower = GET_PLAYER((PlayerTypes)iI).AI_getOurPlotStrength(pLoopCity->plot(), 2, true, false);
				int iOurPower = AI_getOurPlotStrength(pLoopCity->plot(), 2, false, false, true);
				int iOtherPower = GET_PLAYER((PlayerTypes)iI).AI_getEnemyPlotStrength(pLoopCity->plot(), 2, false, false) - iOurPower;

				if( iOtherPower > iOwnerPower )
				{
					iCitiesThreatenedByOthers++;
				}

				if (iOurPower > iOwnerPower)
				{
					iCitiesThreatenedByUs++;
					if( bValuable )
					{
						iValuableCities++;
						iValuableCitiesThreatenedByUs++;
						continue;
					}
				}

				if( bValuable && pLoopCity->getHighestPopulation() < 5 )
				{
					bValuable = false;
				}

				if( bValuable )
				{
					if( AI_isPrimaryArea(pLoopCity->area()) )
					{
						iValuableCities++;
					}
					else
					{
						for (int iJ = 0; iJ < MAX_PLAYERS; iJ++)
						{
							if (GET_PLAYER((PlayerTypes)iJ).isAlive())
							{
								if (GET_PLAYER((PlayerTypes)iJ).getTeam() == getID())
								{
									if( pLoopCity->AI_playerCloseness((PlayerTypes)iJ) > 5 )
									{
										iValuableCities++;
										break;
									}
								}
							}
						}
					}
				}
			}
		}
	}

	if( iValuableCitiesThreatenedByUs > 0 )
	{
		// Press for capture of valuable city
		return false;
	}

	if( iCitiesThreatenedByOthers > (1 + iCitiesThreatenedByUs/2) )
	{
		// Keep others from capturing spoils, but let it go if surrender civ is too small
		// to care about
		if( 6*(iValuableCities + GET_TEAM(eSurrenderTeam).getNumCities()) > getNumCities() )
		{
			return true;
		}
	}

	// If we're low on the totem poll, accept so enemies don't drag anyone else into war with us
	// Top rank is 0, second is 1, etc.
	int iTeamRank = GC.getGameINLINE().getTeamRank(getID());
	if( iTeamRank > (1 + GC.getGameINLINE().countCivTeamsAlive()/3) )
	{
		return true;
	}

	int iOurWarSuccessRatio = AI_getWarSuccessCapitulationRatio();
	if( iOurWarSuccessRatio < -30 )
	{
		// We're doing badly overall, need to be done with this war and gain an ally
		return true;
	}

	int iWarCount = 0;
	for (int iI = 0; iI < MAX_CIV_TEAMS; iI++)
	{
		if (GET_TEAM((TeamTypes)iI).isAlive() && !(GET_TEAM((TeamTypes)iI).isMinorCiv()))
		{
			if ((TeamTypes)iI != eSurrenderTeam && !(GET_TEAM((TeamTypes)iI).isVassal(eSurrenderTeam)))
			{
				if (isAtWar((TeamTypes)iI))
				{
					if( GET_TEAM((TeamTypes)iI).AI_getWarSuccess(getID()) > 5*GC.getDefineINT("WAR_SUCCESS_ATTACKING") )
					{
						iWarCount++;
					}
				}
			}
		}
	}

	if( iWarCount > 0 && iOurWarSuccessRatio < 50 )
	{
		// Accept if we have other wars to fight
		return true;
	}

	// War weariness
	int iWearinessThreshold = (GC.getGameINLINE().isOption(GAMEOPTION_AGGRESSIVE_AI) ? 300 : 240);
	iWearinessThreshold += 10*iValuableCities + 20*iCitiesThreatenedByUs;

	for (int iI = 0; iI < MAX_CIV_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID() )
			{
				int iWarWearinessPercentAnger = (getWarWeariness(eSurrenderTeam) * std::max(0, 100 + GET_TEAM(eSurrenderTeam).getEnemyWarWearinessModifier())) / 10000;
				iWarWearinessPercentAnger = GET_PLAYER((PlayerTypes)iI).getModifiedWarWearinessPercentAnger(iWarWearinessPercentAnger);

				// Significant war weariness from eSurrenderTeam, 1000 = 100%
				if( iWarWearinessPercentAnger > 50 )
				{
					if( GET_PLAYER((PlayerTypes)iI).getWarWearinessPercentAnger() > iWearinessThreshold )
					{
						return true;
					}
				}
			}
		}
	}

	if( (iValuableCities + iCitiesThreatenedByUs) >= (AI_maxWarRand()/100) )
	{
		// Continue conquest
		return false;
	}

	if( GET_TEAM(eSurrenderTeam).getNumCities() < (getNumCities()/4 - (AI_maxWarRand()/100)) )
	{
		// Too small to bother leaving alive
		return false;
	}

	return true;
}

void CvTeamAI::AI_getWarRands( int &iMaxWarRand, int &iLimitedWarRand, int &iDogpileWarRand ) const
{
	iMaxWarRand = AI_maxWarRand();
	iLimitedWarRand = AI_limitedWarRand();
	iDogpileWarRand = AI_dogpileWarRand();

	bool bCult4 = false;
	bool bSpace4 = false;
	bool bCult3 = false;
	bool bFinalWar = false;

	for (int iI = 0; iI < MAX_CIV_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				if (GET_PLAYER((PlayerTypes)iI).AI_isDoStrategy(AI_STRATEGY_FINAL_WAR))
				{
					bFinalWar = true;
				}

				if( GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_CULTURE4))
				{
					bCult4 = true;
				}
				if( GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_CULTURE3))
				{
					bCult3 = true;
				}
				if(GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_SPACE4))
				{
					bSpace4 = true;
				}
			}
		}
	}

	if( bCult4 )
	{
		iMaxWarRand *= 4;
		iLimitedWarRand *= 3;
		iDogpileWarRand *= 2;
	}
	else if( bSpace4 )
	{
		iMaxWarRand *= 3;

		iLimitedWarRand *= 2;

		iDogpileWarRand *= 3;
		iDogpileWarRand /= 2;
	}
	else if( bCult3 )
	{
		iMaxWarRand *= 2;

		iLimitedWarRand *= 3;
		iLimitedWarRand /= 2;

		iDogpileWarRand *= 3;
		iDogpileWarRand /= 2;
	}

	int iNumMembers = getNumMembers();
	int iNumVassals = getVassalCount();

	iMaxWarRand *= (2 + iNumMembers);
	iMaxWarRand /= (2 + iNumMembers + iNumVassals);

	if (bFinalWar)
	{
		iMaxWarRand /= 4;
	}

	iLimitedWarRand *= (2 + iNumMembers);
	iLimitedWarRand /= (2 + iNumMembers + iNumVassals);

	iDogpileWarRand *= (2 + iNumMembers);
	iDogpileWarRand /= (2 + iNumMembers + iNumVassals);
}


void CvTeamAI::AI_getWarThresholds( int &iTotalWarThreshold, int &iLimitedWarThreshold, int &iDogpileWarThreshold ) const
{
	iTotalWarThreshold = 0;
	iLimitedWarThreshold = 0;
	iDogpileWarThreshold = 0;

	int iHighUnitSpendingPercent = 0;
	bool bConq2 = false;
	bool bDom3 = false;
	bool bAggressive = GC.getGameINLINE().isOption(GAMEOPTION_AGGRESSIVE_AI);
	//CD Tweaks - be more aggressive if human is winning, and we've met them, and it's later in the game, and we aren't following another victory strategy
	if (!bAggressive && GET_TEAM(GC.getGameINLINE().getRankTeam(0)).isHuman() && isHasMet(GC.getGameINLINE().getRankTeam(0)) &&
		GET_PLAYER(getLeaderID()).getCurrentEra() >= 3 && !AI_isAnyMemberDoVictoryStrategyLevel4())
	{
		bAggressive = true;
	}
	//CD Tweaks End
	for (int iI = 0; iI < MAX_CIV_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				int iUnitSpendingPercent = (GET_PLAYER((PlayerTypes)iI).calculateUnitCost() * 100) / std::max(1, GET_PLAYER((PlayerTypes)iI).calculatePreInflatedCosts());
				iHighUnitSpendingPercent += (std::max(0, iUnitSpendingPercent - 7) / 2);

				if( GET_PLAYER((PlayerTypes)iI).AI_isDoStrategy(AI_STRATEGY_DAGGER))
				{
					bAggressive = true;
				}
				if( GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_CONQUEST4))
				{
					bAggressive = true;
				}
				if( GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_DOMINATION4))
				{
					bAggressive = true;
				}
				if( GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_CONQUEST2))
				{
					bConq2 = true;
				}
				if(GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_DOMINATION3))
				{
					bDom3 = true;
				}
			}
		}
	}

	// BBAI TODO: Current UU, up aggression?

	iHighUnitSpendingPercent /= std::max(1, getNumMembers());

	iTotalWarThreshold = iHighUnitSpendingPercent * (bAggressive ? 3 : 2);
	if( bDom3 )
	{
		iTotalWarThreshold *= 3;

		iDogpileWarThreshold += 5;
	}
	else if( bConq2 )
	{
		iTotalWarThreshold *= 2;

		iDogpileWarThreshold += 2;
	}
	iTotalWarThreshold /= 3;
	iTotalWarThreshold += bAggressive ? 1 : 0;

	if( bAggressive && GET_PLAYER(getLeaderID()).getCurrentEra() < 3 )
	{
		iLimitedWarThreshold += 2;
	}

	//CD Tweaks - if we're powerfull, don't let the rng stop us conquering weaker nearby civs ASAP!
	//need to be a fair way into the game and not following any non-military victory strategies
	if (GET_PLAYER(getLeaderID()).getCurrentEra() > 0 && getNumCities() > GC.getWorldInfo(GC.getMapINLINE().getWorldSize()).getTargetNumCities() && 
		getAnyWarPlanCount(true) == 0 && getHasMetCivCount(false) > GC.getGameINLINE().countCivPlayersAlive()/4	&& 
		(!AI_isAnyMemberDoVictoryStrategyLevel3() || AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CONQUEST3) || bDom3 ))
	{
		int iMaxDistance = (GET_PLAYER(getLeaderID()).getCurrentEra() * GC.getMapINLINE().maxStepDistance()) / GC.getNumEraInfos();
		int iGoodTargets = 0;
		bool bTopDog = true;
		//increase total war threshold by one for each "good target" and a larger amount if we're more powerful than any other civ we've met
		for (int iTeam = 0; iTeam < MAX_CIV_TEAMS; iTeam++)
		{
			TeamTypes eTeam = (TeamTypes) iTeam;
			if (GET_TEAM(eTeam).isAlive() && iTeam != getID() && isHasMet(eTeam))
			{
				if (canEventuallyDeclareWar(eTeam) && GET_TEAM(eTeam).getNumCities() > 0) 
				{
					if( !GET_TEAM(eTeam).isAVassal() )
					{
						AttitudeTypes eAttitude = AI_getAttitude(eTeam);
						if (AI_noWarAttitudeProb(eAttitude) < 100)
						{
							if (GET_TEAM(eTeam).getNumCities() + GET_TEAM(eTeam).getVassalCount(getID()) > 4)
							{
								if (AI_getClosetCapitalDistance(eTeam, false) <= iMaxDistance)
								{
									int iPowerMultiplier;
									switch (eAttitude)
									{
									case ATTITUDE_FURIOUS:
										iPowerMultiplier = 95;
										break;
									case ATTITUDE_ANNOYED:
										iPowerMultiplier = 85;
										break;
									case ATTITUDE_CAUTIOUS:
										iPowerMultiplier = 75;
										break;
									case ATTITUDE_PLEASED:
										iPowerMultiplier = 50;
										break;
									case ATTITUDE_FRIENDLY:
										iPowerMultiplier = 25;
										break;
									default:
										FAssert(false);
										break;
									}
									if (getPower(false) * iPowerMultiplier > GET_TEAM(eTeam).getDefensivePower() * 100)
									{
										iGoodTargets++;
									}
								}
							}
						}
					}
				}
				if (getPower(false) * 95 < GET_TEAM(eTeam).getPower(false) * 100)
				{
					bTopDog = false;
				}
			}
		}
		if (iGoodTargets > 0)
		{
			iTotalWarThreshold += iGoodTargets;
			if (bTopDog)
			{
				iTotalWarThreshold += (iHighUnitSpendingPercent/3) + 1;
			}
		}
	}
	//CD Tweaks End
}

// Returns odds of player declaring total war times 100
int CvTeamAI::AI_getTotalWarOddsTimes100( ) const
{
	int iTotalWarRand;
	int iLimitedWarRand;
	int iDogpileWarRand;
	AI_getWarRands( iTotalWarRand, iLimitedWarRand, iDogpileWarRand );

	int iTotalWarThreshold;
	int iLimitedWarThreshold;
	int iDogpileWarThreshold;
	AI_getWarThresholds( iTotalWarThreshold, iLimitedWarThreshold, iDogpileWarThreshold );

	return ((100 * 100 * iTotalWarThreshold) / std::max(1, iTotalWarRand));
}
/************************************************************************************************/
/* BETTER_BTS_AI_MOD                       END                                                  */
/************************************************************************************************/

int CvTeamAI::AI_makePeaceTradeVal(TeamTypes ePeaceTeam, TeamTypes eTeam) const
{
	int iModifier;
	int iValue;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");
	FAssertMsg(ePeaceTeam != getID(), "shouldn't call this function on ourselves");
	FAssertMsg(GET_TEAM(ePeaceTeam).isAlive(), "GET_TEAM(ePeaceTeam).isAlive is expected to be true");
	FAssertMsg(atWar(ePeaceTeam, eTeam), "eTeam should be at war with ePeaceTeam");

	iValue = (50 + GC.getGameINLINE().getGameTurn());
	iValue += ((GET_TEAM(eTeam).getNumCities() + GET_TEAM(ePeaceTeam).getNumCities()) * 8);

	iModifier = 0;

	switch ((GET_TEAM(eTeam).AI_getAttitude(ePeaceTeam) + GET_TEAM(ePeaceTeam).AI_getAttitude(eTeam)) / 2)
	{
	case ATTITUDE_FURIOUS:
		iModifier += 400;
		break;

	case ATTITUDE_ANNOYED:
		iModifier += 200;
		break;

	case ATTITUDE_CAUTIOUS:
		iModifier += 100;
		break;

	case ATTITUDE_PLEASED:
		iModifier += 50;
		break;

	case ATTITUDE_FRIENDLY:
		break;

	default:
		FAssert(false);
		break;
	}

	iValue *= std::max(0, (iModifier + 100));
	iValue /= 100;

	iValue *= 40;
	iValue /= (GET_TEAM(eTeam).AI_getAtWarCounter(ePeaceTeam) + 10);

	iValue -= (iValue % GC.getDefineINT("DIPLOMACY_VALUE_REMAINDER"));

	if (isHuman())
	{
		return std::max(iValue, GC.getDefineINT("DIPLOMACY_VALUE_REMAINDER"));
	}
	else
	{
		return iValue;
	}
}


DenialTypes CvTeamAI::AI_makePeaceTrade(TeamTypes ePeaceTeam, TeamTypes eTeam) const
{
	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");
	FAssertMsg(ePeaceTeam != getID(), "shouldn't call this function on ourselves");
	FAssertMsg(GET_TEAM(ePeaceTeam).isAlive(), "GET_TEAM(ePeaceTeam).isAlive is expected to be true");
	FAssertMsg(isAtWar(ePeaceTeam), "should be at war with ePeaceTeam");

	if (GET_TEAM(ePeaceTeam).isHuman())
	{
		return DENIAL_CONTACT_THEM;
	}

	if (GET_TEAM(ePeaceTeam).isAVassal())
	{
		return DENIAL_VASSAL;
	}

	if (isHuman())
	{
		return NO_DENIAL;
	}

	if (!canChangeWarPeace(ePeaceTeam))
	{
		return DENIAL_VASSAL;
	}

	if (AI_endWarVal(ePeaceTeam) > (GET_TEAM(ePeaceTeam).AI_endWarVal(getID()) * 2))
	{
		return DENIAL_CONTACT_THEM;
	}

	int iLandRatio = ((getTotalLand(true) * 100) / std::max(20, GET_TEAM(eTeam).getTotalLand(true)));
	if (iLandRatio > 250)
	{
		return DENIAL_VICTORY;
	}

	return NO_DENIAL;
}


int CvTeamAI::AI_declareWarTradeVal(TeamTypes eWarTeam, TeamTypes eTeam) const
{
	PROFILE_FUNC();

	int iModifier;
	int iValue;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");
	FAssertMsg(eWarTeam != getID(), "shouldn't call this function on ourselves");
	FAssertMsg(GET_TEAM(eWarTeam).isAlive(), "GET_TEAM(eWarTeam).isAlive is expected to be true");
	FAssertMsg(!atWar(eWarTeam, eTeam), "eTeam should be at peace with eWarTeam");

	iValue = 0;
	iValue += (GET_TEAM(eWarTeam).getNumCities() * 10);
	iValue += (GET_TEAM(eWarTeam).getTotalPopulation(true) * 2);
	//CD Tweaks - some account taken of distance to enemy
	iValue += std::max(0, 2*(AI_getClosetCapitalDistance(eWarTeam) - DEFAULT_PLAYER_CLOSENESS - GC.getGameINLINE().getHighestEra()));
	//CD Tweaks End

	iModifier = 0;

	switch (GET_TEAM(eTeam).AI_getAttitude(eWarTeam))
	{
	case ATTITUDE_FURIOUS:
		break;

	case ATTITUDE_ANNOYED:
		iModifier += 25;
		break;

	case ATTITUDE_CAUTIOUS:
		iModifier += 50;
		break;

	case ATTITUDE_PLEASED:
		iModifier += 150;
		break;

	case ATTITUDE_FRIENDLY:
		iModifier += 400;
		break;

	default:
		FAssert(false);
		break;
	}

	iValue *= std::max(0, (iModifier + 100));
	iValue /= 100;

	int iTheirPower = GET_TEAM(eTeam).getPower(true);
	int iWarTeamPower = GET_TEAM(eWarTeam).getPower(true);

	iValue *= 50 + ((100 * iWarTeamPower) / (iTheirPower + iWarTeamPower + 1));
	iValue /= 100;

	if (!(GET_TEAM(eTeam).AI_isAllyLandTarget(eWarTeam)))
	{
		//iValue *= 2;
		//CD Tweaks - less of this if the plots have some strategic value (AI_calculatePlotWarValue now considers nearby coastal cities
		//that could be assaulted
		iValue *= 64;
		iValue /= std::min(64, 32 + AI_calculatePlotWarValue(eWarTeam));
		//CD Tweaks End
	}

	if (!isAtWar(eWarTeam))
	{
		iValue *= 3;
	}
	else
	{
		iValue *= 150;
		iValue /= 100 + ((50 * std::min(100, (100 * AI_getWarSuccess(eWarTeam)) / (8 + getTotalPopulation(false)))) / 100);
	}

	iValue += (GET_TEAM(eTeam).getNumCities() * 20);
	iValue += (GET_TEAM(eTeam).getTotalPopulation(true) * 15);

	if (isAtWar(eWarTeam))
	{
		switch (GET_TEAM(eTeam).AI_getAttitude(getID()))
		{
		case ATTITUDE_FURIOUS:
		case ATTITUDE_ANNOYED:
		case ATTITUDE_CAUTIOUS:
			iValue *= 100;
			break;

		case ATTITUDE_PLEASED:
			iValue *= std::max(75, 100 - getAtWarCount(true) * 10);
			break;

		case ATTITUDE_FRIENDLY:
			iValue *= std::max(50, 100 - getAtWarCount(true) * 20);
			break;

		default:
			FAssert(false);
			break;
		}
		iValue /= 100;
	}

	iValue += GET_TEAM(eWarTeam).getNumNukeUnits() * 250;//Don't want to get nuked
	iValue += GET_TEAM(eTeam).getNumNukeUnits() * 150;//Don't want to use nukes on another's behalf

	if (GET_TEAM(eWarTeam).getAtWarCount(false) == 0)
	{
		iValue *= 2;

		for (int iI = 0; iI < MAX_CIV_TEAMS; iI++)
		{
			if (GET_TEAM((TeamTypes)iI).isAlive())
			{
				if (iI != getID() && iI != eWarTeam && iI != eTeam)
				{
					if (GET_TEAM(eWarTeam).isDefensivePact((TeamTypes)iI))
					{
						iValue += (GET_TEAM((TeamTypes)iI).getNumCities() * 30);
						iValue += (GET_TEAM((TeamTypes)iI).getTotalPopulation(true) * 20);
					}
				}
			}
		}
	}

	iValue *= 60 + (140 * GC.getGameINLINE().getGameTurn()) / std::max(1, GC.getGameINLINE().getEstimateEndTurn());
	iValue /= 100;

	iValue -= (iValue % GC.getDefineINT("DIPLOMACY_VALUE_REMAINDER"));

	if (isHuman())
	{
		return std::max(iValue, GC.getDefineINT("DIPLOMACY_VALUE_REMAINDER"));
	}
	else
	{
		return iValue;
	}
}


DenialTypes CvTeamAI::AI_declareWarTrade(TeamTypes eWarTeam, TeamTypes eTeam, bool bConsiderPower) const
{
	PROFILE_FUNC();

	AttitudeTypes eAttitude;
	AttitudeTypes eAttitudeThem;
	bool bLandTarget;
	int iI;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");
	FAssertMsg(eWarTeam != getID(), "shouldn't call this function on ourselves");
	FAssertMsg(GET_TEAM(eWarTeam).isAlive(), "GET_TEAM(eWarTeam).isAlive is expected to be true");
	FAssertMsg(!isAtWar(eWarTeam), "should be at peace with eWarTeam");

	if (GET_TEAM(eWarTeam).isVassal(eTeam) || GET_TEAM(eWarTeam).isDefensivePact(eTeam))
	{
		return DENIAL_JOKING;
	}

	eAttitude = AI_getAttitude(eTeam);	//CD Tweaks - moved from below
	/************************************************************************************************/
	/* Afforess	                  Start		 04/06/10                                               */
	/*                                                                                              */
	/*  Ruthless AI: Refusing war when we are planning it anyway is silly                           */ 
	/************************************************************************************************/
	//CD Tweaks - makes sense even when not being ruthless, give our plans away to friends, 
	//or if we're ready to attack, or the target is weak
	//if (AI_isChosenWar(eWarTeam) && GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	if (AI_isChosenWar(eWarTeam))
	{
		if	(eAttitude == ATTITUDE_FRIENDLY || !bConsiderPower || getPower(true) > ((120 * GET_TEAM(eWarTeam).getPower(true))/100))
		{
			return NO_DENIAL;
		}
		else
		{
			for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
			{
				if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
				{
					if( GET_PLAYER((PlayerTypes)iPlayer).AI_enemyTargetMissions(eTeam) > 0 )
					{
						return NO_DENIAL;
					}
				}
			}
		}
	}
	//CD Tweaks End		
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/
	if (isHuman())
	{
		return NO_DENIAL;
	}

	if (!canDeclareWar(eWarTeam))
	{
		return DENIAL_VASSAL;
	}

	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                      12/06/09                                jdog5000      */
	/*                                                                                              */
	/* Diplomacy                                                                                    */
	/************************************************************************************************/
	/* original BTS code
	if (getAnyWarPlanCount(true) > 0)
	{
	return DENIAL_TOO_MANY_WARS;
	}
	*/
	if(!GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	{
		//CD Tweaks - consider more wars if our current enemies are weak and we aren't too comitted
		if (getAnyWarPlanCount(true) > 0)
		{
			if (AI_getEnemyPowerPercent() >= 45)
			{
				return DENIAL_TOO_MANY_WARS;
			}
			else
			{
				//if we're involved in sea invasion or are overpowered in one area, we can't afford another war
				CvArea* pLoopArea;
				int iLoop;
				for(pLoopArea = GC.getMapINLINE().firstArea(&iLoop); pLoopArea != NULL; pLoopArea = GC.getMapINLINE().nextArea(&iLoop))
				{
					if (pLoopArea->getAreaAIType(getID()) == AREAAI_DEFENSIVE)
					{
						return DENIAL_TOO_MANY_WARS;
					}
					else if (AI_isPrimaryArea(pLoopArea) && (countNumCitiesByArea(pLoopArea) < pLoopArea->getNumCities() - GET_TEAM(BARBARIAN_TEAM).countNumCitiesByArea(pLoopArea) - 2) &&
						((pLoopArea->getAreaAIType(getID()) == AREAAI_ASSAULT) || (pLoopArea->getAreaAIType(getID()) == AREAAI_ASSAULT_MASSING)))
					{
						return DENIAL_TOO_MANY_WARS;
					}
				}
			}
		}
		//CD Tweaks End
	}
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                       END                                                  */
	/************************************************************************************************/

	if (bConsiderPower)
	{
		bLandTarget = AI_isAllyLandTarget(eWarTeam);

		/* CD Tweaks - better consideration of whether or not this is a bad idea
		if ((GET_TEAM(eWarTeam).getDefensivePower() / ((bLandTarget) ? 2 : 1)) >
		(getPower(true) + ((atWar(eWarTeam, eTeam)) ? GET_TEAM(eTeam).getPower(true) : 0)))*/
		//the below formulas are closer to the calculations of whether the AI would go to war (near, far dogpile) without being asked
		int iDefensivePower = GET_TEAM(eWarTeam).getDefensivePower();
		int iOurPower = (getPower(true) * 3 + getPower(false)) / 4;	//vassal power is counted at 75%
		if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
		{
			//ruthless AI modifies these ratios to prevent declaring war on weaker targets, so let's honour that here
			iOurPower *= bLandTarget ? AI_maxWarNearbyPowerRatio() : AI_maxWarDistantPowerRatio();
			iOurPower /= 100;
		}
		//otherwise only take them into account a little. In doWar our power and enemy power is often modified anyway, depending on type of war
		else if (bLandTarget)
		{
			iOurPower *= 50 + AI_maxWarNearbyPowerRatio();
			iOurPower /= 100;
		}
		else
		{
			iOurPower *= 200 + AI_maxWarDistantPowerRatio();
			iOurPower /= 300;
		}
		iOurPower *= (100 - AI_getEnemyPowerPercent());
		iOurPower /= 100;
		if (atWar(eWarTeam, eTeam))
		{
			iOurPower += (2 * GET_TEAM(eTeam).getPower(true)) / 3;  //other teams power is counted at 66%
		}
		if (iDefensivePower > iOurPower)
			//CD Tweaks End
		{
			if (bLandTarget)
			{
				return DENIAL_POWER_THEM;
			}
			else
			{
				return DENIAL_NO_GAIN;
			}
		}
	}
	/************************************************************************************************/
	/* Afforess	                  Start		 04/06/10                                               */
	/*                                                                                              */
	/* Ruthless AI: Backstab enemies                                                                */
	/************************************************************************************************/
	if ((AI_getMemoryCount(eWarTeam, MEMORY_DECLARED_WAR) > 0) && GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	{
		return NO_DENIAL;
	}
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/
	//CD Tweaks - more likely to join our friends in a war
	if (eAttitude == ATTITUDE_FURIOUS)
	{
		return DENIAL_ATTITUDE;
	}
	else if (!GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI) && (!atWar(eWarTeam, eTeam) || AI_getAttitudeVal(eTeam) <= std::max(7,AI_getAttitudeVal(eWarTeam))))
	{
		for (iI = 0; iI < MAX_PLAYERS; iI++)
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
				{
					if (eAttitude <= GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getDeclareWarRefuseAttitudeThreshold())
					{
						return DENIAL_ATTITUDE;
					}
				}
			}
		}
	}
	//CD Tweaks End

	eAttitudeThem = AI_getAttitude(eWarTeam);

	//CD Tweaks - no need to check if we're furious or already planning a war
	if (eAttitudeThem > ATTITUDE_FURIOUS && !AI_isChosenWar(eWarTeam))
	{
		//CD Tweaks - more likely to join a war against a weaker civ in some circumstances 
		//adjust attitude down. Always consider power for this adjustment
		int iNoWarProb = AI_noWarAttitudeProb(eAttitudeThem);
		if (iNoWarProb < 100 && (getPower(true) + ((atWar(eWarTeam, eTeam)) ? GET_TEAM(eTeam).getPower(true) : 0)) > GET_TEAM(eWarTeam).getDefensivePower())
		{
			int iAdjust = (100 + iNoWarProb - ((100 * getPower(false)) / std::max(GET_TEAM(eWarTeam).getPower(false),1))) / 10;
			iAdjust -= AI_getWarDistrustVal(eWarTeam);
			if (iAdjust < 0)
			{
				eAttitudeThem = GET_PLAYER(getLeaderID()).AI_getAttitudeFromValue(AI_getAttitudeVal(eWarTeam) + iAdjust);
			}
		}
		//CD Tweaks End

		for (iI = 0; iI < MAX_PLAYERS; iI++)
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
				{
					if (eAttitudeThem > GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getDeclareWarThemRefuseAttitudeThreshold())
					{
						return DENIAL_ATTITUDE_THEM;
					}
				}
			}
		}
	}

	if (!atWar(eWarTeam, eTeam))
	{
		if (GET_TEAM(eWarTeam).getNumNukeUnits() > 0)
		{
			return DENIAL_JOKING;
		}
	}

	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                      12/06/09                                jdog5000      */
	/*                                                                                              */
	/* Diplomacy                                                                                    */
	/************************************************************************************************/
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	{
		//CD Tweaks - consider more wars if our current enemies are weak and we aren't too comitted
		if (getAnyWarPlanCount(true) > 0)
		{
			if (AI_getEnemyPowerPercent() >= 45)
			{
				return DENIAL_TOO_MANY_WARS;
			}
			else
			{
				//if we're involved in sea invasion or are overpowered in one area, we can't afford another war
				CvArea* pLoopArea;
				int iLoop;
				for(pLoopArea = GC.getMapINLINE().firstArea(&iLoop); pLoopArea != NULL; pLoopArea = GC.getMapINLINE().nextArea(&iLoop))
				{
					if (pLoopArea->getAreaAIType(getID()) == AREAAI_DEFENSIVE)
					{
						return DENIAL_TOO_MANY_WARS;
					}
					if (AI_isPrimaryArea(pLoopArea) && (countNumCitiesByArea(pLoopArea) < pLoopArea->getNumCities() - GET_TEAM(BARBARIAN_TEAM).countNumCitiesByArea(pLoopArea) - 2) &&
						((pLoopArea->getAreaAIType(getID()) == AREAAI_ASSAULT) || (pLoopArea->getAreaAIType(getID()) == AREAAI_ASSAULT_MASSING)))
					{
						return DENIAL_TOO_MANY_WARS;
					}
				}
			}
		}
		//CD Tweaks End
	}
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                       END                                                  */
	/************************************************************************************************/
	return NO_DENIAL;
}


int CvTeamAI::AI_openBordersTradeVal(TeamTypes eTeam) const
{
	/************************************************************************************************/
	/* Afforess	                  Start		 03/19/10                                               */
	/*                                                                                              */
	/* Ruthless AI                                                                                  */
	/************************************************************************************************/
	//Normal Firaxis calculation
	int iValue;
	iValue = (getNumCities() + GET_TEAM(eTeam).getNumCities());
	/* CD Tweaks - have this value fluctuate gets annoying
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	{
		//if we are planning war, but not against them
		if (AI_getWarPlan(eTeam) == NO_WARPLAN && getAnyWarPlanCount(true) > 0)
		{
			iValue *= 2;
		}
	}*/
	return iValue;
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/
}


DenialTypes CvTeamAI::AI_openBordersTrade(TeamTypes eTeam) const
{
	PROFILE_FUNC();

	AttitudeTypes eAttitude;
	int iI;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	if (isHuman())
	{
		return NO_DENIAL;
	}

	if (isVassal(eTeam))
	{
		return NO_DENIAL;
	}

	if (AI_shareWar(eTeam))
	{
		return NO_DENIAL;
	}

	/************************************************************************************************/
	/* Afforess	                  Start		 03/30/10                                               */
	/*                                                                                              */
	/* Ruthless AI: Get Open Borders with Nearby Allies, reject them with enemies                   */
	/************************************************************************************************/
	//CD Tweaks - always do this
	//if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	//{
	bool bWarplans = getAnyWarPlanCount(true) > 0;
	if (AI_getWarPlan(eTeam) == NO_WARPLAN && bWarplans)
	{
		for (iI = 0; iI < MAX_TEAMS; iI++)
		{
			if (GET_TEAM((TeamTypes)iI).isAlive())
			{
				if (GET_TEAM((TeamTypes)iI).getID() != getID())
				{
					//victim
					if (AI_getWarPlan((TeamTypes)iI) != NO_WARPLAN)
					{
						//Share borders with our enemy
						if (GET_TEAM(eTeam).AI_teamCloseness((TeamTypes)iI) > 0 && !GET_TEAM((TeamTypes)iI).isVassal(eTeam))
						{
							return NO_DENIAL;
						}
					}
				}
			}
		}
	}
	//We are going to attack eTeam soon (only cancel if we are moe powerful, to retain the element of surprise)
	else if ((bWarplans) && (!isOpenBorders(eTeam) || (getPower(true) + GET_TEAM(eTeam).getEnemyPower() > (GET_TEAM(eTeam).getPower(true) * 120) / 100)))
	{
		return DENIAL_MYSTERY;
	}
	//}
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/	

	if (AI_getMemoryCount(eTeam, MEMORY_CANCELLED_OPEN_BORDERS) > 0)
	{
		return DENIAL_RECENT_CANCEL;
	}

	if (AI_getWorstEnemy() == eTeam)
	{
		return DENIAL_WORST_ENEMY;
	}

	//CD Tweaks - get extra info for below checks
	eAttitude = AI_getAttitude(eTeam);
	int iBestTrades = 0;
	int iBestCoastalTrades = 0;
	int iForgeinTradeCities = 0;
	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				if (eAttitude <= GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getOpenBordersRefuseAttitudeThreshold())
				{
					return DENIAL_ATTITUDE;
				}
				else
				{
					iBestTrades = std::max(iBestTrades, GET_PLAYER((PlayerTypes)iI).getTradeRoutes());
					iBestCoastalTrades = std::max(iBestCoastalTrades, GET_PLAYER((PlayerTypes)iI).getCoastalTradeRoutes());
					iForgeinTradeCities += GET_PLAYER((PlayerTypes)iI).countPotentialForeignTradeCitiesConnected();
				}
			}
		}
	}

	//CD Tweaks still don't always open borders, there are some new conditions
	//always open borders with friends
	if (eAttitude = ATTITUDE_FRIENDLY)
	{
		return NO_DENIAL;
	}

	//we are close then open up
	if (AI_teamCloseness(eTeam) > 0)
	{
		return NO_DENIAL;
	}

	//If we share a land area then open borders
	CvArea* pLoopArea = NULL;
	CvTeam& kOtherTeam = GET_TEAM(eTeam);
	int iLoop;
	for(pLoopArea = GC.getMapINLINE().firstArea(&iLoop); pLoopArea != NULL; pLoopArea = GC.getMapINLINE().nextArea(&iLoop))
	{
		if (countNumCitiesByArea(pLoopArea) > 0 && kOtherTeam.countNumCitiesByArea(pLoopArea) > 0)
		{
			return NO_DENIAL;
			break;
		}
	}

	//don't trade with people we're annoyed with, at least until we've got lots of trade routes
	if (AI_getAttitudeVal(eTeam) <= -iBestTrades - iBestCoastalTrades)
	{
		return DENIAL_ATTITUDE;
	}

	//finally, open for trade reasons
	if (isOpenBorders(eTeam) || AI_getWarPlan(eTeam) == NO_WARPLAN)
	{
		//we have we been at peace long enough to trust them (and get full value of trade routes)
		if (AI_getAtPeaceCounter(eTeam) >= GC.getDefineINT("FOREIGN_TRADE_FULL_CREDIT_PEACE_TURNS"))
		{
			return NO_DENIAL;
		}

		//if there's global free trade, we may as well open up to everyone
		if (GC.getGameINLINE().isFreeTrade())
		{
			return NO_DENIAL;
		}

		//we need more forgein cities to trade with
		int iNeededCities = getNumCities() * (iBestTrades + iBestCoastalTrades);
		if (iForgeinTradeCities < iNeededCities)
		{
			return NO_DENIAL;
		}
	}

	return DENIAL_MYSTERY;
	//CD Tweaks end

}


int CvTeamAI::AI_defensivePactTradeVal(TeamTypes eTeam) const
{
	return ((getNumCities() + GET_TEAM(eTeam).getNumCities()) * 3);
}


DenialTypes CvTeamAI::AI_defensivePactTrade(TeamTypes eTeam) const
{
	PROFILE_FUNC();

	AttitudeTypes eAttitude;
	int iI;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	if (isHuman())
	{
		return NO_DENIAL;
	}

	if (GC.getGameINLINE().countCivTeamsAlive() == 2)
	{
		return DENIAL_NO_GAIN;
	}

	if (AI_getWorstEnemy() == eTeam)
	{
		return DENIAL_WORST_ENEMY;
	}

	eAttitude = AI_getAttitude(eTeam);

	//CD Tweaks - reasons to sign a pact on pleased (usually not a good enough attitude)
	if (eAttitude == ATTITUDE_FRIENDLY)
	{
		return NO_DENIAL;	//there are no leaders who refuse at friendly, and there shouldn't be
	}
	else if (eAttitude == ATTITUDE_PLEASED && getAnyWarPlanCount(true) == 0)
	{
		//our gain = their power plus power of our potential enemies they could help with
		int iOurGain = GET_TEAM(eTeam).getPower(true);
		//their gain is our power plus power of their potential enemies
		int iTheirGain = getPower(true);
		for( iI = 0; iI < MAX_CIV_TEAMS; iI++ )
		{
			if (GET_TEAM((TeamTypes)iI).isAlive() && iI != getID() && iI != eTeam && isHasMet((TeamTypes)iI))
			{
				bool bThreatToUs = false;
				if (AI_getMemoryCount((TeamTypes)iI, MEMORY_DECLARED_WAR) > 0 && (GET_TEAM((TeamTypes)iI).getAtWarCount(true) == 0 || GET_TEAM((TeamTypes)iI).AI_getEnemyPowerPercent() < 45))
				{
					bThreatToUs = true;
				}
				else if (AI_teamCloseness((TeamTypes)iI) > 0 && getDefensivePower() < GET_TEAM((TeamTypes)iI).getPower(true))
				{
					if(AI_getAttitude((TeamTypes)iI) <= ATTITUDE_CAUTIOUS || GET_TEAM((TeamTypes)iI).AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CONQUEST4) ||
						GET_TEAM((TeamTypes)iI).AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_DOMINATION4))
					{
						bThreatToUs = true;
					}
				}
				//threats to us and our worst enemys aren't counted as gains for them too, we're only worried about making new enemies
				if (bThreatToUs)
				{
					iOurGain += GET_TEAM((TeamTypes)iI).getPower(true);
				}
				else if (AI_getWorstEnemy() != (TeamTypes)iI)
				{
					//same as above,  but for them
					if (GET_TEAM(eTeam).AI_getMemoryCount((TeamTypes)iI, MEMORY_DECLARED_WAR) > 0 && (GET_TEAM((TeamTypes)iI).getAtWarCount(true) == 0 || GET_TEAM((TeamTypes)iI).AI_getEnemyPowerPercent() < 45))
					{
						iTheirGain += GET_TEAM((TeamTypes)iI).getPower(true);
					}
					else if (GET_TEAM(eTeam).AI_teamCloseness((TeamTypes)iI) > 0 && GET_TEAM(eTeam).getPower(true) < GET_TEAM((TeamTypes)iI).getPower(true))
					{
						if(GET_TEAM((TeamTypes)iI).AI_getAttitude(eTeam) <= ATTITUDE_CAUTIOUS || GET_TEAM((TeamTypes)iI).AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CONQUEST4) ||
							GET_TEAM((TeamTypes)iI).AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_DOMINATION4))
						{
							iTheirGain += GET_TEAM((TeamTypes)iI).getPower(true);
						}
					}
				}
			}
		}
		if (iOurGain > iTheirGain)
		{
			return NO_DENIAL;
		}
	}
	//CD Tweaks

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				if (eAttitude <= GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getDefensivePactRefuseAttitudeThreshold())
				{
					return DENIAL_ATTITUDE;
				}
			}
		}
	}

	return NO_DENIAL;
}


DenialTypes CvTeamAI::AI_permanentAllianceTrade(TeamTypes eTeam) const
{
	PROFILE_FUNC();

	AttitudeTypes eAttitude;
	int iI;

	FAssertMsg(eTeam != getID(), "shouldn't call this function on ourselves");

	if (isHuman())
	{
		return NO_DENIAL;
	}

	if (AI_getWorstEnemy() == eTeam)
	{
		return DENIAL_WORST_ENEMY;
	}

	//CD Tweaks - save power into variables for later use
	int iPower = getPower(true);
	int iTheirPower = GET_TEAM(eTeam).getPower(true);
	if ((iPower + iTheirPower) > (GC.getGameINLINE().countTotalCivPower() / 2))
	{
		if (iPower > iTheirPower)
		{
			return DENIAL_POWER_US;
		}
		else
		{
			return DENIAL_POWER_YOU;
		}
	}

	//Don't ally with much smaller civs
	if (!GET_TEAM(eTeam).canVassalRevolt(getID()) && (!GC.getGameINLINE().isOption(GAMEOPTION_ONE_CITY_CHALLENGE) || !GET_TEAM(eTeam).isHuman()))
	{
		return DENIAL_POWER_US;
	}

	//don't give away our techs
	int iNoTechTradeThreshold = AI_noTechTradeThreshold();

	iNoTechTradeThreshold *= std::max(0, (GC.getHandicapInfo(GET_TEAM(eTeam).getHandicapType()).getNoTechTradeModifier() + 100));
	iNoTechTradeThreshold /= 100;

	int iTechDiff = 0;
	for (int iTech = 0; iTech < GC.getNumTechInfos(); iTech++)
	{
		TechTypes eTech = (TechTypes)iTech;
		if (isHasTech(eTech) && !GET_TEAM(eTeam).isHasTech(eTech))
		{
			iTechDiff++;
		}
		else if (!isHasTech(eTech) && GET_TEAM(eTeam).isHasTech(eTech))
		{
			iTechDiff--;
		}
	}

	if (AI_getMemoryCount(eTeam, MEMORY_RECEIVED_TECH_FROM_ANY) + iTechDiff > iNoTechTradeThreshold)
	{
		if (AI_getMemoryCount(eTeam, MEMORY_RECEIVED_TECH_FROM_ANY) > iTechDiff)
		{
			return DENIAL_TECH_WHORE;
		}
		else
		{
			return DENIAL_NO_GAIN;
		}
	}
	//CD Tweaks end

	if ((AI_getDefensivePactCounter(eTeam) + AI_getShareWarCounter(eTeam)) < 40)
	{
		//CD Tweaks
		//don't worry about shared war or defensive pact too much, but do check some extra conditions
		//return DENIAL_NOT_ALLIED;
		//Don't ally with a weak team
		if (iPower > ((120 * iTheirPower) / 100))
		{
			return DENIAL_POWER_US;
		}

		//very distant civs aren't of much use
		if (!AI_hasCitiesInPrimaryArea(eTeam) && AI_teamCloseness(eTeam) == 0)
		{
			int iMaxDistancePercent = (100 * (getTotalLand(true) + GET_TEAM(eTeam).getTotalLand(true))) / GC.getMapINLINE().getLandPlots();
			iMaxDistancePercent += 100 - AI_calculateWarProximityModifier();
			if (iMaxDistancePercent < 100 && AI_getClosetCapitalDistance(eTeam, true) > (GC.getMapINLINE().maxStepDistance() * iMaxDistancePercent) / 100)
			{
				return DENIAL_TOO_FAR;
			}
		}
		//CD Tweaks end
	}

	eAttitude = AI_getAttitude(eTeam);
	int iAttitudeVal = AI_getAttitudeVal(eTeam);

	//CD Tweaks - for AI always accept if attitude > +7
	if (iAttitudeVal <= 7 || GET_TEAM(eTeam).isHuman())
	{
		for (iI = 0; iI < MAX_PLAYERS; iI++)
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
				{
					if (eAttitude <= GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getPermanentAllianceRefuseAttitudeThreshold())
					{
						return DENIAL_ATTITUDE;
					}
				}
			}
		}
	}

	return NO_DENIAL;
}


TeamTypes CvTeamAI::AI_getWorstEnemy() const
{
	return m_eWorstEnemy;
}


void CvTeamAI::AI_updateWorstEnemy()
{
	PROFILE_FUNC();

	TeamTypes eBestTeam = NO_TEAM;
	int iBestValue = MAX_INT;

	for (int iI = 0; iI < MAX_CIV_TEAMS; iI++)
	{
		TeamTypes eLoopTeam = (TeamTypes) iI;
		CvTeam& kLoopTeam = GET_TEAM(eLoopTeam);
		if (kLoopTeam.isAlive())
		{
			if (iI != getID() && !kLoopTeam.isVassal(getID()))
			{
				if (isHasMet(eLoopTeam))
				{
					if (AI_getAttitude(eLoopTeam) < ATTITUDE_CAUTIOUS)
					{
						//CD Tweaks - don't count peaceweight for worst enemy calculations, only actual transgressions
						//int iValue = AI_getAttitudeVal(eLoopTeam);
						int iValue = AI_getAttitudeVal(eLoopTeam, true, true);
						//CD tweaks End
						/************************************************************************************************/
						/* Afforess	                  Start		 03/19/10                                               */
						/*                                                                                              */
						/* Ruthless AI                                                                                  */
						/************************************************************************************************/
						//Our Worst enemy isn't just the person we hate the most, but the person we hate and is winning!
						if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
						{
							iValue += GC.getGameINLINE().getPlayerRank(kLoopTeam.getLeaderID()) / 2;
							iValue -= GC.getGameINLINE().countCivPlayersAlive() / 2;
						}
						/************************************************************************************************/
						/* Afforess	                     END                                                            */
						/************************************************************************************************/
						if (iValue < iBestValue)
						{
							iBestValue = iValue;
							eBestTeam = eLoopTeam;
						}
						//CD Tweaks - use attitude including peaceweight as a tie-breaker, since it's not included in the main value
						else if (iValue == iBestValue && AI_getAttitudeVal(eLoopTeam, true, false) <  AI_getAttitudeVal(eBestTeam, true, false))
						{
							iBestValue = iValue;
							eBestTeam = eLoopTeam;
						}
					}
				}
			}
		}
	}

	m_eWorstEnemy = eBestTeam;
}


int CvTeamAI::AI_getWarPlanStateCounter(TeamTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiWarPlanStateCounter[eIndex];
}


void CvTeamAI::AI_setWarPlanStateCounter(TeamTypes eIndex, int iNewValue)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	m_aiWarPlanStateCounter[eIndex] = iNewValue;
	FAssert(AI_getWarPlanStateCounter(eIndex) >= 0);
}


void CvTeamAI::AI_changeWarPlanStateCounter(TeamTypes eIndex, int iChange)
{
	AI_setWarPlanStateCounter(eIndex, (AI_getWarPlanStateCounter(eIndex) + iChange));
}


int CvTeamAI::AI_getAtWarCounter(TeamTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiAtWarCounter[eIndex];
}


void CvTeamAI::AI_setAtWarCounter(TeamTypes eIndex, int iNewValue)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	m_aiAtWarCounter[eIndex] = iNewValue;
	FAssert(AI_getAtWarCounter(eIndex) >= 0);
}


void CvTeamAI::AI_changeAtWarCounter(TeamTypes eIndex, int iChange)
{
	AI_setAtWarCounter(eIndex, (AI_getAtWarCounter(eIndex) + iChange));
}


int CvTeamAI::AI_getAtPeaceCounter(TeamTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiAtPeaceCounter[eIndex];
}


void CvTeamAI::AI_setAtPeaceCounter(TeamTypes eIndex, int iNewValue)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	m_aiAtPeaceCounter[eIndex] = iNewValue;
	FAssert(AI_getAtPeaceCounter(eIndex) >= 0);
}


void CvTeamAI::AI_changeAtPeaceCounter(TeamTypes eIndex, int iChange)
{
	AI_setAtPeaceCounter(eIndex, (AI_getAtPeaceCounter(eIndex) + iChange));
}


int CvTeamAI::AI_getHasMetCounter(TeamTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiHasMetCounter[eIndex];
}


void CvTeamAI::AI_setHasMetCounter(TeamTypes eIndex, int iNewValue)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	m_aiHasMetCounter[eIndex] = iNewValue;
	FAssert(AI_getHasMetCounter(eIndex) >= 0);
}


void CvTeamAI::AI_changeHasMetCounter(TeamTypes eIndex, int iChange)
{
	AI_setHasMetCounter(eIndex, (AI_getHasMetCounter(eIndex) + iChange));
}


int CvTeamAI::AI_getOpenBordersCounter(TeamTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiOpenBordersCounter[eIndex];
}


void CvTeamAI::AI_setOpenBordersCounter(TeamTypes eIndex, int iNewValue)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	m_aiOpenBordersCounter[eIndex] = iNewValue;
	FAssert(AI_getOpenBordersCounter(eIndex) >= 0);
}


void CvTeamAI::AI_changeOpenBordersCounter(TeamTypes eIndex, int iChange)
{
	AI_setOpenBordersCounter(eIndex, (AI_getOpenBordersCounter(eIndex) + iChange));
}


int CvTeamAI::AI_getDefensivePactCounter(TeamTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiDefensivePactCounter[eIndex];
}


void CvTeamAI::AI_setDefensivePactCounter(TeamTypes eIndex, int iNewValue)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	m_aiDefensivePactCounter[eIndex] = iNewValue;
	FAssert(AI_getDefensivePactCounter(eIndex) >= 0);
}


void CvTeamAI::AI_changeDefensivePactCounter(TeamTypes eIndex, int iChange)
{
	AI_setDefensivePactCounter(eIndex, (AI_getDefensivePactCounter(eIndex) + iChange));
}


int CvTeamAI::AI_getShareWarCounter(TeamTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiShareWarCounter[eIndex];
}


void CvTeamAI::AI_setShareWarCounter(TeamTypes eIndex, int iNewValue)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	m_aiShareWarCounter[eIndex] = iNewValue;
	FAssert(AI_getShareWarCounter(eIndex) >= 0);
}


void CvTeamAI::AI_changeShareWarCounter(TeamTypes eIndex, int iChange)
{
	AI_setShareWarCounter(eIndex, (AI_getShareWarCounter(eIndex) + iChange));
}


int CvTeamAI::AI_getWarSuccess(TeamTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiWarSuccess[eIndex];
}


void CvTeamAI::AI_setWarSuccess(TeamTypes eIndex, int iNewValue)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                      09/03/09                       poyuzhe & jdog5000     */
	/*                                                                                              */
	/* Efficiency                                                                                   */
	/************************************************************************************************/
	// From Sanguo Mod Performance, ie the CAR Mod
	// Attitude cache
	if (m_aiWarSuccess[eIndex] != iNewValue)
	{
		for (int iI = 0; iI < MAX_PLAYERS; iI++)
		{
			if( GET_PLAYER((PlayerTypes)iI).isAlive() )
			{
				if( GET_PLAYER((PlayerTypes)iI).getTeam() == getID() || GET_PLAYER((PlayerTypes)iI).getTeam() == eIndex )
				{
					for (int iJ = 0; iJ < MAX_PLAYERS; iJ++)
					{
						if( GET_PLAYER((PlayerTypes)iJ).isAlive() && GET_PLAYER((PlayerTypes)iJ).getTeam() != GET_PLAYER((PlayerTypes)iI).getTeam() )
						{
							if( GET_PLAYER((PlayerTypes)iJ).getTeam() == getID() || GET_PLAYER((PlayerTypes)iJ).getTeam() == eIndex )
							{
								GET_PLAYER((PlayerTypes)iJ).AI_invalidateAttitudeCache((PlayerTypes)iI);
								GET_PLAYER((PlayerTypes)iI).AI_invalidateAttitudeCache((PlayerTypes)iJ);
							}
						}
					}
				}
			}
		}
	}
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                       END                                                  */
	/************************************************************************************************/
	m_aiWarSuccess[eIndex] = iNewValue;
	FAssert(AI_getWarSuccess(eIndex) >= 0);
}


void CvTeamAI::AI_changeWarSuccess(TeamTypes eIndex, int iChange)
{
	/************************************************************************************************/
	/* REVOLUTION_MOD                         05/30/08                                jdog5000      */
	/*                                                                                              */
	/* Revolution AI                                                                                */
	/************************************************************************************************/
	// Rebels celebrate successes more
	if( isRebelAgainst(eIndex) )
	{
		iChange *= 3;
		iChange /= 2;
	}
	/************************************************************************************************/
	/* REVOLUTION_MOD                          END                                                  */
	/************************************************************************************************/
	AI_setWarSuccess(eIndex, (AI_getWarSuccess(eIndex) + iChange));
}


int CvTeamAI::AI_getEnemyPeacetimeTradeValue(TeamTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiEnemyPeacetimeTradeValue[eIndex];
}


void CvTeamAI::AI_setEnemyPeacetimeTradeValue(TeamTypes eIndex, int iNewValue)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	m_aiEnemyPeacetimeTradeValue[eIndex] = iNewValue;
	FAssert(AI_getEnemyPeacetimeTradeValue(eIndex) >= 0);
}


void CvTeamAI::AI_changeEnemyPeacetimeTradeValue(TeamTypes eIndex, int iChange)
{
	AI_setEnemyPeacetimeTradeValue(eIndex, (AI_getEnemyPeacetimeTradeValue(eIndex) + iChange));
}


int CvTeamAI::AI_getEnemyPeacetimeGrantValue(TeamTypes eIndex) const
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	return m_aiEnemyPeacetimeGrantValue[eIndex];
}


void CvTeamAI::AI_setEnemyPeacetimeGrantValue(TeamTypes eIndex, int iNewValue)
{
	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");
	m_aiEnemyPeacetimeGrantValue[eIndex] = iNewValue;
	FAssert(AI_getEnemyPeacetimeGrantValue(eIndex) >= 0);
}


void CvTeamAI::AI_changeEnemyPeacetimeGrantValue(TeamTypes eIndex, int iChange)
{
	AI_setEnemyPeacetimeGrantValue(eIndex, (AI_getEnemyPeacetimeGrantValue(eIndex) + iChange));
}


WarPlanTypes CvTeamAI::AI_getWarPlan(TeamTypes eIndex) const
{
	FAssert(eIndex >= 0);
	FAssert(eIndex < MAX_TEAMS);
	FAssert(eIndex != getID() || m_aeWarPlan[eIndex] == NO_WARPLAN);
	return m_aeWarPlan[eIndex];
}


bool CvTeamAI::AI_isChosenWar(TeamTypes eIndex) const
{
	switch (AI_getWarPlan(eIndex))
	{
	case WARPLAN_ATTACKED_RECENT:
	case WARPLAN_ATTACKED:
		return false;
		break;
	case WARPLAN_PREPARING_LIMITED:
	case WARPLAN_PREPARING_TOTAL:
	case WARPLAN_LIMITED:
	case WARPLAN_TOTAL:
	case WARPLAN_DOGPILE:
		return true;
		break;
	}

	return false;
}


bool CvTeamAI::AI_isSneakAttackPreparing(TeamTypes eIndex) const
{
	return ((AI_getWarPlan(eIndex) == WARPLAN_PREPARING_LIMITED) || (AI_getWarPlan(eIndex) == WARPLAN_PREPARING_TOTAL));
}


bool CvTeamAI::AI_isSneakAttackReady(TeamTypes eIndex) const
{
	return (AI_isChosenWar(eIndex) && !(AI_isSneakAttackPreparing(eIndex)));
}


void CvTeamAI::AI_setWarPlan(TeamTypes eIndex, WarPlanTypes eNewValue, bool bWar)
{
	int iI;

	FAssertMsg(eIndex >= 0, "eIndex is expected to be non-negative (invalid Index)");
	FAssertMsg(eIndex < MAX_TEAMS, "eIndex is expected to be within maximum bounds (invalid Index)");

	if (AI_getWarPlan(eIndex) != eNewValue)
	{
		if (bWar || !isAtWar(eIndex))
		{
			m_aeWarPlan[eIndex] = eNewValue;

			AI_setWarPlanStateCounter(eIndex, 0);

			AI_updateAreaStragies();

			for (iI = 0; iI < MAX_PLAYERS; iI++)
			{
				if (GET_PLAYER((PlayerTypes)iI).isAlive())
				{
					if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
					{
						if (!(GET_PLAYER((PlayerTypes)iI).isHuman()))
						{
							GET_PLAYER((PlayerTypes)iI).AI_makeProductionDirty();
						}
					}
				}
			}
		}
	}
}

//if this number is over 0 the teams are "close"
//this may be expensive to run, kinda O(N^2)...
int CvTeamAI::AI_teamCloseness(TeamTypes eIndex, int iMaxDistance) const
{
	PROFILE_FUNC();
	int iI, iJ;

	if (iMaxDistance == -1)
	{
		iMaxDistance = DEFAULT_PLAYER_CLOSENESS;
	}

	FAssert(eIndex != getID());
	int iValue = 0;
	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				for (iJ = 0; iJ < MAX_PLAYERS; iJ++)
				{
					if (GET_PLAYER((PlayerTypes)iJ).isAlive())
					{
						if (GET_PLAYER((PlayerTypes)iJ).getTeam() == eIndex)
						{
							iValue += GET_PLAYER((PlayerTypes)iI).AI_playerCloseness((PlayerTypes)iJ, iMaxDistance);
						}
					}
				}	
			}
		}
	}

	return iValue;	
}


void CvTeamAI::read(FDataStreamBase* pStream)
{
	CvTeam::read(pStream);

	uint uiFlag=0;
	pStream->Read(&uiFlag);	// flags for expansion

	pStream->Read(MAX_TEAMS, m_aiWarPlanStateCounter);
	pStream->Read(MAX_TEAMS, m_aiAtWarCounter);
	pStream->Read(MAX_TEAMS, m_aiAtPeaceCounter);
	pStream->Read(MAX_TEAMS, m_aiHasMetCounter);
	pStream->Read(MAX_TEAMS, m_aiOpenBordersCounter);
	pStream->Read(MAX_TEAMS, m_aiDefensivePactCounter);
	pStream->Read(MAX_TEAMS, m_aiShareWarCounter);
	pStream->Read(MAX_TEAMS, m_aiWarSuccess);
	pStream->Read(MAX_TEAMS, m_aiEnemyPeacetimeTradeValue);
	pStream->Read(MAX_TEAMS, m_aiEnemyPeacetimeGrantValue);

	pStream->Read(MAX_TEAMS, (int*)m_aeWarPlan);
	pStream->Read((int*)&m_eWorstEnemy);
}


void CvTeamAI::write(FDataStreamBase* pStream)
{
	CvTeam::write(pStream);

	uint uiFlag=0;
	pStream->Write(uiFlag);		// flag for expansion

	pStream->Write(MAX_TEAMS, m_aiWarPlanStateCounter);
	pStream->Write(MAX_TEAMS, m_aiAtWarCounter);
	pStream->Write(MAX_TEAMS, m_aiAtPeaceCounter);
	pStream->Write(MAX_TEAMS, m_aiHasMetCounter);
	pStream->Write(MAX_TEAMS, m_aiOpenBordersCounter);
	pStream->Write(MAX_TEAMS, m_aiDefensivePactCounter);
	pStream->Write(MAX_TEAMS, m_aiShareWarCounter);
	pStream->Write(MAX_TEAMS, m_aiWarSuccess);
	pStream->Write(MAX_TEAMS, m_aiEnemyPeacetimeTradeValue);
	pStream->Write(MAX_TEAMS, m_aiEnemyPeacetimeGrantValue);

	pStream->Write(MAX_TEAMS, (int*)m_aeWarPlan);
	pStream->Write(m_eWorstEnemy);
}

// Protected Functions...

int CvTeamAI::AI_noTechTradeThreshold() const
{
	int iRand;
	int iCount;
	int iI;

	iRand = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iRand += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getNoTechTradeThreshold();
				iCount++;
			}
		}
	}

	if (iCount > 0)
	{
		iRand /= iCount;
	}

	/************************************************************************************************/
	/* Afforess	                  Start		 02/19/10                                               */
	/*                                                                                              */
	/* Ruthless AI: Trade More Techs                                                                */
	/************************************************************************************************/
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
		iRand *= 2;	//CD Tweaks - was * 3
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/

	return iRand;
}


int CvTeamAI::AI_techTradeKnownPercent() const
{
	int iRand;
	int iCount;
	int iI;

	iRand = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iRand += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getTechTradeKnownPercent();
				iCount++;
			}
		}
	}

	if (iCount > 0)
	{
		iRand /= iCount;
	}
	/************************************************************************************************/
	/* Afforess	                  Start		 02/19/10                                               */
	/*                                                                                              */
	/* Ruthless AI: Trade More Techs, even techs that others haven't discovered                     */
	/************************************************************************************************/
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
		iRand /= 2;	//CD Tweaks - was /3 but there are other ways this is reduced now
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/

	return iRand;
}


int CvTeamAI::AI_maxWarRand() const
{
	int iRand;
	int iCount;
	int iI;

	iRand = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iRand += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getMaxWarRand();
				iCount++;
			}
		}
	}

	if (iCount > 0)
	{
		iRand /= iCount;
	}

	return iRand;
}


int CvTeamAI::AI_maxWarNearbyPowerRatio() const
{
	int iRand;
	int iCount;
	int iI;

	iRand = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iRand += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getMaxWarNearbyPowerRatio();
				iCount++;
			}
		}
	}

	if (iCount > 1)
	{
		iRand /= iCount;
	}
	/************************************************************************************************/
	/* Afforess	                  Start		 02/19/10                                               */
	/*                                                                                              */
	/* Ruthless AI: Attack Weaker, Closer targets                                                   */
	/************************************************************************************************/
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	{
		iRand /= 2;
	}
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/	


	return iRand;
}


int CvTeamAI::AI_maxWarDistantPowerRatio() const
{
	int iRand;
	int iCount;
	int iI;

	iRand = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iRand += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getMaxWarDistantPowerRatio();
				iCount++;
			}
		}
	}

	if (iCount > 1)
	{
		iRand /= iCount;
	}
	/************************************************************************************************/
	/* Afforess	                  Start		 02/19/10                                               */
	/*                                                                                              */
	/* Ruthless AI: Avoid Far Away targets                                                          */
	/************************************************************************************************/
	//CD Tweaks - also do this if human player is winning and is nearby, he may be looking to pick on players distracted by forgein wars
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	{
		iRand /= 3;
	}
	else if (GET_TEAM(GC.getGameINLINE().getRankTeam(0)).isHuman() && AI_isAllyLandTarget(GC.getGameINLINE().getRankTeam(0)))
	{
		iRand /= 2;
	}
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/

	return iRand;
}


int CvTeamAI::AI_maxWarMinAdjacentLandPercent() const
{
	int iRand;
	int iCount;
	int iI;

	iRand = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iRand += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getMaxWarMinAdjacentLandPercent();
				iCount++;
			}
		}
	}

	if (iCount > 0)
	{
		iRand /= iCount;
	}
	/************************************************************************************************/
	/* Afforess	                  Start		 02/19/10                                               */
	/*                                                                                              */
	/* Ruthless AI: The AI Favors closer targets                                                    */
	/************************************************************************************************/
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
		iRand /= 4;
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/
	return iRand;
}


int CvTeamAI::AI_limitedWarRand() const
{
	int iRand;
	int iCount;
	int iI;

	iRand = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iRand += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getLimitedWarRand();
				iCount++;
			}
		}
	}

	if (iCount > 0)
	{
		iRand /= iCount;
	}

	return iRand;
}


int CvTeamAI::AI_limitedWarPowerRatio() const
{
	int iRand;
	int iCount;
	int iI;

	iRand = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iRand += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getLimitedWarPowerRatio();
				iCount++;
			}
		}
	}

	if (iCount > 0)
	{
		iRand /= iCount;
	}

	return iRand;
}


int CvTeamAI::AI_dogpileWarRand() const
{
	int iRand;
	int iCount;
	int iI;

	iRand = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iRand += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getDogpileWarRand();
				iCount++;
			}
		}
	}

	if (iCount > 0)
	{
		iRand /= iCount;
	}

	return iRand;
}


int CvTeamAI::AI_makePeaceRand() const
{
	int iRand;
	int iCount;
	int iI;

	iRand = 0;
	iCount = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iRand += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getMakePeaceRand();
				iCount++;
			}
		}
	}

	if (iCount > 0)
	{
		iRand /= iCount;
	}

	return iRand;
}

//CD Tweaks - might passed in team capitulate to another team
//moved from AI_acceptSurrender for re-use, and added check that team is small enough to cap
//this doesn't have to be used just to evaluate capitation, even with vassals disabled it can return true
//and is a useful indicator that a civ is being swallowed up by a more powerful enemy
bool CvTeamAI::AI_isLikelyToCapitulateToOther(TeamTypes eSurrenderTeam)
{
	for( int iI = 0; iI < MAX_CIV_TEAMS; iI++ )
	{
		if (GET_TEAM((TeamTypes)iI).isAlive())
		{
			if (iI != getID() && !(GET_TEAM((TeamTypes)iI).isVassal(getID())) )
			{
				if (GET_TEAM(eSurrenderTeam).isAtWar((TeamTypes)iI))
				{
					if (GET_TEAM(eSurrenderTeam).AI_getAtWarCounter((TeamTypes)iI) >= 10)
					{
						if(!GET_TEAM(eSurrenderTeam).canVassalRevolt((TeamTypes)iI))
						{
							if( (GET_TEAM(eSurrenderTeam).AI_getWarSuccess((TeamTypes)iI) + std::min(GET_TEAM(eSurrenderTeam).getNumCities(), 4) * GC.getWAR_SUCCESS_CITY_CAPTURING()) < GET_TEAM((TeamTypes)iI).AI_getWarSuccess(eSurrenderTeam))
							{
								return true;
							}
						}
					}
				}
			}
		}
	}
	return false;
}
//CD Tweaks End

int CvTeamAI::AI_noWarAttitudeProb(AttitudeTypes eAttitude) const
{
	int iProb;
	int iCount;
	int iI;

	iProb = 0;
	iCount = 0;

	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                      03/20/10                                jdog5000      */
	/*                                                                                              */
	/* War Strategy AI                                                                              */
	/************************************************************************************************/
	int iVictoryStrategyAdjust = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				iProb += GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iI).getPersonalityType()).getNoWarAttitudeProb(eAttitude);
				iCount++;

				// In final stages of miltaristic victory, AI may turn on its friends!
				if( GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_CONQUEST4) )
				{
					iVictoryStrategyAdjust += 30;
				}
				else if( GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_DOMINATION4) )
				{
					iVictoryStrategyAdjust += 20;
				}
			}
		}
	}

	if (iCount > 1)
	{
		iProb /= iCount;
		iVictoryStrategyAdjust /= iCount;
	}

	iProb = std::max( 0, iProb - iVictoryStrategyAdjust );
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                       END                                                  */
	/************************************************************************************************/
	/************************************************************************************************/
	/* Afforess	                  Start		 02/19/10                                               */
	/*                                                                                              */
	/* Ruthless AI: Friends are just enemies we haven't made yet.                                   */
	/************************************************************************************************/
	//CD Tweaks - also don't worry about friends quite as much if the human player is winning, concentrate on gaining power
	if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI))
	{
		//For ruthless AI, wait until we've met at least two potential targets before we really start backstabbing
		iProb = std::max(0,std::min(iProb/range(getHasMetCivCount(true),1,5),iProb-20));
	} 
	else if (GET_TEAM(GC.getGameINLINE().getRankTeam(0)).isHuman())
	{	
		iProb = std::max(0, iProb - (isHasMet(GC.getGameINLINE().getRankTeam(0))) ? 20 : 10);
	}
	/************************************************************************************************/
	/* Afforess	                     END                                                            */
	/************************************************************************************************/
	return iProb;
}


void CvTeamAI::AI_doCounter()
{
	int iI;

	for (iI = 0; iI < MAX_TEAMS; iI++)
	{
		if (GET_TEAM((TeamTypes)iI).isAlive())
		{
			if (iI != getID())
			{
				AI_changeWarPlanStateCounter(((TeamTypes)iI), 1);

				if (isAtWar((TeamTypes)iI))
				{
					AI_changeAtWarCounter(((TeamTypes)iI), 1);
				}
				else
				{
					AI_changeAtPeaceCounter(((TeamTypes)iI), 1);
				}

				if (isHasMet((TeamTypes)iI))
				{
					AI_changeHasMetCounter(((TeamTypes)iI), 1);
				}

				if (isOpenBorders((TeamTypes)iI))
				{
					AI_changeOpenBordersCounter(((TeamTypes)iI), 1);
				}

				if (isDefensivePact((TeamTypes)iI))
				{
					AI_changeDefensivePactCounter(((TeamTypes)iI), 1);
				}
				else
				{
					if (AI_getDefensivePactCounter((TeamTypes)iI) > 0)
					{
						AI_changeDefensivePactCounter(((TeamTypes)iI), -1);
					}
				}

				if (isHasMet((TeamTypes)iI))
				{
					if (AI_shareWar((TeamTypes)iI))
					{
						AI_changeShareWarCounter(((TeamTypes)iI), 1);
					}
					//CD Tweaks - this shouldn't last the whole game
					else 
					{
						if (AI_getShareWarCounter((TeamTypes)iI) > 0)
						{
							//if we aren't currently sharing a war, reduce the counter if we're ruthless or in another war
							//that they aren't helping with, or by random chance
							if (GC.getGameINLINE().isOption(GAMEOPTION_RUTHLESS_AI) || getAtWarCount(true) > 0 ||
								GC.getGameINLINE().getSorenRandNum(25, "Share War Counter Decay") == 0)
							{
								AI_changeShareWarCounter(((TeamTypes)iI), -1);
							}
						}
					}
					//CD Tweaks End
				}
			}
		}
	}
}

/************************************************************************************************/
/* BETTER_BTS_AI_MOD                      03/26/10                                jdog5000      */
/*                                                                                              */
/* War Strategy AI                                                                              */
/************************************************************************************************/
// Block AI from declaring war on a distant vassal if it shares an area with the master
bool CvTeamAI::AI_isOkayVassalTarget( TeamTypes eTeam )
{
	if( GET_TEAM(eTeam).isAVassal() )
	{
		if( !(AI_hasCitiesInPrimaryArea(eTeam)) || AI_calculateAdjacentLandPlots(eTeam) == 0 )
		{
			for( int iI = 0; iI < MAX_CIV_TEAMS; iI++ )
			{
				if( GET_TEAM(eTeam).isVassal((TeamTypes)iI) )
				{
					if( AI_hasCitiesInPrimaryArea((TeamTypes)iI) && AI_calculateAdjacentLandPlots((TeamTypes)iI) > 0)
					{
						return false;
					}
				}
			}
		}
	}

	return true;
}
/************************************************************************************************/
/* BETTER_BTS_AI_MOD                       END                                                  */
/************************************************************************************************/


/************************************************************************************************/
/* BETTER_BTS_AI_MOD                      04/25/10                                jdog5000      */
/*                                                                                              */
/* War Strategy, AI logging                                                                     */
/************************************************************************************************/			
/// \brief Make war decisions, mainly for starting or switching war plans.
///
///
void CvTeamAI::AI_doWar()
{
	PROFILE_FUNC();

	CvArea* pLoopArea;
	TeamTypes eBestTeam;
	bool bAreaValid;
	bool bShareValid;
	int iNoWarRoll;
	int iOurPower;
	int iDogpilePower;
	int iValue;
	int iBestValue;
	int iPass;
	int iLoop;
	int iI, iJ;

	FAssert(!isHuman());
	FAssert(!isBarbarian());
	FAssert(!isMinorCiv());

	if (isAVassal())
	{
		return;
	}

	// allow python to handle it
	CyArgsList argsList;
	argsList.add(getID());
	long lResult=0;
	gDLL->getPythonIFace()->callFunction(PYGameModule, "AI_doWar", argsList.makeFunctionArgs(), &lResult);
	if (lResult == 1)
	{
		return;
	}

	int iEnemyPowerPercent = AI_getEnemyPowerPercent(false,true);	//CD tweaks - consider success of current wars

	for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
	{
		if (GET_TEAM((TeamTypes)iI).isAlive() && isHasMet((TeamTypes)iI))
		{
			if (AI_getWarPlan((TeamTypes)iI) != NO_WARPLAN)
			{
				//CD Tweaks - to fix recently introduced bug
				if (!isAtWar((TeamTypes)iI) && !canEventuallyDeclareWar((TeamTypes)iI))
				{
					AI_setWarPlan(((TeamTypes)iI), NO_WARPLAN);
				}
				//CD Tweaks

				int iTimeModifier = 100;
				{
					int iThreshold = (80*AI_maxWarNearbyPowerRatio())/100;

					if( iEnemyPowerPercent < iThreshold )
					{
						iTimeModifier *= iEnemyPowerPercent;
						iTimeModifier /= iThreshold;
					}

					iTimeModifier *= 50 + GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getTrainPercent();
					iTimeModifier /= 150;

					FAssert(iTimeModifier >= 0);
				}

				int iAbandonTimeModifier = 100;
				iAbandonTimeModifier *= 50 + GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getTrainPercent();
				iAbandonTimeModifier /= 150;

				//CD Tweaks - speed up progress/abandonment if the civ is about to capitulate to another civ
				if (AI_isLikelyToCapitulateToOther((TeamTypes)iI))
				{
					iTimeModifier /= 2;
					//if they're close we may still have time to get some spoils if we get involved soon, but not otherwise
					if (AI_teamCloseness((TeamTypes)iI) == 0)
					{
						iAbandonTimeModifier *= std::min(GET_TEAM((TeamTypes)iI).getNumCities(), 2);
						iAbandonTimeModifier /= 3;							
					}
				}
				//intercontinental wars take more prep time
				else if (!AI_hasCitiesInPrimaryArea((TeamTypes)iI))
				{
					iTimeModifier *= 5;
					iTimeModifier /= 4;
					iAbandonTimeModifier *= 5;
					iAbandonTimeModifier /= 4;
				}
				//CD Tweaks End

				bool bEnemyVictoryLevel4 = GET_TEAM((TeamTypes)iI).AI_isAnyMemberDoVictoryStrategyLevel4();

				if (AI_getWarPlan((TeamTypes)iI) == WARPLAN_ATTACKED_RECENT)
				{
					FAssert(isAtWar((TeamTypes)iI));

					if (AI_getAtWarCounter((TeamTypes)iI) > ((GET_TEAM((TeamTypes)iI).AI_isLandTarget(getID())) ? 9 : 3))
					{
						if( gTeamLogLevel >= 1 )
						{
							logBBAI("      Team %d (%S) switching WARPLANS against team %d (%S) from ATTACKED_RECENT to ATTACKED with enemy power percent %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), iI, GET_PLAYER(GET_TEAM((TeamTypes)iI).getLeaderID()).getCivilizationDescription(0), iEnemyPowerPercent );
						}
						AI_setWarPlan(((TeamTypes)iI), WARPLAN_ATTACKED);
					}
				}
				else if (AI_getWarPlan((TeamTypes)iI) == WARPLAN_PREPARING_LIMITED)
				{
					FAssert(canEventuallyDeclareWar((TeamTypes)iI));

					if (AI_getWarPlanStateCounter((TeamTypes)iI) > ((5 * iTimeModifier) / (bEnemyVictoryLevel4 ? 400 : 100)))
					{
						if( gTeamLogLevel >= 1 )
						{
							logBBAI("      Team %d (%S) switching WARPLANS against team %d (%S) from PREPARING_LIMITED to LIMITED after %d turns with enemy power percent %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), iI, GET_PLAYER(GET_TEAM((TeamTypes)iI).getLeaderID()).getCivilizationDescription(0), AI_getWarPlanStateCounter((TeamTypes)iI), iEnemyPowerPercent );
						}
						AI_setWarPlan(((TeamTypes)iI), WARPLAN_LIMITED);
					}
				}
				else if (AI_getWarPlan((TeamTypes)iI) == WARPLAN_LIMITED || AI_getWarPlan((TeamTypes)iI) == WARPLAN_DOGPILE)
				{
					if( !isAtWar((TeamTypes)iI) )
					{
						FAssert(canEventuallyDeclareWar((TeamTypes)iI));

						bool bActive = false;
						for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
						{
							if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
							{
								if( GET_PLAYER((PlayerTypes)iPlayer).AI_enemyTargetMissions((TeamTypes)iI) > 0 )
								{
									bActive = true;
									break;
								}
							}
						}

						if( !bActive )
						{
							if (AI_getWarPlanStateCounter((TeamTypes)iI) > ((15 * iAbandonTimeModifier) / (100)))
							{
								if( gTeamLogLevel >= 1 )
								{
									logBBAI("      Team %d (%S) abandoning WARPLAN_LIMITED or WARPLAN_DOGPILE against team %d (%S) after %d turns with enemy power percent %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), iI, GET_PLAYER(GET_TEAM((TeamTypes)iI).getLeaderID()).getCivilizationDescription(0), AI_getWarPlanStateCounter((TeamTypes)iI), iEnemyPowerPercent );
								}
								AI_setWarPlan(((TeamTypes)iI), NO_WARPLAN);
							}
						}

						if( AI_getWarPlan((TeamTypes)iI) == WARPLAN_DOGPILE )
						{
							if( GET_TEAM((TeamTypes)iI).getAtWarCount(true) == 0 )
							{
								if( gTeamLogLevel >= 1 )
								{
									logBBAI("      Team %d (%S) abandoning WARPLAN_DOGPILE against team %d (%S) after %d turns because enemy has no war with enemy power percent %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), iI, GET_PLAYER(GET_TEAM((TeamTypes)iI).getLeaderID()).getCivilizationDescription(0), AI_getWarPlanStateCounter((TeamTypes)iI), iEnemyPowerPercent );
								}
								AI_setWarPlan(((TeamTypes)iI), NO_WARPLAN);
							}
						}
					}
				}
				else if (AI_getWarPlan((TeamTypes)iI) == WARPLAN_PREPARING_TOTAL)
				{
					FAssert(canEventuallyDeclareWar((TeamTypes)iI));

					if (AI_getWarPlanStateCounter((TeamTypes)iI) > ((10 * iTimeModifier) / (bEnemyVictoryLevel4 ? 400 : 100)))
					{
						bAreaValid = false;
						bShareValid = false;

						for(pLoopArea = GC.getMapINLINE().firstArea(&iLoop); pLoopArea != NULL; pLoopArea = GC.getMapINLINE().nextArea(&iLoop))
						{
							if (AI_isPrimaryArea(pLoopArea))
							{
								if (GET_TEAM((TeamTypes)iI).countNumCitiesByArea(pLoopArea) > 0)
								{
									bShareValid = true;

									AreaAITypes eAreaAI = AI_calculateAreaAIType(pLoopArea, true);

									if ( eAreaAI == AREAAI_DEFENSIVE)
									{
										bAreaValid = false;
									}
									else if( eAreaAI == AREAAI_OFFENSIVE )
									{
										bAreaValid = true;
									}
								}
							}
						}

						if ( (bAreaValid && (iEnemyPowerPercent < 140)) || (!bShareValid && (iEnemyPowerPercent < 110)) || (GET_TEAM((TeamTypes)iI).AI_getLowestVictoryCountdown() >= 0) )
						{
							if( gTeamLogLevel >= 1 )
							{
								logBBAI("      Team %d (%S) switching WARPLANS against team %d (%S) from PREPARING_TOTAL to TOTAL after %d turns with enemy power percent %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), iI, GET_PLAYER(GET_TEAM((TeamTypes)iI).getLeaderID()).getCivilizationDescription(0), AI_getWarPlanStateCounter((TeamTypes)iI), iEnemyPowerPercent );
							}
							AI_setWarPlan(((TeamTypes)iI), WARPLAN_TOTAL);
						}
						else if (AI_getWarPlanStateCounter((TeamTypes)iI) > ((20 * iAbandonTimeModifier) / 100))
						{
							if( gTeamLogLevel >= 1 )
							{
								logBBAI("      Team %d (%S) abandoning WARPLAN_TOTAL_PREPARING against team %d (%S) after %d turns with enemy power percent %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), iI, GET_PLAYER(GET_TEAM((TeamTypes)iI).getLeaderID()).getCivilizationDescription(0), AI_getWarPlanStateCounter((TeamTypes)iI), iEnemyPowerPercent );
							}
							AI_setWarPlan(((TeamTypes)iI), NO_WARPLAN);
						}
					}
					//CD Tweaks - opportunistic convert to dogpile to take advantage of a powerful target being attacked by someone else
					else if (bShareValid && iEnemyPowerPercent >= 140 && GET_TEAM((TeamTypes)iI).getAtWarCount(true) > 0)
					{
						iDogpilePower = getPower(true) - getEnemyPower();
						for (iJ = 0; iJ < MAX_CIV_TEAMS; iJ++)
						{
							if (GET_TEAM((TeamTypes)iJ).isAlive())
							{
								if (iJ != iI)
								{
									//only include teams that have attacked them recently
									if (atWar(((TeamTypes)iJ), ((TeamTypes)iI)) && 
										GET_TEAM((TeamTypes)iI).AI_getWarPlan((TeamTypes)iJ) == WARPLAN_ATTACKED_RECENT)
									{
										iDogpilePower += GET_TEAM((TeamTypes)iJ).getPower(false);
									}
								}
							}
						}
						if (iDogpilePower > (GET_TEAM((TeamTypes)iI).getDefensivePower() * 3) / 2)
						{
							AI_setWarPlan(((TeamTypes)iI), WARPLAN_DOGPILE);	//this will reset the timer, gives us ~15 turns to get ready
						}
					}
					//CD Tweaks End
				}
				else if (AI_getWarPlan((TeamTypes)iI) == WARPLAN_TOTAL)
				{
					if( !isAtWar((TeamTypes)iI) )
					{
						FAssert(canEventuallyDeclareWar((TeamTypes)iI));

						bool bActive = false;
						for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
						{
							if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
							{
								if( GET_PLAYER((PlayerTypes)iPlayer).AI_enemyTargetMissions((TeamTypes)iI) > 0 )
								{
									bActive = true;
									break;
								}
							}
						}

						if( !bActive )
						{
							if (AI_getWarPlanStateCounter((TeamTypes)iI) > ((25 * iAbandonTimeModifier) / (100)))
							{
								if( gTeamLogLevel >= 1 )
								{
									logBBAI("      Team %d (%S) abandoning WARPLAN_TOTAL against team %d (%S) after %d turns with enemy power percent %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), iI, GET_PLAYER(GET_TEAM((TeamTypes)iI).getLeaderID()).getCivilizationDescription(0), AI_getWarPlanStateCounter((TeamTypes)iI), iEnemyPowerPercent );
								}
								AI_setWarPlan(((TeamTypes)iI), NO_WARPLAN);
							}
						}
					}
				}
			}
		}
	}

	for (iI = 0; iI < MAX_CIV_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				GET_PLAYER((PlayerTypes)iI).AI_doPeace();
			}
		}
	}

	int iNumMembers = getNumMembers();
	int iHighUnitSpendingPercent = 0;
	int iLowUnitSpendingPercent = 0;

	for (iI = 0; iI < MAX_PLAYERS; iI++)
	{
		if (GET_PLAYER((PlayerTypes)iI).isAlive())
		{
			if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
			{
				int iUnitSpendingPercent = (GET_PLAYER((PlayerTypes)iI).calculateUnitCost() * 100) / std::max(1, GET_PLAYER((PlayerTypes)iI).calculatePreInflatedCosts());
				iHighUnitSpendingPercent += (std::max(0, iUnitSpendingPercent - 7) / 2);
				iLowUnitSpendingPercent += iUnitSpendingPercent;
			}			
		}
	}


	// if at war, check for making peace
	if (getAtWarCount(true) > 0) // XXX
	{
		if (GC.getGameINLINE().getSorenRandNum(AI_makePeaceRand(), "AI Make Peace") == 0)
		{
			for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
			{
				if (GET_TEAM((TeamTypes)iI).isAlive())
				{
					if (iI != getID())
					{
						if (!(GET_TEAM((TeamTypes)iI).isHuman()))
						{
							if (canContact((TeamTypes)iI))
							{
								FAssert(!(GET_TEAM((TeamTypes)iI).isMinorCiv()));

								if (isAtWar((TeamTypes)iI))
								{
									if (AI_isChosenWar((TeamTypes)iI))
									{
										if( AI_getAtWarCounter((TeamTypes)iI) > std::max(10, (14 * GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getVictoryDelayPercent())/100) )
										{
											// If nothing is happening in war
											if( AI_getWarSuccess((TeamTypes)iI) + GET_TEAM((TeamTypes)iI).AI_getWarSuccess(getID()) < 2*GC.getDefineINT("WAR_SUCCESS_ATTACKING") )
											{
												if( (GC.getGameINLINE().getSorenRandNum(8, "AI Make Peace 1") == 0) )
												{
													bool bValid = true;

													for( int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++ )
													{
														if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == getID() )
														{
															if( GET_PLAYER((PlayerTypes)iPlayer).AI_enemyTargetMissions((TeamTypes)iI) > 0 )
															{
																bValid = false;
																break;
															}
														}

														if( GET_PLAYER((PlayerTypes)iPlayer).getTeam() == iI )
														{
															MissionAITypes eMissionAI = MISSIONAI_ASSAULT;
															if( GET_PLAYER((PlayerTypes)iPlayer).AI_enemyTargetMissions(getID()) > 0 )
															{
																bValid = false;
																break;
															}
														}
													}

													if( bValid )
													{
														makePeace((TeamTypes)iI);

														if( gTeamLogLevel >= 1 )
														{
															logBBAI("  Team %d (%S) making peace due to time and no fighting", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0) );
														}

														break;
													}
												}
											}

											// Fought to a long draw
											if (AI_getAtWarCounter((TeamTypes)iI) > ((((AI_getWarPlan((TeamTypes)iI) == WARPLAN_TOTAL) ? 40 : 30) * GC.getGameSpeedInfo(GC.getGameINLINE().getGameSpeedType()).getVictoryDelayPercent())/100) )
											{
												int iOurValue = AI_endWarVal((TeamTypes)iI);
												int iTheirValue = GET_TEAM((TeamTypes)iI).AI_endWarVal(getID());
												//CD Tweaks - no need to make peace with small civs right away
												if (!GET_TEAM((TeamTypes)iI).canVassalRevolt(getID()) && iEnemyPowerPercent < 100)
												{
													iTheirValue *= 100;
													iTheirValue /= std::max(50, iEnemyPowerPercent);
												}
												//CD Tweaks end
												if ((iOurValue > (iTheirValue / 2)) && (iTheirValue > (iOurValue / 2)))
												{
													if( gTeamLogLevel >= 1 )
													{
														logBBAI("  Team %d (%S) making peace due to time and endWarVal %d vs their %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0) , iOurValue, iTheirValue );
													}
													makePeace((TeamTypes)iI);
													break;
												}
											}

											// All alone in what we thought was a dogpile
											if (AI_getWarPlan((TeamTypes)iI) == WARPLAN_DOGPILE)
											{
												if (GET_TEAM((TeamTypes)iI).getAtWarCount(true) == 1)
												{
													int iOurValue = AI_endWarVal((TeamTypes)iI);
													int iTheirValue = GET_TEAM((TeamTypes)iI).AI_endWarVal(getID());
													if ((iTheirValue > (iOurValue / 2)))
													{
														if( gTeamLogLevel >= 1 )
														{
															logBBAI("  Team %d (%S) making peace due to being only dog-piler left", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0) );
														}
														makePeace((TeamTypes)iI);
														break;
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
	}

	// if no war plans, consider starting one!
	if (getAnyWarPlanCount(true) == 0 || iEnemyPowerPercent < 45)
	{
		bool bAggressive = (GC.getGameINLINE().isOption(GAMEOPTION_AGGRESSIVE_AI));
		bool bVictoryPlanned = false;

		int iFinancialTroubleCount = 0;
		int iDaggerCount = 0;
		int iGetBetterUnitsCount = 0;
		for (iI = 0; iI < MAX_PLAYERS; iI++)
		{
			if (GET_PLAYER((PlayerTypes)iI).isAlive())
			{
				if (GET_PLAYER((PlayerTypes)iI).getTeam() == getID())
				{
					if ( GET_PLAYER((PlayerTypes)iI).AI_isDoStrategy(AI_STRATEGY_DAGGER)
						|| GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_CONQUEST3)
						|| GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategy(AI_VICTORY_DOMINATION4) )
					{
						iDaggerCount++;
						bAggressive = true;
					}
					else if (GET_PLAYER((PlayerTypes)iI).AI_isDoVictoryStrategyLevel3())
					{
						bVictoryPlanned = true;
					}

					if (GET_PLAYER((PlayerTypes)iI).AI_isDoStrategy(AI_STRATEGY_GET_BETTER_UNITS))
					{
						iGetBetterUnitsCount++;
					}

					if (GET_PLAYER((PlayerTypes)iI).AI_isFinancialTrouble())
					{
						iFinancialTroubleCount++;
					}
				}
			}
		}

		//CD Tweaks - if a human is winning and we have no victory plans of our own, switch to aggresive to try and catch up
		//only do this if unit spending is already high, or we are fairly advanced
		if (GET_TEAM(GC.getGameINLINE().getRankTeam(0)).isHuman() && !bVictoryPlanned && 
			(iLowUnitSpendingPercent >= 12 || getBestKnownTechScorePercent() >= 85))
		{
			bAggressive = true;
		}

		//CD Tweaks - do we want to plan pre-emptive strikes against teams we don't trust, not to gain ground but under the theory
		//that attack is the best form of defense. Only if we're going for culture victory or have launched our spaceship
		bool bPreemptiveStrike = false;
		if (AI_isAnyMemberDoVictoryStrategy(AI_VICTORY_CULTURE3) || getVictoryCountdown(GC.getGameINLINE().getSpaceVictory()) >= 0)
		{
			bool bPreemptiveStrike = true;
		}
		//CD Tweaks End

		// if random in this range is 0, we go to war of this type (so lower numbers are higher probablity)
		// average of everyone on our team
		int iTotalWarRand;
		int iLimitedWarRand;
		int iDogpileWarRand;
		AI_getWarRands( iTotalWarRand, iLimitedWarRand, iDogpileWarRand );

		int iTotalWarThreshold;
		int iLimitedWarThreshold;
		int iDogpileWarThreshold;
		AI_getWarThresholds( iTotalWarThreshold, iLimitedWarThreshold, iDogpileWarThreshold );

		// we oppose war if half the non-dagger teammates in financial trouble
		bool bFinancesOpposeWar = false;
		if ((iFinancialTroubleCount - iDaggerCount) >= std::max(1, getNumMembers() / 2 ))
		{
			// this can be overridden by by the pro-war booleans
			bFinancesOpposeWar = true;
		}

		// if agressive, we may start a war to get money
		bool bFinancesProTotalWar = false;
		bool bFinancesProLimitedWar = false;
		bool bFinancesProDogpileWar = false;
		if (iFinancialTroubleCount > 0)
		{
			// do we like all out wars?
			if (iDaggerCount > 0 || iTotalWarRand < 100)
			{
				bFinancesProTotalWar = true;
			}

			// do we like limited wars?
			if (iLimitedWarRand < 100)
			{
				bFinancesProLimitedWar = true;
			}

			// do we like dogpile wars?
			if (iDogpileWarRand < 100)
			{
				bFinancesProDogpileWar = true;
			}
		}
		bool bFinancialProWar = (bFinancesProTotalWar || bFinancesProLimitedWar || bFinancesProDogpileWar);

		// overall war check (quite frequently true)
		bool bMakeWarChecks = false;
		if ((iGetBetterUnitsCount - iDaggerCount) * 3 < iNumMembers * 2)
		{
			if (bFinancialProWar || !bFinancesOpposeWar)
			{
				// random overall war chance (at noble+ difficulties this is 100%)
				if (GC.getGameINLINE().getSorenRandNum(100, "AI Declare War 1") < GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAIDeclareWarProb())
				{
					bMakeWarChecks = true;
				}
			}
		}

		if (bMakeWarChecks)
		{
			iOurPower = getPower(true);

			if (bAggressive && (getAnyWarPlanCount(true) == 0))
			{
				iOurPower *= 4;
				iOurPower /= 3;
			}

			iOurPower *= (100 - iEnemyPowerPercent);
			iOurPower /= 100;

			//CD Tweaks - cache these for later use, and so we know ahead of time if we have any land targets
			bool canTargetTeams[MAX_CIV_TEAMS];
			bool landTargetTeams[MAX_CIV_TEAMS];
			bool neighborTargetTeams[MAX_CIV_TEAMS];
			bool bAnyGoodLandTargets = false;
			for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
			{
				canTargetTeams[iI] = false;  //until proven otherwise
				if (GET_TEAM((TeamTypes)iI).isAlive())
				{
					if (iI != getID())
					{
						if (isHasMet((TeamTypes)iI))
						{
							if (canEventuallyDeclareWar((TeamTypes)iI))
							{
								if( !GET_TEAM((TeamTypes)iI).isAVassal() || AI_isOkayVassalTarget((TeamTypes)iI) )
								{
									canTargetTeams[iI] = true;
									if (AI_isLandTarget((TeamTypes)iI, false))
									{
										landTargetTeams[iI] = true;
										neighborTargetTeams[iI] = (AI_isLandTarget((TeamTypes)iI, true));
										if (!bAnyGoodLandTargets)
										{
											bAnyGoodLandTargets = (GET_PLAYER(getLeaderID()).getCurrentEra() < (GC.getNumEraInfos() / 3) || 
												((AI_noWarAttitudeProb(AI_getAttitude((TeamTypes)iI)) <= 70) && (3 * iOurPower > 2 * GET_TEAM((TeamTypes)iI).getDefensivePower())));
										}
									}
									else
									{
										landTargetTeams[iI] = false;
										neighborTargetTeams[iI] = false;
									}
								}
							}
						}
					}
				}
			}
			//CD Tweaks End

			if ((bFinancesProTotalWar || !bFinancesOpposeWar) &&
				(GC.getGameINLINE().getSorenRandNum(iTotalWarRand, "AI Maximum War") <= iTotalWarThreshold))
			{
				iNoWarRoll = GC.getGameINLINE().getSorenRandNum(100, "AI No War");
				iNoWarRoll = range(iNoWarRoll + (bAggressive ? 10 : 0) + (bFinancesProTotalWar ? 10 : 0) - (20*iGetBetterUnitsCount)/iNumMembers, 0, 99);

				iBestValue = 0;
				eBestTeam = NO_TEAM;

				for (iPass = 0; iPass < 3; iPass++)
				{
					for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
					{
						if (canTargetTeams[iI])	//CD Tweaks - use cache
						{
							if (iNoWarRoll >= AI_noWarAttitudeProb(AI_getAttitude((TeamTypes)iI)))
							{
								int iDefensivePower = (GET_TEAM((TeamTypes)iI).getDefensivePower() * 2) / 3;

								if (iDefensivePower < ((iOurPower * ((iPass > 1) ? AI_maxWarDistantPowerRatio() : AI_maxWarNearbyPowerRatio())) / 100))
								{
									// XXX make sure they share an area....

									FAssertMsg(!(GET_TEAM((TeamTypes)iI).isBarbarian()), "Expected to not be declaring war on the barb civ");
									FAssertMsg(iI != getID(), "Expected not to be declaring war on self (DOH!)");

									//RevDCM - uncommented Fuyu changes
									//CD Tweaks
									bool bAttackOverSea = GET_TEAM((TeamTypes)iI).AI_isAnyMemberDoVictoryStrategyLevel4() || //this was the only time the AI would attack oversea on firstpass
										(bPreemptiveStrike && AI_getWarDistrustVal((TeamTypes)iI) > 0);
									if ((iPass > 1) || neighborTargetTeams[iI] || !bAnyGoodLandTargets || AI_isAnyCapitalAreaAlone() || bAttackOverSea)
									{
										if ((iPass > 0) || (AI_calculateAdjacentLandPlots((TeamTypes)iI) >= (bAnyGoodLandTargets ? ((getTotalLand() * AI_maxWarMinAdjacentLandPercent()) / 100) : AI_maxWarMinAdjacentLandPercent())) || bAttackOverSea)
											//CD Tweaks End
										{
											iValue = AI_startWarVal((TeamTypes)iI);

											if( iValue > 0 && gTeamLogLevel >= 2 )
											{
												logBBAI("      Team %d (%S) considering starting TOTAL warplan with team %d with value %d on pass %d with %d adjacent plots", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), iI, iValue, iPass, AI_calculateAdjacentLandPlots((TeamTypes)iI) );
											}

											if (iValue > iBestValue)
											{
												iBestValue = iValue;
												eBestTeam = ((TeamTypes)iI);
											}
										}
									}
								}
							}
						}
					}

					if (eBestTeam != NO_TEAM)
					{
						if( gTeamLogLevel >= 1 )
						{
							logBBAI("    Team %d (%S) starting TOTAL warplan preparations against team %d on pass %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), eBestTeam, iPass );
						}

						AI_setWarPlan(eBestTeam, (iDaggerCount > 0) ? WARPLAN_TOTAL : WARPLAN_PREPARING_TOTAL);
						break;
					}
				}
			}
			/************************************************************************************************/
			/* UNOFFICIAL_PATCH                       01/02/09                                jdog5000      */
			/*                                                                                              */
			/* Bugfix                                                                                       */
			/************************************************************************************************/
			else if ((bFinancesProLimitedWar || !bFinancesOpposeWar) &&
				(GC.getGameINLINE().getSorenRandNum(iLimitedWarRand, "AI Limited War") <= iLimitedWarThreshold))
				/************************************************************************************************/
				/* UNOFFICIAL_PATCH                        END                                                  */
				/************************************************************************************************/
			{
				iNoWarRoll = GC.getGameINLINE().getSorenRandNum(100, "AI No War") - 10;
				iNoWarRoll = range(iNoWarRoll + (bAggressive ? 10 : 0) + (bFinancesProLimitedWar ? 10 : 0), 0, 99);

				iBestValue = 0;
				eBestTeam = NO_TEAM;

				for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
				{
					if (canTargetTeams[iI])  //cd tweaks - use cache
					{
						if (iNoWarRoll >= AI_noWarAttitudeProb(AI_getAttitude((TeamTypes)iI)))
						{
							//RevDCM uncommented Fuyu changes. CD tweaks - use cache
							if (neighborTargetTeams[iI] || (AI_isAnyCapitalAreaAlone() && GET_TEAM((TeamTypes)iI).AI_isAnyCapitalAreaAlone()))
							{
								if (GET_TEAM((TeamTypes)iI).getDefensivePower() < ((iOurPower * AI_limitedWarPowerRatio()) / 100))
								{
									iValue = AI_startWarVal((TeamTypes)iI);

									if( iValue > 0 && gTeamLogLevel >= 2 )
									{
										logBBAI("      Team %d (%S) considering starting LIMITED warplan with team %d with value %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), iI, iValue );
									}

									if (iValue > iBestValue)
									{
										FAssert(!AI_shareWar((TeamTypes)iI));
										iBestValue = iValue;
										eBestTeam = ((TeamTypes)iI);
									}
								}
							}
						}
					}
				}

				if (eBestTeam != NO_TEAM)
				{
					if( gTeamLogLevel >= 1 )
					{
						logBBAI("    Team %d (%S) starting LIMITED warplan preparations against team %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), eBestTeam );
					}

					AI_setWarPlan(eBestTeam, (iDaggerCount > 0) ? WARPLAN_LIMITED : WARPLAN_PREPARING_LIMITED);
				}
			}
			else if ((bFinancesProDogpileWar || !bFinancesOpposeWar) &&
				(GC.getGameINLINE().getSorenRandNum(iDogpileWarRand, "AI Dogpile War") <= iDogpileWarThreshold))
			{
				iNoWarRoll = GC.getGameINLINE().getSorenRandNum(100, "AI No War") - 20;
				iNoWarRoll = range(iNoWarRoll + (bAggressive ? 10 : 0) + (bFinancesProDogpileWar ? 10 : 0), 0, 99);

				iBestValue = 0;
				eBestTeam = NO_TEAM;

				//CD Tweaks - smarter dogpile logic
				//vanilla logic, is a land target or a civ who is about to win and is fighting someone else
				bool dogpileTargetTeams[MAX_CIV_TEAMS];	//doesn't consider attitude or power, that comes later
				for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
				{
					dogpileTargetTeams[iI] = false;	//until proven otherwise
					if (canTargetTeams[iI] && GET_TEAM((TeamTypes)iI).getAtWarCount(true) > 0)
					{
						if (landTargetTeams[iI] || GET_TEAM((TeamTypes)iI).AI_isAnyMemberDoVictoryStrategyLevel4() || 
							(bPreemptiveStrike && AI_getWarDistrustVal((TeamTypes)iI) > 0) || (!bAnyGoodLandTargets && AI_teamCloseness((TeamTypes)iI) > 0))
						{
							dogpileTargetTeams[iI] = true;	
						}
						else
						{
							for (iJ = 0; iJ < MAX_CIV_TEAMS; iJ++)
							{
								//if they're at war with someone near us, and have cities in our area, we'll also consider getting involved on the other side
								if (atWar((TeamTypes)iI,(TeamTypes)iJ) && (landTargetTeams[iJ] || (!bAnyGoodLandTargets && AI_teamCloseness((TeamTypes)iJ) > 0)))
								{
									dogpileTargetTeams[iI] = AI_hasCitiesInPrimaryArea((TeamTypes)iI);
								}
							}
						}
					}
				}
				//CD Tweaks End

				for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
				{
					if (dogpileTargetTeams[iI])
					{
						if (iNoWarRoll >= AI_noWarAttitudeProb(AI_getAttitude((TeamTypes)iI)))
						{
							iDogpilePower = iOurPower;
							//CD tweaks - consider ourselves less powerfull for distant targets though
							if (!landTargetTeams[iI])
							{
								iDogpilePower *= 100 + AI_maxWarDistantPowerRatio();
								iDogpilePower /= 200;
							}
							//CD Tweaks end

							for (iJ = 0; iJ < MAX_CIV_TEAMS; iJ++)
							{
								if (GET_TEAM((TeamTypes)iJ).isAlive())
								{
									if (iJ != iI)
									{
										if (atWar(((TeamTypes)iJ), ((TeamTypes)iI)))
										{
											iDogpilePower += GET_TEAM((TeamTypes)iJ).getPower(false);
										}
									}
								}
							}

							FAssert(GET_TEAM((TeamTypes)iI).getPower(true) == GET_TEAM((TeamTypes)iI).getDefensivePower() || GET_TEAM((TeamTypes)iI).isAVassal());

							//CD Tweaks - if we are up against our worst enemy, we don't need much excuse to dogpile
							//if (((GET_TEAM((TeamTypes)iI).getDefensivePower() * 3) / 2) < iDogpilePower)
							int iTheirPower = 0;
							if (AI_getWorstEnemy() == (TeamTypes)iI)
							{
								iTheirPower = GET_TEAM((TeamTypes)iI).getDefensivePower();
							}
							else
							{
								iTheirPower = (GET_TEAM((TeamTypes)iI).getDefensivePower() * 3) / 2;
							}
							if (iTheirPower < iDogpilePower)
								//CD Tweaks end
							{
								iValue = AI_startWarVal((TeamTypes)iI);

								if( iValue > 0 && gTeamLogLevel >= 2 )
								{
									logBBAI("      Team %d (%S) considering starting DOGPILE warplan with team %d with value %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), iI, iValue );
								}

								if (iValue > iBestValue)
								{
									FAssert(!AI_shareWar((TeamTypes)iI));
									iBestValue = iValue;
									eBestTeam = ((TeamTypes)iI);
								}
							}
						}
					}
				}

				if (eBestTeam != NO_TEAM)
				{
					if( gTeamLogLevel >= 1 )
					{
						logBBAI("  Team %d (%S) starting DOGPILE warplan preparations with team %d", getID(), GET_PLAYER(getLeaderID()).getCivilizationDescription(0), eBestTeam );
					}
					AI_setWarPlan(eBestTeam, WARPLAN_DOGPILE);
				}
			}
		}
	}
}
/************************************************************************************************/
/* BETTER_BTS_AI_MOD                       END                                                  */
/************************************************************************************************/

//CD Tweaks - consider going to war against one of the bassed in teams to gain a strategic bonus of the passed in value
void CvTeamAI::AI_warForBonus(bool* pTargetTeams, int iBonusValue)
{
	int iI;
	//if we have no active wars and no planned wars against strong enemies
	if (AI_getEnemyPowerPercent() <= 45 && getAtWarCount(true) == 0)
	{
		int iTotalWarRand;
		int iLimitedWarRand;
		int iDogpileWarRand;
		AI_getWarRands( iTotalWarRand, iLimitedWarRand, iDogpileWarRand );

		int iTotalWarThreshold;
		int iLimitedWarThreshold;
		int iDogpileWarThreshold;
		AI_getWarThresholds( iTotalWarThreshold, iLimitedWarThreshold, iDogpileWarThreshold );

		bool bWarAlready;
		for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
		{
			if (AI_getWarPlan((TeamTypes)iI) != NO_WARPLAN)
			{
				if (pTargetTeams[iI])
				{
					//don't start a new war if we're already planning a war against any enemy that has this bonus
					bWarAlready = true;
					break;
				}
				else if (!AI_hasCitiesInPrimaryArea((TeamTypes)iI))
				{
					//don't start a new war when already planning an assault
					bWarAlready = true;
					break;
				}
			}
		}
		if (!bWarAlready)
		{
			iTotalWarThreshold += iBonusValue - 20;	//should be more than 20 before this function is called
			//the checks are more likely, and we only consider war against teams that have this bonus, but we don't declare war
			//against anyone we wouldn't normally, and we use the same function to determine the relative value of the war
			if (GC.getGameINLINE().getSorenRandNum(iTotalWarRand, "AI Maximum War") <= iTotalWarThreshold)
			{
				int iValue;
				int iBestValue = 0;
				TeamTypes eBestTeam = NO_TEAM;
				WarPlanTypes eBestWarPlan = NO_WARPLAN;
				int iNoWarRoll = GC.getGameINLINE().getSorenRandNum(100, "AI No War") + iBonusValue;
				for (iI = 0; iI < MAX_CIV_TEAMS; iI++)
				{
					if (GET_TEAM((TeamTypes)iI).isAlive())
					{
						if (iI != getID())
						{
							if (isHasMet((TeamTypes)iI) && pTargetTeams [iI])
							{
								if (canEventuallyDeclareWar((TeamTypes)iI))
								{
									if( !GET_TEAM((TeamTypes)iI).isAVassal() || AI_isOkayVassalTarget((TeamTypes)iI) )
									{
										if(iNoWarRoll >= AI_noWarAttitudeProb(AI_getAttitude((TeamTypes)iI)))
										{
											//don't declare war if we could just trade for it
											bool bCanTrade = false;
											for( int iJ = 0; iJ < MAX_CIV_PLAYERS; iJ++ )
											{
												if( getID() == GET_PLAYER((PlayerTypes)iJ).getTeam() && GET_PLAYER((PlayerTypes)iJ).isAlive() )
												{
													if (AI_getAttitude((TeamTypes)iI) > GC.getLeaderHeadInfo(GET_PLAYER((PlayerTypes)iJ).getPersonalityType()).getStrategicBonusRefuseAttitudeThreshold())
													{
														bCanTrade = true;
														break;
													}
												}
											}
											if (!bCanTrade)
											{
												if (GET_TEAM((TeamTypes)iI).getDefensivePower() < getPower(true))
												{
													iValue = AI_startWarVal((TeamTypes)iI);
													if (iValue > iBestValue)
													{
														iBestValue = iValue;
														eBestTeam = ((TeamTypes)iI);
														eBestWarPlan = WARPLAN_PREPARING_TOTAL;
													}
												}
												else if ((GET_TEAM((TeamTypes)iI).getDefensivePower() * 3) / 2 < (getPower(true) + GET_TEAM((TeamTypes)iI).getEnemyPower()))
												{
													iValue = AI_startWarVal((TeamTypes)iI);
													if (iValue > iBestValue)
													{
														iBestValue = iValue;
														eBestTeam = ((TeamTypes)iI);
														eBestWarPlan = WARPLAN_DOGPILE;
													}
												}													
											}
										}
									}
								}
							}
						}
					}
				}
				if (eBestTeam != NO_TEAM && eBestWarPlan != NO_WARPLAN)
				{
					AI_setWarPlan(eBestTeam, eBestWarPlan);
				}
			}
		}
	}
}
//CD Tweaks End

//returns true if war is veto'd by rolls.
bool CvTeamAI::AI_performNoWarRolls(TeamTypes eTeam)
{

	if (GC.getGameINLINE().getSorenRandNum(100, "AI Declare War 1") > GC.getHandicapInfo(GC.getGameINLINE().getHandicapType()).getAIDeclareWarProb())
	{
		return true;
	}

	if (GC.getGameINLINE().getSorenRandNum(100, "AI No War") <= AI_noWarAttitudeProb(AI_getAttitude(eTeam)))
	{
		return true;		
	}



	return false;	
}

int CvTeamAI::AI_getAttitudeWeight(TeamTypes eTeam)
{
	int iAttitudeWeight = 0;
	switch (AI_getAttitude(eTeam))
	{
	case ATTITUDE_FURIOUS:
		iAttitudeWeight = -100;
		break;
	case ATTITUDE_ANNOYED:
		iAttitudeWeight = -40;
		break;
	case ATTITUDE_CAUTIOUS:
		iAttitudeWeight = -5;
		break;
	case ATTITUDE_PLEASED:
		iAttitudeWeight = 50;
		break;
	case ATTITUDE_FRIENDLY:
		iAttitudeWeight = 100;			
		break;
	}

	return iAttitudeWeight;
}

int CvTeamAI::AI_getLowestVictoryCountdown() const
{
	int iBestVictoryCountdown = MAX_INT;
	for (int iVictory = 0; iVictory < GC.getNumVictoryInfos(); iVictory++)
	{
		int iCountdown = getVictoryCountdown((VictoryTypes)iVictory);
		if (iCountdown > 0)
		{
			iBestVictoryCountdown = std::min(iBestVictoryCountdown, iCountdown);
		}
	}
	if (MAX_INT == iBestVictoryCountdown)
	{
		iBestVictoryCountdown = -1;
	}
	return iBestVictoryCountdown;	
}

int CvTeamAI::AI_getTechMonopolyValue(TechTypes eTech, TeamTypes eTeam) const
{
	int iValue = 0;
	int iI;

	bool bWarPlan = (getAnyWarPlanCount(eTeam) > 0);

	for (iI = 0; iI < GC.getNumUnitClassInfos(); iI++)
	{
		UnitTypes eLoopUnit = ((UnitTypes)GC.getUnitClassInfo((UnitClassTypes)iI).getDefaultUnitIndex());

		if (eLoopUnit != NO_UNIT)
		{
			if (isTechRequiredForUnit((eTech), eLoopUnit))
			{
				if (isWorldUnitClass((UnitClassTypes)iI))
				{
					iValue += 50;
				}

				//CD Tweaks - masters share military techs with vassals when at war and tech brockering is off
				if (getAtWarCount(true) == 0 || !GC.getGameINLINE().isOption(GAMEOPTION_NO_TECH_BROKERING) || !GET_TEAM(eTeam).isVassal(getID()))
					//CD Tweaks End
				{
					if (GC.getUnitInfo(eLoopUnit).getPrereqAndTech() == eTech)
					{
						int iNavalValue = 0;

						int iCombatRatio = (GC.getUnitInfo(eLoopUnit).getCombat() * 100) / std::max(1, GC.getGameINLINE().getBestLandUnitCombat());
						if (iCombatRatio > 50)
						{
							iValue += ((bWarPlan ? 100 : 50) * (iCombatRatio - 40)) / 50;
						}

						switch (GC.getUnitInfo(eLoopUnit).getDefaultUnitAIType())
						{
						case UNITAI_UNKNOWN:
						case UNITAI_ANIMAL:
						case UNITAI_SETTLE:
						case UNITAI_WORKER:
							break;

						case UNITAI_ATTACK:
						case UNITAI_ATTACK_CITY:
						case UNITAI_COLLATERAL:
							iValue += bWarPlan ? 50 : 20;
							break;

						case UNITAI_PILLAGE:
						case UNITAI_RESERVE:
						case UNITAI_COUNTER:
						case UNITAI_PARADROP:
						case UNITAI_CITY_DEFENSE:
						case UNITAI_CITY_COUNTER:
						case UNITAI_CITY_SPECIAL:
							iValue += bWarPlan ? 40 : 15;
							break;


						case UNITAI_EXPLORE:
						case UNITAI_MISSIONARY:
							break;

						case UNITAI_PROPHET:
						case UNITAI_ARTIST:
						case UNITAI_SCIENTIST:
						case UNITAI_GENERAL:
						case UNITAI_MERCHANT:
						case UNITAI_ENGINEER:
							break;

						case UNITAI_SPY:
							break;

						case UNITAI_ICBM:
							iValue += bWarPlan ? 80 : 40;
							break;

						case UNITAI_WORKER_SEA:
							break;

						case UNITAI_ATTACK_SEA:
							iNavalValue += 50;
							break;

						case UNITAI_RESERVE_SEA:
						case UNITAI_ESCORT_SEA:
							iNavalValue += 30;
							break;

						case UNITAI_EXPLORE_SEA:
							iValue += GC.getGame().circumnavigationAvailable() ? 100 : 0;
							break;

						case UNITAI_ASSAULT_SEA:
							iNavalValue += 60;
							break;

						case UNITAI_SETTLER_SEA:
						case UNITAI_MISSIONARY_SEA:
						case UNITAI_SPY_SEA:
							break;

						case UNITAI_CARRIER_SEA:
						case UNITAI_MISSILE_CARRIER_SEA:
							iNavalValue += 40;
							break;

						case UNITAI_PIRATE_SEA:
							iNavalValue += 20;
							break;

						case UNITAI_ATTACK_AIR:
						case UNITAI_DEFENSE_AIR:
							iValue += bWarPlan ? 60 : 30;
							break;

						case UNITAI_CARRIER_AIR:
							iNavalValue += 40;
							break;

						case UNITAI_MISSILE_AIR:
							iValue += bWarPlan ? 40 : 20;
							break;

						default:
							FAssert(false);
							break;
						}

						if (iNavalValue > 0)
						{
							if (AI_isAnyCapitalAreaAlone())
							{
								iValue += iNavalValue / 2;
							}
							if (bWarPlan && !AI_isLandTarget(eTeam))
							{
								iValue += iNavalValue / 2;
							}
						}
					}
				}
			}
		}
	}

	int iCorpFoundCount = 0;	//CD Tweaks
	for (iI = 0; iI < GC.getNumBuildingInfos(); iI++)
	{
		if (isTechRequiredForBuilding(eTech, ((BuildingTypes)iI)))
		{
			CvBuildingInfo& kLoopBuilding = GC.getBuildingInfo((BuildingTypes)iI);
			if (kLoopBuilding.getReligionType() == NO_RELIGION)
			{
				iValue += 30;
			}
			if (isWorldWonderClass((BuildingClassTypes)kLoopBuilding.getBuildingClassType()))
			{
				if (!(GC.getGameINLINE().isBuildingClassMaxedOut((BuildingClassTypes)kLoopBuilding.getBuildingClassType())))
				{
					//CD Tweaks - if the tech enables multiple corporations to be founded, only count the first
					if (kLoopBuilding.getFoundsCorporation() == NO_CORPORATION)
					{
						//for wonders - check if he can research all required techs or someone has all techs to build the wonder, if not don't worry so much
						bool bOtherTeamNearAllTechs = true;
						TechTypes ePreReqTech = (TechTypes)kLoopBuilding.getPrereqAndTech();
						if (ePreReqTech != NO_TECH && eTech != ePreReqTech && GC.getGameINLINE().countKnownTechNumTeams(ePreReqTech) == 0 && !GET_PLAYER(GET_TEAM(eTeam).getLeaderID()).canResearch(ePreReqTech))
						{
							bOtherTeamNearAllTechs = false;
						}
						else
						{
							int iJ = 0;
							for (iJ = 0; iJ < GC.getNUM_BUILDING_AND_TECH_PREREQS(); iJ++)
							{
								ePreReqTech =  (TechTypes)kLoopBuilding.getPrereqAndTechs(iJ);
								if (ePreReqTech != NO_TECH && eTech != ePreReqTech && GC.getGameINLINE().countKnownTechNumTeams(ePreReqTech) == 0 && !GET_PLAYER(GET_TEAM(eTeam).getLeaderID()).canResearch(ePreReqTech))
								{
									bOtherTeamNearAllTechs = false;
									break;
								}
							}
						}
						if (bOtherTeamNearAllTechs)
						{
							iValue += 50;
						}
						else
						{
							iValue += 30;
						}
					}
					else if (iCorpFoundCount == 0)
					{
						if (kLoopBuilding.getFoundsCorporation() != NO_CORPORATION)
						{
							iCorpFoundCount++;
						}
						iValue += 50;
					}
					//CD Tweaks End
				}
			}
		}
	}

	for (iI = 0; iI < GC.getNumProjectInfos(); iI++)
	{
		if (GC.getProjectInfo((ProjectTypes)iI).getTechPrereq() == eTech)
		{
			if (isWorldProject((ProjectTypes)iI))
			{
				if (!(GC.getGameINLINE().isProjectMaxedOut((ProjectTypes)iI)))
				{
					iValue += 100;
				}
			}
			else
			{
				iValue += 50;
			}
		}
	}

	//CD Tweaks - consider civics
	for (int iJ = 0; iJ < GC.getNumCivicInfos(); iJ++)
	{
		if (GC.getCivicInfo((CivicTypes)iJ).getTechPrereq() == eTech)
		{
			iValue += 30;
			break;	//I've decided not to count multiple civics, 30% increase if any civics are enabled
		}
	}
	//CD Tweaks

	return iValue;
}

bool CvTeamAI::AI_isWaterAreaRelevant(CvArea* pArea)
{
	int iTeamCities = 0;
	int iOtherTeamCities = 0;

	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                      01/15/09                                jdog5000      */
	/*                                                                                              */
	/* City AI                                                                                      */
	/************************************************************************************************/
	CvArea* pBiggestArea = GC.getMap().findBiggestArea(true);
	if (pBiggestArea == pArea)
	{
		return true;
	}

	// An area is deemed relevant if it has at least 2 cities of our and different teams.
	// Also count lakes which are connected to ocean by a bridge city
	for (int iPlayer = 0; iPlayer < MAX_CIV_PLAYERS; iPlayer++)
	{
		CvPlayerAI& kPlayer = GET_PLAYER((PlayerTypes)iPlayer);

		if ((iTeamCities < 2 && (kPlayer.getTeam() == getID())) || (iOtherTeamCities < 2 && (kPlayer.getTeam() != getID())))
		{
			int iLoop;
			CvCity* pLoopCity;

			for (pLoopCity = kPlayer.firstCity(&iLoop); pLoopCity != NULL; pLoopCity = kPlayer.nextCity(&iLoop))
			{
				if (pLoopCity->plot()->isAdjacentToArea(pArea->getID()))
				{
					if (kPlayer.getTeam() == getID())
					{
						iTeamCities++;

						if( pLoopCity->waterArea() == pBiggestArea )
						{
							return true;
						}
					}
					else
					{
						iOtherTeamCities++;
						if (iOtherTeamCities >= 2)
						{
							break;
						}
					}
				}				
			}
		}
		if (iTeamCities >= 2 && iOtherTeamCities >= 2)
		{
			return true;
		}
	}
	/************************************************************************************************/
	/* BETTER_BTS_AI_MOD                       END                                                  */
	/************************************************************************************************/	

	return false;
}

// Private Functions...
